#include "Controllable.h"

extern HANDLE semapho;
_STL::vector<_STL::string> tori;
extern volatile bool terminated;
extern VOYA_SETTING *vs;

namespace {
    const char NEW_LINE[3] = {'\r', '\n', '\0'};
    _STL::string info;
    _STL::vector<_STL::string> voya_error;
}

bool Controllable::closing() {
    bool already_stopped;

    if( stop() ) {
        already_stopped = true;
        shellExcute(VYBM_EXE, "-b");
    } else {
        already_stopped = false;
    }

    return already_stopped;
}

bool Controllable::movingInside(bool showWhere) {
    const int BORDER = ::GetSystemMetrics(SM_CYSIZEFRAME);
    const int TITLE_BAR_HEIGHT = ::GetSystemMetrics(SM_CYCAPTION);
    const int MENU_HEIGHT = ::GetSystemMetrics(SM_CYMENU);
    const int HEAD = BORDER + TITLE_BAR_HEIGHT + MENU_HEIGHT + WHERE_LABEL_HEIGHT;
    POINT pos;
    RECT windowRect;

    ::GetWindowRect(hwnd, &windowRect);
    ::GetCursorPos(&pos);

    bool through = false;
    const int UPPER_LEFT_X = windowRect.left;
    const int UPPER_RIGHT_X = windowRect.right;
    const int UPPER_LEFT_Y = windowRect.top + HEAD;
    const int LOWER_LEFT_Y = windowRect.bottom - STATUS_BAR_HEIGHT - BORDER;
    if(UPPER_LEFT_X < pos.x && pos.x < UPPER_RIGHT_X) {
        if(UPPER_LEFT_Y < pos.y && pos.y < LOWER_LEFT_Y) {
            through = true;
        }
    }

    if(showWhere) {
        if(through) {
            wsprintf(buffer, "%d : %d ( inside ) ", pos.x, pos.y); // char buffer[BUFFER_SIZE];
        } else {
            wsprintf(buffer, "%d : %d ( outside ) ", pos.x, pos.y); // char buffer[BUFFER_SIZE];
        }

        if(whereLabel == TITLE_BAR_USED_AS_LABEL) {
            ::SetWindowText(hwnd, buffer);
        } else {
            ::SetWindowText(whereLabel, buffer);
        }
    }

    if(inside) {
        ;
    } else {
        if(through) {
            stop();
        }
    }

    return through;
}

bool removeVoyaErrorFiles() {
    char *p;
    char buffer[BUFFER_SIZE];
    char c[SMALL_BUFFER_SIZE];
    unsigned long length;

    HANDLE text = ::CreateFile(REMOVE_ERROR_TEXT, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
    if(text == INVALID_HANDLE_VALUE) {
        return false;
    } else {
        ::ReadFile(text, buffer, sizeof(buffer), &length, NULL);
        p = &buffer[0];
        while(*p != '\0') {
            p = getLine(p, c);
            if(p == NULL) {
                break;
            }

            ::DeleteFile(c);
        }

        ::CloseHandle(text);
    }

    return true;
}

bool shellExcute(const char *p, const char *q) {
    using namespace _STL;

    bool result;
    char fileName[SMALL_BUFFER_SIZE];
    char parameters[SMALL_BUFFER_SIZE];
    lstrcpy(fileName, p);
    lstrcpy(parameters, q);

    SHELLEXECUTEINFO execinfo;
    ::ZeroMemory(&execinfo, sizeof(execinfo) );
    execinfo.cbSize = sizeof(execinfo);
    execinfo.fMask = SEE_MASK_NOCLOSEPROCESS;
    execinfo.lpFile = fileName;
    execinfo.lpParameters = parameters;
    execinfo.lpVerb = "open";
    execinfo.nShow = SW_SHOWDEFAULT;
    int error_value = ::ShellExecuteEx(&execinfo);
    if(error_value > 32) {
        result = true;
    } else {
        stringstream ss;
        ss << "error_value = " << error_value;
        info = ss.str();
        result = false;
    }

    return result;
}

bool Thread::stop() {
    bool already_stopped;

    if( ::WaitForSingleObject(semapho, WAITED_TOO_LONG) == WAIT_TIMEOUT ) {
        already_stopped = false;
    } else {
        if(terminated) {
            already_stopped = true;
        } else {
            already_stopped = false;
            terminated = true;
            ::Sleep(terminating_period);
        }

        ::ReleaseSemaphore(semapho, 1, NULL);
    }

    return already_stopped;
}

bool Thread::stop(const char *message) {
    using namespace _STL;

    bool already_stopped;

    if( ::WaitForSingleObject(semapho, WAITED_TOO_LONG) == WAIT_TIMEOUT ) {
        already_stopped = false;
    } else {
        if(terminated) {
            already_stopped = true;
        } else {
            already_stopped = false;
            terminated = true;
            ::Sleep(terminating_period);

// temporary file name
            char buffer[SMALL_BUFFER_SIZE];
            char voyaErrorName[ MAX_PATH + 1 ];
            ::GetTempFileName(".", VOY, 0, voyaErrorName);
            const string VOYA_ERROR(voyaErrorName);
            voya_error.push_back(VOYA_ERROR);

            lstrcpy(buffer, message);
            lstrcat(buffer, " stopped the thread.");
            lstrcat(buffer, NEW_LINE);

            HANDLE who = ::CreateFile(voyaErrorName, GENERIC_READ | GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
            unsigned long length;
            ::WriteFile(who, buffer, lstrlen(buffer), &length, NULL);
            ::CloseHandle(who);
        }

        ::ReleaseSemaphore(semapho, 1, NULL);
    }

    return already_stopped;
}

FILETIME Controllable::readVoyaCtrlText() {
    char *p;
    char c[SMALL_BUFFER_SIZE];
    const FILETIME invalid = {0xFFFFFFFF, 0xFFFFFFFF};
    FILETIME untouched;
    int line = INSIDE_LINE;
    int setting[VOYA_MODES_LIST_SIZE];
    setting[NOT_AVAILABLE] = 0;
    unsigned long length;

    HANDLE text = ::CreateFile(VOYA_CTRL_TEXT, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
    if(text == INVALID_HANDLE_VALUE) {
        return invalid;
    } else {
        ::ReadFile(text, buffer, sizeof(buffer), &length, NULL);  // char buffer[BUFFER_SIZE];
        p = &buffer[0];
        while(*p != '\0') {
            p = getLine(p, c);
            if(p == NULL) {
                break;
            }

            setting[line] = c[0] - '0';
            if(++line > LEFTMOST_SALT_LINE) {
                break;
            }
        }

        inside = setting[INSIDE_LINE];
        messages = setting[MESSAGES_LINE];
        priority = setting[PRIORITY_LINE];

// Getting the file time of VOYA_CTRL_TEXT
        ::GetFileTime(text, NULL, NULL, &untouched);
        ::CloseHandle(text);
    }

    switch(priority) {
    case IDLE:
        ::SetPriorityClass(handle, IDLE_PRIORITY_CLASS);
        ::SetThreadPriority(handle, THREAD_PRIORITY_IDLE);
        break;
    case LOWEST_PRIORITY:
        ::SetPriorityClass(handle, NORMAL_PRIORITY_CLASS);
        ::SetThreadPriority(handle, THREAD_PRIORITY_LOWEST);
        break;
    case LOWER_PRIORITY:
        ::SetPriorityClass(handle, NORMAL_PRIORITY_CLASS);
        ::SetThreadPriority(handle, THREAD_PRIORITY_BELOW_NORMAL);
        break;
    case NORMAL_PRIORITY:
        ::SetPriorityClass(handle, NORMAL_PRIORITY_CLASS);
        ::SetThreadPriority(handle, THREAD_PRIORITY_NORMAL);
        break;
    case HIGHER_PRIORITY:
        ::SetPriorityClass(handle, NORMAL_PRIORITY_CLASS);
        ::SetThreadPriority(handle, THREAD_PRIORITY_ABOVE_NORMAL);
        break;
    case HIGHEST_PRIORITY:
        ::SetPriorityClass(handle, NORMAL_PRIORITY_CLASS);
        ::SetThreadPriority(handle, THREAD_PRIORITY_HIGHEST);
        break;
    case SYSTEM_PRIORITY:
        ::SetPriorityClass(handle, REALTIME_PRIORITY_CLASS);
        ::SetThreadPriority(handle, THREAD_PRIORITY_TIME_CRITICAL);
        break;
    }

    return untouched;
}

void Controllable::closeVoyaError() {
    if(voyaErrorOpen) {
        ::CloseHandle(voyaErrorHandle);
        voyaErrorOpen = false;
    }

}

void Controllable::makeBatchFile() {
    using namespace _STL;

    lstrcpy(buffer, "COPY /B ");
    lstrcat(buffer, vs->log_file_name);
    string s;
    for(vector<string>::iterator i = tori.begin(); i != tori.end(); ++i) {
        lstrcat(buffer, "+");

        s = *i;
        lstrcat(buffer, s.c_str() );
    }

    lstrcat(buffer, " ");

// temporary file name
    ::GetTempFileName(".", VOY, 0, voyaErrorName);
    const string VOYA_ERROR(voyaErrorName);
    voya_error.push_back(VOYA_ERROR);

    lstrcat(buffer, & voyaErrorName[2]);
    lstrcat(buffer, NEW_LINE);
    lstrcat(buffer, "DEL ");
    lstrcat(buffer, vs->log_file_name);
    lstrcat(buffer, NEW_LINE);
    lstrcat(buffer, "REN "); // rename
    lstrcat(buffer, & voyaErrorName[2]);
    lstrcat(buffer, " ");
    lstrcat(buffer, vs->log_file_name);
    lstrcat(buffer, NEW_LINE);

    if(messages) {
        vector<string>::reverse_iterator i = voya_error.rbegin();
        ++i; // voyaErrorName has been renamed.
        while( i != voya_error.rend() ) {
            lstrcat(buffer, "DEL ");

            s = *i;
            lstrcat(buffer, s.c_str() );
            lstrcat(buffer, NEW_LINE);
            ++i;
        }

    }

    HANDLE batch = ::CreateFile(VOYA_LOG_BATCH, GENERIC_READ | GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
    unsigned long length;
    ::WriteFile(batch, buffer, lstrlen(buffer), &length, NULL);
    ::CloseHandle(batch);
}

void Controllable::openVoyaError() {
    using namespace _STL;

    ::GetTempFileName(".", VOY, 0, voyaErrorName);
    const string VOYA_ERROR(voyaErrorName);
    voya_error.push_back(VOYA_ERROR);

    voyaErrorHandle = ::CreateFile(voyaErrorName, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
    if(voyaErrorHandle == INVALID_HANDLE_VALUE) {
        voyaErrorOpen = false;
    } else {
        voyaErrorOpen = true;
        tori.push_back(VOYA_ERROR);
    }
}

void Controllable::voyaFlush(const char *message) {
    if(voyaErrorOpen) {
        unsigned long length;
        lstrcpy(buffer, message); // char buffer[BUFFER_SIZE];
        lstrcat(buffer, NEW_LINE);
        ::WriteFile(voyaErrorHandle, buffer, lstrlen(buffer), &length, NULL);
    }
}

void Controllable::voyaFlush(double sum) {
    if(voyaErrorOpen) {
        unsigned long length;
        int s = (int)sum;
        wsprintf(buffer, "%d", s); // char buffer[BUFFER_SIZE];
        lstrcat(buffer, NEW_LINE);
        ::WriteFile(voyaErrorHandle, buffer, lstrlen(buffer), &length, NULL);
    }
}

void Controllable::voyaFlush(int count) {
    if(voyaErrorOpen) {
        unsigned long length;
        wsprintf(buffer, "%d", count); // char buffer[BUFFER_SIZE];
        lstrcat(buffer, NEW_LINE);
        ::WriteFile(voyaErrorHandle, buffer, lstrlen(buffer), &length, NULL);
    }
}

void Controllable::voyaFlush(_STL::string & info) {
    if(voyaErrorOpen) {
        unsigned long length;
        lstrcpy(buffer, info.c_str() ); // char buffer[BUFFER_SIZE];
        lstrcat(buffer, NEW_LINE);
        ::WriteFile(voyaErrorHandle, buffer, lstrlen(buffer), &length, NULL);
    }
}

void Controllable::voyaMessage(const char *message) {
    if(voyaErrorOpen) {
        unsigned long length;
        lstrcpy(buffer, message); // char buffer[BUFFER_SIZE];
        ::WriteFile(voyaErrorHandle, buffer, lstrlen(buffer), &length, NULL);
    }
}

void Controllable::voyaMessage(_STL::string & info) {
    if(voyaErrorOpen) {
        unsigned long length;
        lstrcpy(buffer, info.c_str() ); // char buffer[BUFFER_SIZE];
        ::WriteFile(voyaErrorHandle, buffer, lstrlen(buffer), &length, NULL);
    }
}

void createVoyaLog() {
    char buffer[SMALL_BUFFER_SIZE];
    HANDLE voya_log = ::CreateFile(VOYA_LOG, GENERIC_READ | GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
    unsigned long length;
    lstrcpy(buffer, VOYA_LOG);
    lstrcat(buffer, NEW_LINE);
    ::WriteFile(voya_log, buffer, lstrlen(buffer), &length, NULL);
    ::CloseHandle(voya_log);
}

void createVoyaLog(char *message) {
    char buffer[SMALL_BUFFER_SIZE];
    HANDLE voya_log = ::CreateFile(VOYA_LOG, GENERIC_READ | GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
    unsigned long length;
    lstrcpy(buffer, message);
    lstrcat(buffer, NEW_LINE);
    ::WriteFile(voya_log, buffer, lstrlen(buffer), &length, NULL);
    ::CloseHandle(voya_log);
}

void Thread::postMessage(int UM) {
    ::PostMessage(hwnd, UM, (WPARAM)0, (LPARAM)0);
}

void Thread::restart() {
    if( ::WaitForSingleObject(semapho, WAITED_TOO_LONG) == WAIT_TIMEOUT ) {
        ;
    } else {
        terminated = false;
        ::ReleaseSemaphore(semapho, 1, NULL);
    }

    ::ResumeThread(handle);
}

void Thread::setWindowText(char * s) {
    ::SetWindowText(hwnd, s);
}

void Thread::startSuspend() {
    if( ::WaitForSingleObject(semapho, WAITED_TOO_LONG) == WAIT_TIMEOUT ) {
        ;
    } else {
        terminated = false;
        ::ReleaseSemaphore(semapho, 1, NULL);
    }

    handle = ::CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)startSuspendThread, (LPVOID)hwnd, CREATE_SUSPENDED, &threadID);
}

