#ifndef CONTROLLABLE_H
#define CONTROLLABLE_H

#define STATUS_BAR_HEIGHT 20
#define TITLE_BAR_USED_AS_LABEL 0
#define VOY "voy"
#define VOYA_LOG "voya.log"
#define VOYA_LOG_BATCH "voyaLog.bat"
#define VYBM_EXE "vybm.exe"
#define WAITED_TOO_LONG 10000
#define WHERE_LABEL_HEIGHT 25
#include <sstream>
#include <string>
#include <vector>
#include <windows.h>
#include "struct_vs.h"
#include "voyaID.h"

enum VOYA_CTRL_TEXT_LINES {NOT_AVAILABLE, INSIDE_LINE, MESSAGES_LINE, PRIORITY_LINE, UTC_LINE, CORNER_LINE, PERIOD_LINE, SMALL_WINDOW_LINE, TITLE_BAR_LINE, LEFTMOST_SALT_LINE, QUOTES_GREATER_THAN_LINE, VOYA_MODES_LIST_SIZE};
enum THREAD_PRIORITY {IDLE, LOWEST_PRIORITY, LOWER_PRIORITY, NORMAL_PRIORITY, HIGHER_PRIORITY, HIGHEST_PRIORITY, SYSTEM_PRIORITY};

class Thread {
protected:
    DWORD threadID;
    HANDLE handle;
    HWND hwnd;
    int terminating_period;

public:
    virtual ~Thread(){}

    bool stop();
    bool stop(const char *message);
    void postMessage(int UM);
    void restart();
    void setWindowText(char *s);
    void startSuspend();
    virtual DWORD WINAPI ThreadFunc(LPVOID param) = 0;
};

class Controllable : public Thread {
protected:
    bool voyaErrorOpen;
    char buffer[BUFFER_SIZE];
    char voyaErrorName[ MAX_PATH + 1 ];
    HANDLE voyaErrorHandle;
    HWND whereLabel;
    int inside, messages, priority;

    void closeVoyaError();
    void makeBatchFile();
    void openVoyaError();

public:
    virtual ~Controllable(){}

    void setMessages(int m) {
        messages = m;
    }

    bool closing();
    bool movingInside(bool showWhere);
    FILETIME readVoyaCtrlText();
    void voyaFlush(const char *message);
    void voyaFlush(double sum);
    void voyaFlush(int count);
    void voyaFlush(_STL::string & info);
    void voyaMessage(const char *message);
    void voyaMessage(_STL::string & info);
    virtual DWORD WINAPI ThreadFunc(LPVOID param) = 0;
};

bool removeVoyaErrorFiles();
bool shellExcute(const char *p, const char *q);
char *getLine(char *p, char *line); // voyaFile.cpp
DWORD WINAPI startSuspendThread(LPVOID param);
void createVoyaLog();
void createVoyaLog(char *message);

#endif // CONTROLLABLE_H
