#include "ModalDialog.h"

extern VoyaVersion *version;
namespace {
    const char NEW_LINE[3] = {'\r', '\n', '\0'};
}

BOOL CALLBACK versionDialogFunc(HWND hdwnd, UINT message, WPARAM wParam, LPARAM lParam) {
    return version->DialogFunc(hdwnd, message, wParam, lParam);
}

BOOL CALLBACK VoyaVersion::DialogFunc(HWND h, UINT message, WPARAM wParam, LPARAM lParam) {
    BOOL b = 0;
    hdwnd = h;

    switch(message) {
    case WM_COMMAND:
        switch( LOWORD(wParam) ) {
        case IDCANCEL:
            b = TRUE;
            ::EndDialog(hdwnd, 0);
            break;
        case IDO_HELP:
            ::WinHelp( (HWND)wParam, HELP_FILE_NAME, HELP_CONTEXT, IDH_OVERVIEW);
            b = TRUE;
            break;
        }

        break;
    case WM_INITDIALOG:
        infoLabel = ::CreateWindow(
            "STATIC",
            "",
            WS_CHILD | SS_LEFT,
            infoLabelX,
            infoLabelY,
            infoLabelWidth,
            infoLabelHeight,
            hdwnd,
            (HMENU)infoLabelID,
            instance,
            NULL
        );

        inform();
        ::ShowWindow(infoLabel, SW_SHOW);
        setWindowText("Version");
        centering();
        b = TRUE;
        break;
    }

    return b;
}

bool testSalt(int x) {
    bool b = false;
    char salt[SALT_ARRAY_SIZE] = {' ', '!', '"', '#', '(', ')', '*', '+', ',', '-', '.', '/', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '{', '|', '}', '~', '$', '%', '&', 0x27};
    for(int i = 0; i < SALT_ARRAY_SIZE; ++i) {
        if(salt[i] == x) {
            b = true;
        }

    }

    return b;
}

bool testSalt(_STL::string & g) {
    using namespace _STL;

    bool b = true;
    const string salt(" !\"#()*+,-./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz{|}~$%&'");
    int q = salt.find(g);
    if(q == string::npos) {
        b = false;
    }

    return b;
}

int VoyaVersion::modalDialog() {
    int button = ::DialogBox(instance, "voyaIconDialog", hwnd, (DLGPROC)versionDialogFunc);
    return button;
}

void createVersionText(bool comments) {
    char buffer[BUFFER_SIZE];
    HANDLE version_text = ::CreateFile(VERSION_TEXT, GENERIC_READ | GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
    unsigned long length;

    lstrcpy(buffer, "voya Win32API Version 1.34 \n\n");
    if(comments) {
        lstrcat(buffer, "voya -a # ascii "); lstrcat(buffer, NEW_LINE);
        lstrcat(buffer, "voya -b # title bar of small window "); lstrcat(buffer, NEW_LINE);
        lstrcat(buffer, "voya -c3 # corner-3 "); lstrcat(buffer, NEW_LINE);
        lstrcat(buffer, "voya -g # GMT "); lstrcat(buffer, NEW_LINE);
        lstrcat(buffer, "voya -l # salt "); lstrcat(buffer, NEW_LINE);
        lstrcat(buffer, "voya -m- # No removing "); lstrcat(buffer, NEW_LINE);
        lstrcat(buffer, "voya -n # normalize "); lstrcat(buffer, NEW_LINE);
        lstrcat(buffer, "voya -p9 # period of 9 seconds"); lstrcat(buffer, NEW_LINE);
        lstrcat(buffer, "voya -r # Remove temporary files first."); lstrcat(buffer, NEW_LINE);
        lstrcat(buffer, "voya -s # small "); lstrcat(buffer, NEW_LINE);
        lstrcat(buffer, "voya -t # through the text area "); lstrcat(buffer, NEW_LINE);
        lstrcat(buffer, "voya -V # version "); lstrcat(buffer, NEW_LINE);
        lstrcat(buffer, "voya -VC # version with comments "); lstrcat(buffer, NEW_LINE);
        lstrcat(buffer, "voya -w # wide "); lstrcat(buffer, NEW_LINE);
        lstrcat(buffer, "voya -x # Exclude quotes. "); lstrcat(buffer, NEW_LINE);
    } else {
        lstrcat(buffer, "voya -a "); lstrcat(buffer, NEW_LINE);
        lstrcat(buffer, "voya -b "); lstrcat(buffer, NEW_LINE);
        lstrcat(buffer, "voya -c "); lstrcat(buffer, NEW_LINE);
        lstrcat(buffer, "voya -g "); lstrcat(buffer, NEW_LINE);
        lstrcat(buffer, "voya -l "); lstrcat(buffer, NEW_LINE);
        lstrcat(buffer, "voya -m- "); lstrcat(buffer, NEW_LINE);
        lstrcat(buffer, "voya -n "); lstrcat(buffer, NEW_LINE);
        lstrcat(buffer, "voya -p "); lstrcat(buffer, NEW_LINE);
        lstrcat(buffer, "voya -r "); lstrcat(buffer, NEW_LINE);
        lstrcat(buffer, "voya -s "); lstrcat(buffer, NEW_LINE);
        lstrcat(buffer, "voya -t "); lstrcat(buffer, NEW_LINE);
        lstrcat(buffer, "voya -V "); lstrcat(buffer, NEW_LINE);
        lstrcat(buffer, "voya -VC "); lstrcat(buffer, NEW_LINE);
        lstrcat(buffer, "voya -w "); lstrcat(buffer, NEW_LINE);
        lstrcat(buffer, "voya -x "); lstrcat(buffer, NEW_LINE);
    }

    ::WriteFile(version_text, buffer, lstrlen(buffer), &length, NULL);
    ::CloseHandle(version_text);
}

void ModalDialog::centering() {
    const int HALF_WIDTH = width/2;
    const int HALF_HEIGHT = height/2;

    const BOOL REPAINT = TRUE;
    const int MENU_HEIGHT = ::GetSystemMetrics(SM_CYMENU);
    const int SCREEN_WIDTH = ::GetSystemMetrics(SM_CXFULLSCREEN);
    int screen_height = ::GetSystemMetrics(SM_CYFULLSCREEN);
    screen_height += MENU_HEIGHT;
    const int CENTER_X = SCREEN_WIDTH/2 - HALF_WIDTH;
    const int CENTER_Y = screen_height/2 - HALF_HEIGHT;

    ::MoveWindow(hdwnd, CENTER_X, CENTER_Y, width, height, REPAINT);
}

void ModalDialog::inform() {
    ::SetWindowText(infoLabel, info.c_str() );
    ::ShowWindow(infoLabel, SW_SHOW);
}

void ModalDialog::clear() {
    using namespace _STL;

    const string EMPTY("");
    info = EMPTY;
    ::SetWindowText(infoLabel, info.c_str() );
    ::ShowWindow(infoLabel, SW_HIDE);
}

void ModalDialog::postMessage(int UM) {
    ::PostMessage(hwnd, UM, (WPARAM)0, (LPARAM)0);
}

void ModalDialog::setWindowText(char * s) {
    ::SetWindowText(hdwnd, s);
}

VoyaVersion::VoyaVersion(HINSTANCE i, HWND h): INFO_LABEL_HEIGHT(25), VERSION_HEIGHT(283), VERSION_WIDTH(273) {
    using namespace _STL;

    const int BORDER = ::GetSystemMetrics(SM_CYDLGFRAME);
    const int TITLE_BAR_HEIGHT = ::GetSystemMetrics(SM_CYCAPTION);

    hwnd = h;
    instance = i;
    infoLabelHeight = INFO_LABEL_HEIGHT;
    infoLabelID = IDO_LABEL;
    infoLabelWidth = VERSION_WIDTH - 2*BORDER;
    infoLabelX = BORDER;
    infoLabelY = VERSION_HEIGHT - TITLE_BAR_HEIGHT - INFO_LABEL_HEIGHT - BORDER;
    width = VERSION_WIDTH;
    height = VERSION_HEIGHT;

    stringstream ss;
    ss << "Version " << VERSION;
    info = ss.str();
}

