#include "voyaControl.h"

extern VOYA_SETTING *vs;
extern VoyaControl *control;

BOOL CALLBACK controlDialogFunc(HWND hdwnd, UINT message, WPARAM wParam, LPARAM lParam) {
    return control->DialogFunc(hdwnd, message, wParam, lParam);
}

BOOL CALLBACK VoyaControl::DialogFunc(HWND h, UINT message, WPARAM wParam, LPARAM lParam) {
    BOOL b = 0;
    hdwnd = h;

    switch(message) {
    case WM_COMMAND:
        switch( LOWORD(wParam) ) {
        case IDCANCEL:
            readVoyaCtrlText();
            b = TRUE;
            ::EndDialog(hdwnd, 0);
            break;
        case IDC_CANCEL:
            readVoyaCtrlText();
            b = TRUE;
            ::EndDialog(hdwnd, 0);
            break;
        case IDC_CONTROL_HELP:
            ::WinHelp( (HWND)wParam, HELP_FILE_NAME, HELP_CONTEXT, IDH_CONTROL);
            b = TRUE;
            break;
        case IDC_RADIOBUTTON1:
            setInside(true);
            ::SendDlgItemMessage(hdwnd, IDC_RADIOBUTTON1, BM_SETCHECK, BST_CHECKED, 0);
            ::SendDlgItemMessage(hdwnd, IDC_RADIOBUTTON2, BM_SETCHECK, BST_UNCHECKED, 0);
            break;
        case IDC_RADIOBUTTON2:
            setInside(false);
            ::SendDlgItemMessage(hdwnd, IDC_RADIOBUTTON1, BM_SETCHECK, BST_UNCHECKED, 0);
            ::SendDlgItemMessage(hdwnd, IDC_RADIOBUTTON2, BM_SETCHECK, BST_CHECKED, 0);
            break;
        case IDC_RADIOBUTTON3:
            setMessages(true);
            ::SendDlgItemMessage(hdwnd, IDC_RADIOBUTTON3, BM_SETCHECK, BST_CHECKED, 0);
            ::SendDlgItemMessage(hdwnd, IDC_RADIOBUTTON4, BM_SETCHECK, BST_UNCHECKED, 0);
            break;
        case IDC_RADIOBUTTON4:
            setMessages(false);
            ::SendDlgItemMessage(hdwnd, IDC_RADIOBUTTON3, BM_SETCHECK, BST_UNCHECKED, 0);
            ::SendDlgItemMessage(hdwnd, IDC_RADIOBUTTON4, BM_SETCHECK, BST_CHECKED, 0);
            break;
        case IDC_SEARCH:
            clear();
            if( writeVoyaCtrlText() ) {
                readVoyaCtrlText();
                b = TRUE;
                ::EndDialog(hdwnd, IDC_SEARCH);
            } else {
                inform();
            }

            break;
        }

        break;
    case WM_INITDIALOG:
        readVoyaCtrlText();

        infoLabel = ::CreateWindow(
            "STATIC",
            "",
            WS_CHILD | SS_LEFT,
            infoLabelX,
            infoLabelY,
            infoLabelWidth,
            infoLabelHeight,
            hdwnd,
            (HMENU)infoLabelID,
            instance,
            NULL
        );

        ::ShowWindow(infoLabel, SW_HIDE);
        setWindowText("Control");
        centering();
        b = TRUE;
        break;
    }

    return b;
}

bool VoyaControl::writeVoyaCtrlText() {
    using namespace _STL;

    ifstream text(VOYA_CTRL_TEXT);
    if( text.fail() ) {
        const string FILE_ERROR("Please prepare voyaCtrl.txt");
        info = FILE_ERROR;
        return false;
    }

    int setting[VOYA_MODES_LIST_SIZE];
    for(int i = 0; i < VOYA_MODES_LIST_SIZE; ++i) {
        setting[i] = 0;
    }

    char c[SMALL_BUFFER_SIZE];
    int i = INSIDE_LINE;
    while( text.getline(c, sizeof(c) ) ) {
        setting[i] = c[0] - '0';

        if(++i > LEFTMOST_SALT_LINE) {
            break;
        }
    }

    text.close();
    ofstream voyaCtrl(VOYA_CTRL_TEXT);

// writeVoyaCtrlText() of VoyaControl can edit two items: inside and messages.
    setting[INSIDE_LINE] = inside;
    setting[MESSAGES_LINE] = messages;
    for(int i = INSIDE_LINE; i < VOYA_MODES_LIST_SIZE; ++i) {
        voyaCtrl << setting[i] << endl;
    }

    return true;
}

int VoyaControl::modalDialog() {
    int button = ::DialogBox(instance, "voyaControlDialog", hwnd, (DLGPROC)controlDialogFunc);
    return button;
}

void VoyaControl::readVoyaCtrlText() {
    using namespace _STL;

    ifstream text(VOYA_CTRL_TEXT);
    if( text.fail() ) {
        return;
    }

    int setting[VOYA_MODES_LIST_SIZE];
    for(int i = 0; i < VOYA_MODES_LIST_SIZE; ++i) {
        setting[i] = 0;
    }

    char c[SMALL_BUFFER_SIZE];
    int i = INSIDE_LINE;
    while( text.getline(c, sizeof(c) ) ) {
        setting[i] = c[0] - '0';

        if(++i > LEFTMOST_SALT_LINE) {
            break;
        }
    }

    text.close();
    priority = setting[PRIORITY_LINE];

    bool searchWhenMovingInside;
    inside = setting[INSIDE_LINE];
    switch(inside) {
    case 0:
        ::SendDlgItemMessage(hdwnd, IDC_RADIOBUTTON1, BM_SETCHECK, BST_UNCHECKED, 0);
        ::SendDlgItemMessage(hdwnd, IDC_RADIOBUTTON2, BM_SETCHECK, BST_CHECKED, 0);
        searchWhenMovingInside = false;
        break;
    default:
        ::SendDlgItemMessage(hdwnd, IDC_RADIOBUTTON1, BM_SETCHECK, BST_CHECKED, 0);
        ::SendDlgItemMessage(hdwnd, IDC_RADIOBUTTON2, BM_SETCHECK, BST_UNCHECKED, 0);
        inside = 1;
        searchWhenMovingInside = true;
        break;
    }

    bool removeVoyaError;
    messages = setting[MESSAGES_LINE];
    switch(messages) {
    case 0:
        ::SendDlgItemMessage(hdwnd, IDC_RADIOBUTTON3, BM_SETCHECK, BST_UNCHECKED, 0);
        ::SendDlgItemMessage(hdwnd, IDC_RADIOBUTTON4, BM_SETCHECK, BST_CHECKED, 0);
        removeVoyaError = false;
        break;
    default:
        ::SendDlgItemMessage(hdwnd, IDC_RADIOBUTTON3, BM_SETCHECK, BST_CHECKED, 0);
        ::SendDlgItemMessage(hdwnd, IDC_RADIOBUTTON4, BM_SETCHECK, BST_UNCHECKED, 0);
        messages = 1;
        removeVoyaError = true;
        break;
    }

// struct VOYA_SETTING
    vs->removeVoyaError = removeVoyaError;
    vs->searchWhenMovingInside = searchWhenMovingInside;
    vs->thread_priority = priority;
}

void VoyaControl::setInside(bool b) {
    if(b) {
        inside = 1;
    } else {
        inside = 0;
    }

    return;
}

void VoyaControl::setMessages(bool b) {
    if(b) {
        messages = 1;
    } else {
        messages = 0;
    }

    return;
}

void VoyaControl::setPriority(int p) {
    priority = p;
    return;
}

VoyaControl::VoyaControl(HINSTANCE i, HWND h): INFO_LABEL_HEIGHT(25), CONTROL_HEIGHT(266), CONTROL_WIDTH(201) {
    const int BORDER = ::GetSystemMetrics(SM_CYDLGFRAME);
    const int TITLE_BAR_HEIGHT = ::GetSystemMetrics(SM_CYCAPTION);

    hwnd = h;
    instance = i;
    infoLabelHeight = INFO_LABEL_HEIGHT;
    infoLabelID = IDC_LABEL;
    infoLabelWidth = CONTROL_WIDTH - 2*BORDER;
    infoLabelX = BORDER;
    infoLabelY = CONTROL_HEIGHT - TITLE_BAR_HEIGHT - INFO_LABEL_HEIGHT - BORDER;
    width = CONTROL_WIDTH;
    height = CONTROL_HEIGHT;
}

