#include "voyaSetting.h"

extern Controllable *thread;
VOYA_SETTING voyaSetting;
VOYA_SETTING *vs = &voyaSetting;
extern VoyaDetails *details;
extern VoyaSetting *setting;

namespace {
    const double AIM = 30000.;
    double EPSILON = 0.5;
}

BOOL CALLBACK detailsDialogFunc(HWND hdwnd, UINT message, WPARAM wParam, LPARAM lParam) {
    return details->DialogFunc(hdwnd, message, wParam, lParam);
}

BOOL CALLBACK settingDialogFunc(HWND hdwnd, UINT message, WPARAM wParam, LPARAM lParam) {
    return setting->DialogFunc(hdwnd, message, wParam, lParam);
}

BOOL CALLBACK VoyaDetails::DialogFunc(HWND h, UINT message, WPARAM wParam, LPARAM lParam) {
    using namespace _STL;

    BOOL b = 0;
    double sum;
    hdwnd = h;
    stringstream ss;

    switch(message) {
    case WM_COMMAND:
        switch( LOWORD(wParam) ) {
        case IDCANCEL:
            b = TRUE;
            ::EndDialog(hdwnd, 0);
            break;
        case IDD_ASCII:
            clear();
            useNoCharacterButASCII();
            if( settingBy(VOYA_CSV) ) {
                readLabels();
                b = TRUE;
                ::EndDialog(hdwnd, IDD_SEARCH);
            } else {
                inform();
            }

            break;
        case IDD_CANCEL:
            b = TRUE;
            ::EndDialog(hdwnd, 0);
            break;
        case IDD_HELP:
            ::WinHelp( (HWND)wParam, HELP_FILE_NAME, HELP_CONTEXT, IDH_DETAILS);
            b = TRUE;
            break;
        case IDD_NORMALIZE:
            clear();
            writeVoyaCSV();
            if( settingBy(VOYA_CSV) ) {
                readLabels();
                sum = shiftJisKanaAscii();
                normalizing(sum, AIM);
            } else {
                inform();
            }

            break;
        case IDD_SEARCH:
            clear();
            writeVoyaCSV();
            if( settingBy(VOYA_CSV) ) {
                readLabels();
                b = TRUE;
                ::EndDialog(hdwnd, IDD_SEARCH);
            } else {
                inform();
            }

            break;
        }

        break;
    case WM_INITDIALOG:
        readDetails();

        infoLabel = ::CreateWindow(
            "STATIC",
            "",
            WS_CHILD | SS_LEFT,
            infoLabelX,
            infoLabelY,
            infoLabelWidth,
            infoLabelHeight,
            hdwnd,
            (HMENU)infoLabelID,
            instance,
            NULL
        );

        ::ShowWindow(infoLabel, SW_HIDE);
        setWindowText("Details");
        centering();
        b = TRUE;
        break;
    }

    return b;
}

BOOL CALLBACK VoyaSetting::DialogFunc(HWND h, UINT message, WPARAM wParam, LPARAM lParam) {
    BOOL b = 0;
    hdwnd = h;

    switch(message) {
    case WM_COMMAND:
        switch( LOWORD(wParam) ) {
        case IDCANCEL:
            b = TRUE;
            ::EndDialog(hdwnd, 0);
            break;
        case IDI_CANCEL:
            b = TRUE;
            ::EndDialog(hdwnd, 0);
            break;
        case IDI_HELP:
            ::WinHelp( (HWND)wParam, HELP_FILE_NAME, HELP_CONTEXT, IDH_SETTING);
            b = TRUE;
            break;
        case IDI_SEARCH:
            clear();
            if( writeVoyaCSV() ) {
                settingBy(VOYA_CSV);
                readLabels();
                b = TRUE;
                ::EndDialog(hdwnd, IDI_SEARCH);
            } else {
                inform();
            }

            break;
        }

        break;
    case WM_INITDIALOG:
        readSetting();

        infoLabel = ::CreateWindow(
            "STATIC",
            "",
            WS_CHILD | SS_LEFT,
            infoLabelX,
            infoLabelY,
            infoLabelWidth,
            infoLabelHeight,
            hdwnd,
            (HMENU)infoLabelID,
            instance,
            NULL
        );

        ::ShowWindow(infoLabel, SW_HIDE);
        setWindowText("Setting");
        centering();
        b = TRUE;
        break;
    }

    return b;
}

bool copyLogFileName(char *sequence, int end) {
    char *edited_line;
    int i = end;
    while(i > 0) {
        if(sequence[i] == ',') {
            break;
        }

        --i;
    }

    if(sequence[i] != ',') {
        thread->voyaFlush("Comma should come before log file.");
        return false;
    }

    for(;;) {
        if(sequence[i] == ',') {
            ++i;
            continue;
        } else if(sequence[i] == ' ') {
            ++i;
            continue;
        } else {
            break;
        }
    }

    edited_line = & vs->edited[LOG_LINE][0];
    lstrcpy(edited_line, &sequence[i]);
    lstrcpy(vs->log_file_name, &sequence[i]);
    return true;
}

bool copyRequest(char *sequence, int end) {
    char *edited_line;
    int i = end;
    while(i > 0) {
        if(sequence[i] == ',') {
            break;
        }

        --i;
    }

    if(sequence[i] != ',') {
        thread->voyaFlush("Comma should come before request.");
        return false;
    }

    for(;;) {
        if(sequence[i] == ',') {
            ++i;
            continue;
        } else if(sequence[i] == ' ') {
            ++i;
            continue;
        } else {
            break;
        }
    }

    edited_line = & vs->edited[REQUEST_LINE][0];
    lstrcpy(edited_line, &sequence[i]);
    lstrcpy(vs->request, &sequence[i]);
    return true;
}

bool copySalt(char *sequence, int end) {
    char *edited_line;
    int i = end;
    while(i > 0) {
        if(sequence[i] == ',') {
            break;
        }

        --i;
    }

    if(sequence[i] != ',') {
        thread->voyaFlush("Comma should come before salt.");
        return false;
    }

    for(;;) {
        if(sequence[i] == ',') {
            ++i;
            continue;
        } else if(sequence[i] == ' ') {
            ++i;
            continue;
        } else {
            break;
        }
    }

    edited_line = & vs->edited[SALT_LINE][0];
    lstrcpy(edited_line, &sequence[i]);
    lstrcpy(vs->salt, &sequence[i]);
    return true;
}

bool createRemoveErrorText() { // Clean
    using namespace _STL;

    bool found;
    const int FILE_TYPE_COUNT = 8;
    const string e0("voy*.tmp");
    const string e1("TMP*.$$$");
    const string e2("");
    const string e3("");
    const string e4("");
    const string e5("");
    const string e6("");
    const string e7("");
    string file_type[FILE_TYPE_COUNT];
    HANDLE find_first_next_handle;
    ofstream ret(REMOVE_ERROR_TEXT);
    WIN32_FIND_DATA find_data;

    file_type[0] = e0;
    file_type[1] = e1;
    file_type[2] = e2;
    file_type[3] = e3;
    file_type[4] = e4;
    file_type[5] = e5;
    file_type[6] = e6;
    file_type[7] = e7;
    for(int i = 0; i < FILE_TYPE_COUNT; ++i) {
        if( file_type[i].empty() ) {
            break;
        }

        find_first_next_handle = ::FindFirstFile( file_type[i].c_str(), &find_data);
        if(find_first_next_handle == INVALID_HANDLE_VALUE) {
            continue;
        }

        found = true;
        while(found) {
            ret << find_data.cFileName << endl;
            found = ::FindNextFile(find_first_next_handle, &find_data);
        }

        ::FindClose(find_first_next_handle);
    }

    return true;
}

bool readLabels() {
    using namespace _STL;

    char c[SMALL_BUFFER_SIZE];
    const string HEADER_WORD("Word");
    const string HEADER_DATA("Data");
    string *p = new string("");
    DividingCSV *t = new DividingCSV(*p);
    string cells[CSV_COLUMN_COUNT][ LABEL_COUNT + 1 ];
    ifstream in(VOYA_CSV);
    if( in.fail() ) {
        thread->voyaFlush("readLabels(), ifstream in(VOYA_CSV);");
        return false;
    }

// Reading and copying to produce an array
    cells[0][0] = HEADER_WORD;
    cells[1][0] = HEADER_DATA;
    int line = 1;
    while( in.getline(c, SMALL_BUFFER_SIZE) ) {
        delete p;
        p = new string(c);

        delete t;
        t = new DividingCSV(*p);
        for(int i = 0; i < CSV_COLUMN_COUNT; ++i) {
            cells[i][line] = t->a(i);
        }

        if(++line > LABEL_COUNT) {
            break;
        }
    }

// Storing labels
    vs->label[0] = HEADER_WORD;
    for(line = 1; line <= LABEL_COUNT; ++line) {
        vs->label[line] = cells[0][line];
    }

    delete t;
    delete p;
    return true;
}

bool removeSalt() {
    using namespace _STL;

    char c[BUFFER_SIZE];
    const int COMMA_SIZE = 1;
    const string COMMA(",");
    const string NEW_LINE("\n");
    int q;
    int line = 0;
    ifstream in(VOYA_CSV);
    if( in.fail() ) {
        return false;
    }

    string head;
    stringstream ss;
    string *p = new string("");
    while( in.getline(c, SMALL_BUFFER_SIZE) ) {
        line += 1;
        delete p;
        p = new string(c);

        if(line == SALT_LINE) {
// Removing salt from VOYA_CSV
            q = p->find(COMMA);
            if(q != string::npos) {
                q += COMMA_SIZE;
                head = p->substr(0, q);
                ss << head << NEW_LINE;
                continue;
            }

        }

        ss << c << NEW_LINE;
    }

    in.close();

// Writing
    ofstream file(VOYA_CSV);
    string s = ss.str();
    file << s;
    return true;
}

bool settingBy(const char *fileName) {
    char *p;
    char buffer[BUFFER_SIZE];
    char sequence[BUFFER_SIZE];
    int length, m;
    unsigned long read_length;

    HANDLE voyaCSV = ::CreateFile(fileName, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
    if(voyaCSV == INVALID_HANDLE_VALUE) {
        thread->voyaFlush("voyaCSV == INVALID_HANDLE_VALUE");
        return false;
    }

    ::ReadFile(voyaCSV, buffer, sizeof(buffer), &read_length, NULL);
    buffer[read_length] = '\0';
    p = &buffer[0];

// request
    p = getLine(p, sequence);
    if(p == NULL) {
        ::CloseHandle(voyaCSV);
        thread->voyaFlush("request, p = getLine(p, sequence);");
        return false;
    }

    length = removeComment(sequence);
    if( copyRequest(sequence, length) ) {
        if( metaCharacterIncluded(vs->request) ) {
            vs->regular_expression = true;

/* 
012345678901
^[Rr]uda[/.]
*/
            int u = lstrlen(vs->request);
            if(vs->request[0] == '^') {
                if(vs->request[1] == '[') {
                    if(vs->request[4] == ']') {
                        if(vs->request[ u - 1 ] == ']') {
                            if(vs->request[ u - 4 ] == '[') {
                                vs->dragon_seek = true;
                            } else {
                                vs->dragon_seek = false;
                            }
                        } else {
                            vs->dragon_seek = false;
                        }
                    } else {
                        vs->dragon_seek = false;
                    }
                } else {
                    vs->dragon_seek = false;
                }
            } else {
                vs->dragon_seek = false;
            }

        } else {
            vs->regular_expression = false;
            vs->dragon_seek = false;
        }
    } else {
        ::CloseHandle(voyaCSV);
        thread->voyaFlush("copyRequest(sequence, length)");
        return false;
    }

// log file name
    p = getLine(p, sequence);
    if(p == NULL) {
        ::CloseHandle(voyaCSV);
        thread->voyaFlush("log file name, p = getLine(p, sequence);");
        return false;
    }

    length = removeComment(sequence);
    if(! copyLogFileName(sequence, length) ) {
        ::CloseHandle(voyaCSV);
        thread->voyaFlush("copyLogFileName(sequence, length)");
        return false;
    }

// Converting strings to integers
    char *edited_line;
    int a[ARRAY_LENGTH];
    int line = LEFTMOST_LINE;
    for(int i = 0; i < ARRAY_LENGTH; ++i) {
        p = getLine(p, sequence);
        if(p == NULL) {
            ::CloseHandle(voyaCSV);
            thread->voyaFlush("Converting strings to integers, p = getLine(p, sequence);");
            return false;
        }

        length = removeComment(sequence);
        a[i] = commaShouldComeBeforeInteger(sequence, length, i); // vs->buffer
        edited_line = & vs->edited[line][0];
        lstrcpy(edited_line, vs->buffer);
        ++line;
    }

// salt
    p = getLine(p, sequence);
    if(p == NULL) {
        ::CloseHandle(voyaCSV);
        thread->voyaFlush("salt, voyaCSV.getline(sequence, BUFFER_SIZE)");
        return false;
    }

    length = removeComment(sequence);
    if( copySalt(sequence, length) ) {
        if( testSalt(vs->salt[0]) ) {
            if( testSalt(vs->salt[1]) ) {
                if(vs->leftmost_salt) {
                    ;
                } else {
                    thread->voyaFlush("testSalt(vs->salt[0]) && testSalt(vs->salt[1]) is inconsistent with vs->leftmost_salt.");
                }

            }

        }

    } else {
        ::CloseHandle(voyaCSV);
        thread->voyaFlush("copySalt(sequence, length)");
        return false;
    }

// weight
    vs->ascii_weight = a[4];
    vs->capital_omega_weight = a[5];
    vs->capital_ya_weight = a[6];
    vs->capital_z_weight = a[7];
    vs->cap_weight = a[8];
    vs->circle_weight = a[9];
    vs->cup_weight = a[10];
    vs->double_integral_weight = a[11];
    vs->exists_weight = a[12];
    vs->hankaku_katakana_weight = a[13];
    vs->heisei_weight = a[14];
    vs->hiragana_weight = a[15];
    vs->jis_level_one_kanji_weight = a[16];
    vs->jis_level_two_kanji_weight = a[17];
    vs->katakana_weight = a[18];
    vs->keisen_weight = a[19];
    vs->nine_weight = a[20];
    vs->omega_weight = a[21];
    vs->paragraph_weight = a[22];
    vs->parallel_weight = a[23];
    vs->roman_weight = a[24];
    vs->square_meter_weight = a[25];
    vs->ya_weight = a[26];
    vs->z_weight = a[27];

// constants
    vs->interim_report = a[2];
    vs->idling = a[3];

    if(vs->regular_expression) {
        if(vs->dragon_seek) {
            vs->case_sensitive = true;
            vs->leftmost = 1;
            vs->lower_case = false;
        } else {
            vs->case_sensitive = false;
            vs->leftmost = 0;
            vs->lower_case = false;
        }

    } else {
        bool used;
        int tail, u;
        vs->case_sensitive = (bool)a[1];
        vs->leftmost = a[0];
        if(vs->leftmost == -1) { // rightmost
            u = lstrlen(vs->request);
            u -= 1;
            tail = (int)vs->request[u]; // the last character of request
            used = tenthCharacterOfTori(tail);
            if(used) {
                useLowerCaseUnlessCaseSensitive();
            } else {
                thread->voyaFlush(SHOULD_BE_USED_AS_TENTH_CHARACTER);
                if(vs->case_sensitive) {
                    ::CloseHandle(voyaCSV);
                    return false;
                } else {
// Correcting automatically
                    int aeim[AEIM_CHARACTER] = {'a', 'e', 'i', 'm', 'q', 'u', 'y', 'C', 'G', 'K', 'O', 'S', 'W'};
                    used = false;
                    for(m = 0; m < AEIM_CHARACTER; ++m) {
                        if(tail == aeim[m]) {
                            used = true;
                            break;
                        }
                    }

                    if(used) {
                        int CGKO[CGKO_CHARACTER] = {'C', 'G', 'K', 'O', 'S', 'W'};
                        used = false;
                        for(m = 0; m < CGKO_CHARACTER; ++m) {
                            if(tail == CGKO[m]) {
                                used = true;
                                break;
                            }
                        }

                        if(used) { // 'C', 'G', 'K', 'O', 'S', 'W'
                            vs->lower_case = true;
                            ::CharLower(vs->request);
                        } else { // 'a', 'e', 'i', 'm', 'q', 'u', 'y' 
                            vs->lower_case = false;
                            ::CharUpper(vs->request);
                        }
                    } else {
                        ::CloseHandle(voyaCSV);
                        thread->voyaFlush("int aeim[AEIM_CHARACTER] = {'a', 'e', 'i', 'm', 'q', 'u', 'y', 'C', 'G', 'K', 'O', 'S', 'W'}; if(used) ");
                        return false;
                    }
                }
            }
        } else {
            useLowerCaseUnlessCaseSensitive();
        }
    }

    ::CloseHandle(voyaCSV);
    return true;
}

bool VoyaDetails::readDetails() {
    using namespace _STL;

    char c[SMALL_BUFFER_SIZE];
    const string HEADER_WORD("Word");
    const string HEADER_DATA("Data");
    string *p = new string("");
    DividingCSV *t = new DividingCSV(*p);
    string cells[CSV_COLUMN_COUNT][ LABEL_COUNT + 1 ];
    ifstream in(VOYA_CSV);
    if( in.fail() ) {
        const string FILE_ERROR("readDetails(), ifstream in(VOYA_CSV);");
        info = FILE_ERROR;
        return false;
    }

// Reading and copying to produce an array
    cells[0][0] = HEADER_WORD;
    cells[1][0] = HEADER_DATA;
    int line = 1;
    while( in.getline(c, SMALL_BUFFER_SIZE) ) {
        delete p;
        p = new string(c);

        delete t;
        t = new DividingCSV(*p);
        for(int i = 0; i < CSV_COLUMN_COUNT; ++i) {
            cells[i][line] = t->a(i);
        }

        if(++line > LABEL_COUNT) {
            break;
        }
    }

// Storing labels
    vs->label[0] = HEADER_WORD;
    for(line = 1; line <= LABEL_COUNT; ++line) {
        vs->label[line] = cells[0][line];
    }

    delete t;
    delete p;

    ::SetDlgItemText(hdwnd, IDD_EDITBOX01, cells[1][19].c_str() ); // kanji-1
    ::SetDlgItemText(hdwnd, IDD_EDITBOX02, cells[1][20].c_str() ); // kanji-2
    ::SetDlgItemText(hdwnd, IDD_EDITBOX03, cells[1][13].c_str() );
    ::SetDlgItemText(hdwnd, IDD_EDITBOX04, cells[1][17].c_str() );
    ::SetDlgItemText(hdwnd, IDD_EDITBOX05, cells[1][22].c_str() ); // keisen
    ::SetDlgItemText(hdwnd, IDD_EDITBOX06, cells[1][26].c_str() );
    ::SetDlgItemText(hdwnd, IDD_EDITBOX07, cells[1][16].c_str() ); // hankaku
    ::SetDlgItemText(hdwnd, IDD_EDITBOX08, cells[1][28].c_str() );
    ::SetDlgItemText(hdwnd, IDD_EDITBOX09, cells[1][21].c_str() );
    ::SetDlgItemText(hdwnd, IDD_EDITBOX10, cells[1][18].c_str() ); // hiragana
    ::SetDlgItemText(hdwnd, IDD_EDITBOX11, cells[1][7].c_str() ); // ASCII
    ::SetDlgItemText(hdwnd, IDD_EDITBOX12, cells[1][10].c_str() );
    ::SetDlgItemText(hdwnd, IDD_EDITBOX13, cells[1][9].c_str() );
    ::SetDlgItemText(hdwnd, IDD_EDITBOX14, cells[1][8].c_str() );
    ::SetDlgItemText(hdwnd, IDD_EDITBOX15, cells[1][30].c_str() );
    ::SetDlgItemText(hdwnd, IDD_EDITBOX16, cells[1][29].c_str() );
    ::SetDlgItemText(hdwnd, IDD_EDITBOX17, cells[1][24].c_str() );
    ::SetDlgItemText(hdwnd, IDD_EDITBOX18, cells[1][27].c_str() );
    ::SetDlgItemText(hdwnd, IDD_EDITBOX19, cells[1][23].c_str() );
    ::SetDlgItemText(hdwnd, IDD_EDITBOX20, cells[1][12].c_str() );
    ::SetDlgItemText(hdwnd, IDD_EDITBOX21, cells[1][25].c_str() );
    ::SetDlgItemText(hdwnd, IDD_EDITBOX22, cells[1][14].c_str() ); // roman
    ::SetDlgItemText(hdwnd, IDD_EDITBOX23, cells[1][15].c_str() );
    ::SetDlgItemText(hdwnd, IDD_EDITBOX24, cells[1][11].c_str() );

    ::SetDlgItemText(hdwnd, IDD_LABEL01, cells[0][19].c_str() );
    ::SetDlgItemText(hdwnd, IDD_LABEL02, cells[0][20].c_str() );
    ::SetDlgItemText(hdwnd, IDD_LABEL03, cells[0][13].c_str() );
    ::SetDlgItemText(hdwnd, IDD_LABEL04, cells[0][17].c_str() );
    ::SetDlgItemText(hdwnd, IDD_LABEL05, cells[0][22].c_str() );
    ::SetDlgItemText(hdwnd, IDD_LABEL06, cells[0][26].c_str() );
    ::SetDlgItemText(hdwnd, IDD_LABEL07, cells[0][16].c_str() );
    ::SetDlgItemText(hdwnd, IDD_LABEL08, cells[0][28].c_str() );
    ::SetDlgItemText(hdwnd, IDD_LABEL09, cells[0][21].c_str() );
    ::SetDlgItemText(hdwnd, IDD_LABEL10, cells[0][18].c_str() );
    ::SetDlgItemText(hdwnd, IDD_LABEL11, cells[0][7].c_str() );
    ::SetDlgItemText(hdwnd, IDD_LABEL12, cells[0][10].c_str() );
    ::SetDlgItemText(hdwnd, IDD_LABEL13, cells[0][9].c_str() );
    ::SetDlgItemText(hdwnd, IDD_LABEL14, cells[0][8].c_str() );
    ::SetDlgItemText(hdwnd, IDD_LABEL15, cells[0][30].c_str() );
    ::SetDlgItemText(hdwnd, IDD_LABEL16, cells[0][29].c_str() );
    ::SetDlgItemText(hdwnd, IDD_LABEL17, cells[0][24].c_str() );
    ::SetDlgItemText(hdwnd, IDD_LABEL18, cells[0][27].c_str() );
    ::SetDlgItemText(hdwnd, IDD_LABEL19, cells[0][23].c_str() );
    ::SetDlgItemText(hdwnd, IDD_LABEL20, cells[0][12].c_str() );
    ::SetDlgItemText(hdwnd, IDD_LABEL21, cells[0][25].c_str() );
    ::SetDlgItemText(hdwnd, IDD_LABEL22, cells[0][14].c_str() );
    ::SetDlgItemText(hdwnd, IDD_LABEL23, cells[0][15].c_str() );
    ::SetDlgItemText(hdwnd, IDD_LABEL24, cells[0][11].c_str() );
    return true;
}

bool VoyaSetting::readSetting() {
    using namespace _STL;

    char c[SMALL_BUFFER_SIZE];
    const string HEADER_WORD("Word");
    const string HEADER_DATA("Data");
    string *p = new string("");
    DividingCSV *t = new DividingCSV(*p);
    string cells[CSV_COLUMN_COUNT][ LABEL_COUNT + 1 ];
    ifstream in(VOYA_CSV);
    if( in.fail() ) {
        const string FILE_ERROR("readSetting(), ifstream in(VOYA_CSV);");
        info = FILE_ERROR;
        return false;
    }

// Reading and copying to produce an array
    cells[0][0] = HEADER_WORD;
    cells[1][0] = HEADER_DATA;
    int line = 1;
    while( in.getline(c, SMALL_BUFFER_SIZE) ) {
        delete p;
        p = new string(c);

        delete t;
        t = new DividingCSV(*p);
        for(int i = 0; i < CSV_COLUMN_COUNT; ++i) {
            cells[i][line] = t->a(i);
        }

        if(++line > LABEL_COUNT) {
            break;
        }
    }

// Storing labels
    vs->label[0] = HEADER_WORD;
    for(line = 1; line <= LABEL_COUNT; ++line) {
        vs->label[line] = cells[0][line];
    }

    delete t;
    delete p;

    ::SetDlgItemText(hdwnd, IDI_EDITBOX1, cells[1][1].c_str() ); 
    ::SetDlgItemText(hdwnd, IDI_EDITBOX2, cells[1][2].c_str() ); 
    ::SetDlgItemText(hdwnd, IDI_EDITBOX3, cells[1][5].c_str() ); 
    ::SetDlgItemText(hdwnd, IDI_EDITBOX4, cells[1][6].c_str() ); 

    ::SetDlgItemText(hdwnd, IDI_LABEL1, cells[0][REQUEST_LINE].c_str() ); 
    ::SetDlgItemText(hdwnd, IDI_LABEL2, cells[0][LOG_LINE].c_str() ); 
    ::SetDlgItemText(hdwnd, IDI_LABEL3, cells[0][LEFTMOST_LINE].c_str() ); 
    ::SetDlgItemText(hdwnd, IDI_LABEL4, cells[0][CASE_SENSITIVE_LINE].c_str() ); 
    ::SetDlgItemText(hdwnd, IDI_LABEL5, cells[0][REPORT_LINE].c_str() ); 
    ::SetDlgItemText(hdwnd, IDI_LABEL6, cells[0][IDLING_LINE].c_str() ); 

    ::SendDlgItemMessage(hdwnd, IDI_COMBOBOX1, CB_INSERTSTRING, 0, (LPARAM)"1 # Yes" );
    ::SendDlgItemMessage(hdwnd, IDI_COMBOBOX1, CB_INSERTSTRING, 1, (LPARAM)"0" );
    ::SendDlgItemMessage(hdwnd, IDI_COMBOBOX1, CB_INSERTSTRING, 2, (LPARAM)"-1 # Rightmost" );
    ::SendDlgItemMessage(hdwnd, IDI_COMBOBOX2, CB_INSERTSTRING, 0, (LPARAM)"1 # Yes" );
    ::SendDlgItemMessage(hdwnd, IDI_COMBOBOX2, CB_INSERTSTRING, 1, (LPARAM)"0" );

    int m;
    stringstream sl(cells[1][LEFTMOST_LINE]);
    sl >> m;
    switch(m) {
    case -1: // rightmost
        ::SendDlgItemMessage(hdwnd, IDI_COMBOBOX1, CB_SETCURSEL, 2, 0);
        break;
    case 0:
        ::SendDlgItemMessage(hdwnd, IDI_COMBOBOX1, CB_SETCURSEL, 1, 0);
        break;
    default:
        ::SendDlgItemMessage(hdwnd, IDI_COMBOBOX1, CB_SETCURSEL, 0, 0);
        break;
    }

    stringstream sc(cells[1][CASE_SENSITIVE_LINE]);
    sc >> m;
    switch(m) {
    case 0:
        ::SendDlgItemMessage(hdwnd, IDI_COMBOBOX2, CB_SETCURSEL, 1, 0);
        break;
    default:
        ::SendDlgItemMessage(hdwnd, IDI_COMBOBOX2, CB_SETCURSEL, 0, 0);
        break;
    }

    return true;
}

bool VoyaSetting::writeVoyaCSV() {
    using namespace _STL;

    char *edited_line;
    char data1[SMALL_BUFFER_SIZE];
    char data2[SMALL_BUFFER_SIZE];
    char data3[SMALL_BUFFER_SIZE];
    char data4[SMALL_BUFFER_SIZE];
    char data5[SMALL_BUFFER_SIZE];
    char data6[SMALL_BUFFER_SIZE];
    int index, m;

    ::GetDlgItemText(hdwnd, IDI_EDITBOX1, data1, SMALL_BUFFER_SIZE); // Request
    ::GetDlgItemText(hdwnd, IDI_EDITBOX2, data2, SMALL_BUFFER_SIZE); // Log
    ::GetDlgItemText(hdwnd, IDI_EDITBOX3, data3, SMALL_BUFFER_SIZE); // Report
    ::GetDlgItemText(hdwnd, IDI_EDITBOX4, data4, SMALL_BUFFER_SIZE); // Idling

// Checking erroneous selection
    index = ::SendDlgItemMessage(hdwnd, IDI_COMBOBOX1, CB_GETCURSEL, 0, 0);
    if(index == CB_ERR) {
        const string COMBOBOX_ERROR("IDI_COMBOBOX1, CB_GETCURSEL, CB_ERR");
        info = COMBOBOX_ERROR;
        return false;
    } else {
        ::SendDlgItemMessage(hdwnd, IDI_COMBOBOX1, CB_GETLBTEXT, index, (LPARAM)data5); // Leftmost
        removeComment(data5);
    }

    index = ::SendDlgItemMessage(hdwnd, IDI_COMBOBOX2, CB_GETCURSEL, 0, 0);
    if(index == CB_ERR) {
        const string COMBOBOX_ERROR("IDI_COMBOBOX2, CB_GETCURSEL, CB_ERR");
        info = COMBOBOX_ERROR;
        return false;
    } else {
        ::SendDlgItemMessage(hdwnd, IDI_COMBOBOX2, CB_GETLBTEXT, index, (LPARAM)data6); // Case-sensitive
        removeComment(data6);
    }

    edited_line = & vs->edited[REQUEST_LINE][0]; lstrcpy(edited_line, data1);
    edited_line = & vs->edited[LOG_LINE][0]; lstrcpy(edited_line, data2);
    edited_line = & vs->edited[REPORT_LINE][0]; lstrcpy(edited_line, data3);
    edited_line = & vs->edited[IDLING_LINE][0]; lstrcpy(edited_line, data4);
    edited_line = & vs->edited[LEFTMOST_LINE][0]; lstrcpy(edited_line, data5);
    edited_line = & vs->edited[CASE_SENSITIVE_LINE][0]; lstrcpy(edited_line, data6);

// Checking erroneous input
   edited_line = & vs->edited[LEFTMOST_LINE][0];
    stringstream sl(edited_line);
    sl >> m;
    if(m < -1 || 1 < m) {
        const string LEFTMOST_RANGE_ERROR("edited_line = & vs->edited[LEFTMOST_LINE][0]; m < -1 || 1 < m");
        info = LEFTMOST_RANGE_ERROR;
        return false;
    }

    edited_line = & vs->edited[CASE_SENSITIVE_LINE][0];
    stringstream sc(edited_line);
    sc >> m;
    if(m < 0 || 1 < m) {
        const string RANGE_ERROR("edited_line = & vs->edited[CASE_SENSITIVE_LINE][0]; m < 0 || 1 < m");
        info = RANGE_ERROR;
        return false;
    }

    edited_line = & vs->edited[REPORT_LINE][0];
    stringstream sr(edited_line);
    sr >> m;
    if(m < 1) {
        const string REPORT_RANGE_ERROR("edited_line = & vs->edited[REPORT_LINE][0]; m < 1");
        info = REPORT_RANGE_ERROR;
        return false;
    }

    edited_line = & vs->edited[IDLING_LINE][0];
    stringstream si(edited_line);
    si >> m;
    if(m < 0) {
        const string IDLING_RANGE_ERROR("edited_line = & vs->edited[IDLING_LINE][0]; m < 0");
        info = IDLING_RANGE_ERROR;
        return false;
    }

    writeLabelsEditedLines();
    return true;
}

int commaShouldComeBeforeInteger(char *sequence, int length, int index) {
    const int COMMA_SHOULD_COME_ERROR = -2;
    int i = length;
    while(i > 0) {
        if(sequence[i] == ',') {
            break;
        }

        --i;
    }

    if(sequence[i] != ',') {
        switch(index) {
        case 0:
            thread->voyaFlush("Comma should come before the leftmost value.");
            break;
        case 1:
            thread->voyaFlush("Comma should come before the case_sensitive value.");
            break;
        case 2:
            thread->voyaFlush("Comma should come before the interim report value.");
            break;
        case 3:
            thread->voyaFlush("Comma should come before the idling value.");
            break;
        default:
            thread->voyaMessage("Comma should come before weight: ");
            thread->voyaFlush(i);
            break;
        }

        return COMMA_SHOULD_COME_ERROR;
    }

    for(;;) {
        if(sequence[i] == ',') {
            ++i;
            continue;
        } else if(sequence[i] == ' ') {
            ++i;
            continue;
        } else {
            break;
        }
    }

    lstrcpy(vs->buffer, &sequence[i]);
    i = cstring_to_integer(vs->buffer);
    return i;
}

int VoyaDetails::modalDialog() {
    int button = ::DialogBox(instance, "voyaDetailsDialog", hwnd, (DLGPROC)detailsDialogFunc);
    return button;
}

int VoyaSetting::modalDialog() {
    int button = ::DialogBox(instance, "voyaSettingDialog", hwnd, (DLGPROC)settingDialogFunc);
    return button;
}

void centering(HWND hwnd, int width, int height) {
    const int HALF_WIDTH = width/2;
    const int HALF_HEIGHT = height/2;

    const BOOL REPAINT = TRUE;
    const int MENU_HEIGHT = ::GetSystemMetrics(SM_CYMENU);
    const int SCREEN_WIDTH = ::GetSystemMetrics(SM_CXFULLSCREEN);
    int screen_height = ::GetSystemMetrics(SM_CYFULLSCREEN);
    screen_height += MENU_HEIGHT;
    const int CENTER_X = SCREEN_WIDTH/2 - HALF_WIDTH;
    const int CENTER_Y = screen_height/2 - HALF_HEIGHT;

    ::MoveWindow(hwnd, CENTER_X, CENTER_Y, width, height, REPAINT);
}

void editedLinePrint(int line, int a) {
    using namespace _STL;

    stringstream os;
    os << a;
    string s = os.str();
    char *edited_line = & vs->edited[line][0];
    strcpy(edited_line, s.c_str() );
}

void normalizing(double sum, double aim) {
    double d, rate;
    double weight[WEIGHT_ARRAY_LENGTH];
    int a[WEIGHT_ARRAY_LENGTH];

    a[0] = vs->ascii_weight;
    a[1] = vs->capital_omega_weight;
    a[2] = vs->capital_ya_weight;
    a[3] = vs->capital_z_weight;
    a[4] = vs->cap_weight;
    a[5] = vs->circle_weight;
    a[6] = vs->cup_weight;
    a[7] = vs->double_integral_weight;
    a[8] = vs->exists_weight;
    a[9] = vs->hankaku_katakana_weight;
    a[10] = vs->heisei_weight;
    a[11] = vs->hiragana_weight;
    a[12] = vs->jis_level_one_kanji_weight;
    a[13] = vs->jis_level_two_kanji_weight;
    a[14] = vs->katakana_weight;
    a[15] = vs->keisen_weight;
    a[16] = vs->nine_weight;
    a[17] = vs->omega_weight;
    a[18] = vs->paragraph_weight;
    a[19] = vs->parallel_weight;
    a[20] = vs->roman_weight;
    a[21] = vs->square_meter_weight;
    a[22] = vs->ya_weight;
    a[23] = vs->z_weight;

    for(int n = 0; n < REPEATEDLY_NORMALIZING; ++n) {
        for(int i = 0; i < WEIGHT_ARRAY_LENGTH; ++i) {
            weight[i] = (double) a[i];
        }

        rate = aim/sum;
        for(int i = 0; i < WEIGHT_ARRAY_LENGTH; ++i) {
            d = rate * weight[i];
            if(1 < d) {
                a[i] = (int)d;
            } else if(EPSILON < d) {
                a[i] = 1;
            } else {
                a[i] = 0;
            }

        }

        vs->ascii_weight = a[0];
        vs->capital_omega_weight = a[1];
        vs->capital_ya_weight = a[2];
        vs->capital_z_weight = a[3];
        vs->cap_weight = a[4];
        vs->circle_weight = a[5];
        vs->cup_weight = a[6];
        vs->double_integral_weight = a[7];
        vs->exists_weight = a[8];
        vs->hankaku_katakana_weight = a[9];
        vs->heisei_weight = a[10];
        vs->hiragana_weight = a[11];
        vs->jis_level_one_kanji_weight = a[12];
        vs->jis_level_two_kanji_weight = a[13];
        vs->katakana_weight = a[14];
        vs->keisen_weight = a[15];
        vs->nine_weight = a[16];
        vs->omega_weight = a[17];
        vs->paragraph_weight = a[18];
        vs->parallel_weight = a[19];
        vs->roman_weight = a[20];
        vs->square_meter_weight = a[21];
        vs->ya_weight = a[22];
        vs->z_weight = a[23];
        sum = shiftJisKanaAscii();
    }

    editedLinePrint(7, a[0]);
    editedLinePrint(8, a[1]);
    editedLinePrint(9, a[2]);
    editedLinePrint(10, a[3]);
    editedLinePrint(11, a[4]);
    editedLinePrint(12, a[5]);
    editedLinePrint(13, a[6]);
    editedLinePrint(14, a[7]);
    editedLinePrint(15, a[8]);
    editedLinePrint(16, a[9]);
    editedLinePrint(17, a[10]);
    editedLinePrint(18, a[11]);
    editedLinePrint(19, a[12]);
    editedLinePrint(20, a[13]);
    editedLinePrint(21, a[14]);
    editedLinePrint(22, a[15]);
    editedLinePrint(23, a[16]);
    editedLinePrint(24, a[17]);
    editedLinePrint(25, a[18]);
    editedLinePrint(26, a[19]);
    editedLinePrint(27, a[20]);
    editedLinePrint(28, a[21]);
    editedLinePrint(29, a[22]);
    editedLinePrint(30, a[23]);
}

void useLowerCaseUnlessCaseSensitive() {
    if(vs->case_sensitive) {
        vs->lower_case = false;
    } else {
        vs->lower_case = true;
        ::CharLower(vs->request);
    }
}

void useNoCharacterButASCII() {
    char data1[2] = "0";
    char data2[2] = "0";
    char data3[2] = "0";
    char data4[2] = "0";
    char data5[2] = "0";
    char data6[2] = "0";
    char data7[2] = "0";
    char data8[2] = "0";
    char data9[2] = "0";
    char data10[2] = "0";
    char data11[4] = "315"; // ASCII
    char data12[2] = "0";
    char data13[2] = "0";
    char data14[2] = "0";
    char data15[2] = "0";
    char data16[2] = "0";
    char data17[2] = "0";
    char data18[2] = "0";
    char data19[2] = "0";
    char data20[2] = "0";
    char data21[2] = "0";
    char data22[2] = "0";
    char data23[2] = "0";
    char data24[2] = "0";

    char *edited_line;
    edited_line = & vs->edited[19][0]; lstrcpy(edited_line, data1); // kanji-1
    edited_line = & vs->edited[20][0]; lstrcpy(edited_line, data2); // kanji-2
    edited_line = & vs->edited[13][0]; lstrcpy(edited_line, data3);
    edited_line = & vs->edited[17][0]; lstrcpy(edited_line, data4);
    edited_line = & vs->edited[22][0]; lstrcpy(edited_line, data5); // keisen
    edited_line = & vs->edited[26][0]; lstrcpy(edited_line, data6);
    edited_line = & vs->edited[16][0]; lstrcpy(edited_line, data7); // hankaku
    edited_line = & vs->edited[28][0]; lstrcpy(edited_line, data8);
    edited_line = & vs->edited[21][0]; lstrcpy(edited_line, data9);
    edited_line = & vs->edited[18][0]; lstrcpy(edited_line, data10); // hiragana
    edited_line = & vs->edited[7][0]; lstrcpy(edited_line, data11); // ASCII
    edited_line = & vs->edited[10][0]; lstrcpy(edited_line, data12);
    edited_line = & vs->edited[9][0]; lstrcpy(edited_line, data13);
    edited_line = & vs->edited[8][0]; lstrcpy(edited_line, data14);
    edited_line = & vs->edited[30][0]; lstrcpy(edited_line, data15);
    edited_line = & vs->edited[29][0]; lstrcpy(edited_line, data16);
    edited_line = & vs->edited[24][0]; lstrcpy(edited_line, data17);
    edited_line = & vs->edited[27][0]; lstrcpy(edited_line, data18);
    edited_line = & vs->edited[23][0]; lstrcpy(edited_line, data19);
    edited_line = & vs->edited[12][0]; lstrcpy(edited_line, data20);
    edited_line = & vs->edited[25][0]; lstrcpy(edited_line, data21);
    edited_line = & vs->edited[14][0]; lstrcpy(edited_line, data22); // roman
    edited_line = & vs->edited[15][0]; lstrcpy(edited_line, data23);
    edited_line = & vs->edited[11][0]; lstrcpy(edited_line, data24);

    writeLabelsEditedLines();
}

void VoyaDetails::normalizing(double sum, double aim) {
    char *edited_line;
    double d, rate;
    double weight[WEIGHT_ARRAY_LENGTH];
    int a[WEIGHT_ARRAY_LENGTH];

    a[0] = vs->ascii_weight;
    a[1] = vs->capital_omega_weight;
    a[2] = vs->capital_ya_weight;
    a[3] = vs->capital_z_weight;
    a[4] = vs->cap_weight;
    a[5] = vs->circle_weight;
    a[6] = vs->cup_weight;
    a[7] = vs->double_integral_weight;
    a[8] = vs->exists_weight;
    a[9] = vs->hankaku_katakana_weight;
    a[10] = vs->heisei_weight;
    a[11] = vs->hiragana_weight;
    a[12] = vs->jis_level_one_kanji_weight;
    a[13] = vs->jis_level_two_kanji_weight;
    a[14] = vs->katakana_weight;
    a[15] = vs->keisen_weight;
    a[16] = vs->nine_weight;
    a[17] = vs->omega_weight;
    a[18] = vs->paragraph_weight;
    a[19] = vs->parallel_weight;
    a[20] = vs->roman_weight;
    a[21] = vs->square_meter_weight;
    a[22] = vs->ya_weight;
    a[23] = vs->z_weight;

    for(int n = 0; n < REPEATEDLY_NORMALIZING; ++n) {
        for(int i = 0; i < WEIGHT_ARRAY_LENGTH; ++i) {
            weight[i] = (double) a[i];
        }

        rate = aim/sum;
        int line = ASCII_LINE;
        for(int i = 0; i < WEIGHT_ARRAY_LENGTH; ++i) {
            d = rate * weight[i];
            if(1 < d) {
                a[i] = (int)d;
            } else if(EPSILON < d) {
                a[i] = 1;
            } else {
                a[i] = 0;
            }

            edited_line = & vs->edited[line][0];
            wsprintf(edited_line, "%d", a[i]);
            ++line;
        }

        vs->ascii_weight = a[0];
        vs->capital_omega_weight = a[1];
        vs->capital_ya_weight = a[2];
        vs->capital_z_weight = a[3];
        vs->cap_weight = a[4];
        vs->circle_weight = a[5];
        vs->cup_weight = a[6];
        vs->double_integral_weight = a[7];
        vs->exists_weight = a[8];
        vs->hankaku_katakana_weight = a[9];
        vs->heisei_weight = a[10];
        vs->hiragana_weight = a[11];
        vs->jis_level_one_kanji_weight = a[12];
        vs->jis_level_two_kanji_weight = a[13];
        vs->katakana_weight = a[14];
        vs->keisen_weight = a[15];
        vs->nine_weight = a[16];
        vs->omega_weight = a[17];
        vs->paragraph_weight = a[18];
        vs->parallel_weight = a[19];
        vs->roman_weight = a[20];
        vs->square_meter_weight = a[21];
        vs->ya_weight = a[22];
        vs->z_weight = a[23];
        sum = shiftJisKanaAscii();
    }

    edited_line = & vs->edited[19][0];     ::SetDlgItemText(hdwnd, IDD_EDITBOX01, edited_line); // kanji-1
    edited_line = & vs->edited[20][0];     ::SetDlgItemText(hdwnd, IDD_EDITBOX02, edited_line); // kanji-2
    edited_line = & vs->edited[13][0];     ::SetDlgItemText(hdwnd, IDD_EDITBOX03, edited_line);
    edited_line = & vs->edited[17][0];     ::SetDlgItemText(hdwnd, IDD_EDITBOX04, edited_line);
    edited_line = & vs->edited[22][0];     ::SetDlgItemText(hdwnd, IDD_EDITBOX05, edited_line); // keisen
    edited_line = & vs->edited[26][0];     ::SetDlgItemText(hdwnd, IDD_EDITBOX06, edited_line);
    edited_line = & vs->edited[16][0];     ::SetDlgItemText(hdwnd, IDD_EDITBOX07, edited_line); // hankaku
    edited_line = & vs->edited[28][0];     ::SetDlgItemText(hdwnd, IDD_EDITBOX08, edited_line);
    edited_line = & vs->edited[21][0];     ::SetDlgItemText(hdwnd, IDD_EDITBOX09, edited_line);
    edited_line = & vs->edited[18][0];     ::SetDlgItemText(hdwnd, IDD_EDITBOX10, edited_line); // hiragana
    edited_line = & vs->edited[7][0];     ::SetDlgItemText(hdwnd, IDD_EDITBOX11, edited_line); // ASCII
    edited_line = & vs->edited[10][0];     ::SetDlgItemText(hdwnd, IDD_EDITBOX12, edited_line);
    edited_line = & vs->edited[9][0];     ::SetDlgItemText(hdwnd, IDD_EDITBOX13, edited_line);
    edited_line = & vs->edited[8][0];     ::SetDlgItemText(hdwnd, IDD_EDITBOX14, edited_line);
    edited_line = & vs->edited[30][0];     ::SetDlgItemText(hdwnd, IDD_EDITBOX15, edited_line);
    edited_line = & vs->edited[29][0];     ::SetDlgItemText(hdwnd, IDD_EDITBOX16, edited_line);
    edited_line = & vs->edited[24][0];     ::SetDlgItemText(hdwnd, IDD_EDITBOX17, edited_line);
    edited_line = & vs->edited[27][0];     ::SetDlgItemText(hdwnd, IDD_EDITBOX18, edited_line);
    edited_line = & vs->edited[23][0];     ::SetDlgItemText(hdwnd, IDD_EDITBOX19, edited_line);
    edited_line = & vs->edited[12][0];     ::SetDlgItemText(hdwnd, IDD_EDITBOX20, edited_line);
    edited_line = & vs->edited[25][0];     ::SetDlgItemText(hdwnd, IDD_EDITBOX21, edited_line);
    edited_line = & vs->edited[14][0];     ::SetDlgItemText(hdwnd, IDD_EDITBOX22, edited_line); // roman
    edited_line = & vs->edited[15][0];     ::SetDlgItemText(hdwnd, IDD_EDITBOX23, edited_line);
    edited_line = & vs->edited[11][0];     ::SetDlgItemText(hdwnd, IDD_EDITBOX24, edited_line);

    writeLabelsEditedLines();
}

void VoyaDetails::writeVoyaCSV() {
    char data1[SMALL_BUFFER_SIZE];
    char data2[SMALL_BUFFER_SIZE];
    char data3[SMALL_BUFFER_SIZE];
    char data4[SMALL_BUFFER_SIZE];
    char data5[SMALL_BUFFER_SIZE];
    char data6[SMALL_BUFFER_SIZE];
    char data7[SMALL_BUFFER_SIZE];
    char data8[SMALL_BUFFER_SIZE];
    char data9[SMALL_BUFFER_SIZE];
    char data10[SMALL_BUFFER_SIZE];
    char data11[SMALL_BUFFER_SIZE];
    char data12[SMALL_BUFFER_SIZE];
    char data13[SMALL_BUFFER_SIZE];
    char data14[SMALL_BUFFER_SIZE];
    char data15[SMALL_BUFFER_SIZE];
    char data16[SMALL_BUFFER_SIZE];
    char data17[SMALL_BUFFER_SIZE];
    char data18[SMALL_BUFFER_SIZE];
    char data19[SMALL_BUFFER_SIZE];
    char data20[SMALL_BUFFER_SIZE];
    char data21[SMALL_BUFFER_SIZE];
    char data22[SMALL_BUFFER_SIZE];
    char data23[SMALL_BUFFER_SIZE];
    char data24[SMALL_BUFFER_SIZE];

    ::GetDlgItemText(hdwnd, IDD_EDITBOX01, data1, SMALL_BUFFER_SIZE);
    ::GetDlgItemText(hdwnd, IDD_EDITBOX02, data2, SMALL_BUFFER_SIZE);
    ::GetDlgItemText(hdwnd, IDD_EDITBOX03, data3, SMALL_BUFFER_SIZE);
    ::GetDlgItemText(hdwnd, IDD_EDITBOX04, data4, SMALL_BUFFER_SIZE);
    ::GetDlgItemText(hdwnd, IDD_EDITBOX05, data5, SMALL_BUFFER_SIZE);
    ::GetDlgItemText(hdwnd, IDD_EDITBOX06, data6, SMALL_BUFFER_SIZE);
    ::GetDlgItemText(hdwnd, IDD_EDITBOX07, data7, SMALL_BUFFER_SIZE);
    ::GetDlgItemText(hdwnd, IDD_EDITBOX08, data8, SMALL_BUFFER_SIZE);
    ::GetDlgItemText(hdwnd, IDD_EDITBOX09, data9, SMALL_BUFFER_SIZE);
    ::GetDlgItemText(hdwnd, IDD_EDITBOX10, data10, SMALL_BUFFER_SIZE);
    ::GetDlgItemText(hdwnd, IDD_EDITBOX11, data11, SMALL_BUFFER_SIZE);
    ::GetDlgItemText(hdwnd, IDD_EDITBOX12, data12, SMALL_BUFFER_SIZE);
    ::GetDlgItemText(hdwnd, IDD_EDITBOX13, data13, SMALL_BUFFER_SIZE);
    ::GetDlgItemText(hdwnd, IDD_EDITBOX14, data14, SMALL_BUFFER_SIZE);
    ::GetDlgItemText(hdwnd, IDD_EDITBOX15, data15, SMALL_BUFFER_SIZE);
    ::GetDlgItemText(hdwnd, IDD_EDITBOX16, data16, SMALL_BUFFER_SIZE);
    ::GetDlgItemText(hdwnd, IDD_EDITBOX17, data17, SMALL_BUFFER_SIZE);
    ::GetDlgItemText(hdwnd, IDD_EDITBOX18, data18, SMALL_BUFFER_SIZE);
    ::GetDlgItemText(hdwnd, IDD_EDITBOX19, data19, SMALL_BUFFER_SIZE);
    ::GetDlgItemText(hdwnd, IDD_EDITBOX20, data20, SMALL_BUFFER_SIZE);
    ::GetDlgItemText(hdwnd, IDD_EDITBOX21, data21, SMALL_BUFFER_SIZE);
    ::GetDlgItemText(hdwnd, IDD_EDITBOX22, data22, SMALL_BUFFER_SIZE);
    ::GetDlgItemText(hdwnd, IDD_EDITBOX23, data23, SMALL_BUFFER_SIZE);
    ::GetDlgItemText(hdwnd, IDD_EDITBOX24, data24, SMALL_BUFFER_SIZE);

    char *edited_line;
    edited_line = & vs->edited[19][0]; lstrcpy(edited_line, data1); // kanji-1
    edited_line = & vs->edited[20][0]; lstrcpy(edited_line, data2); // kanji-2
    edited_line = & vs->edited[13][0]; lstrcpy(edited_line, data3);
    edited_line = & vs->edited[17][0]; lstrcpy(edited_line, data4);
    edited_line = & vs->edited[22][0]; lstrcpy(edited_line, data5); // keisen
    edited_line = & vs->edited[26][0]; lstrcpy(edited_line, data6);
    edited_line = & vs->edited[16][0]; lstrcpy(edited_line, data7); // hankaku
    edited_line = & vs->edited[28][0]; lstrcpy(edited_line, data8);
    edited_line = & vs->edited[21][0]; lstrcpy(edited_line, data9);
    edited_line = & vs->edited[18][0]; lstrcpy(edited_line, data10); // hiragana
    edited_line = & vs->edited[7][0]; lstrcpy(edited_line, data11); // ASCII
    edited_line = & vs->edited[10][0]; lstrcpy(edited_line, data12);
    edited_line = & vs->edited[9][0]; lstrcpy(edited_line, data13);
    edited_line = & vs->edited[8][0]; lstrcpy(edited_line, data14);
    edited_line = & vs->edited[30][0]; lstrcpy(edited_line, data15);
    edited_line = & vs->edited[29][0]; lstrcpy(edited_line, data16);
    edited_line = & vs->edited[24][0]; lstrcpy(edited_line, data17);
    edited_line = & vs->edited[27][0]; lstrcpy(edited_line, data18);
    edited_line = & vs->edited[23][0]; lstrcpy(edited_line, data19);
    edited_line = & vs->edited[12][0]; lstrcpy(edited_line, data20);
    edited_line = & vs->edited[25][0]; lstrcpy(edited_line, data21);
    edited_line = & vs->edited[14][0]; lstrcpy(edited_line, data22); // roman
    edited_line = & vs->edited[15][0]; lstrcpy(edited_line, data23);
    edited_line = & vs->edited[11][0]; lstrcpy(edited_line, data24);

    writeLabelsEditedLines();
}

VoyaDetails::VoyaDetails(HINSTANCE i, HWND h): INFO_LABEL_HEIGHT(25), DETAILS_HEIGHT(499), DETAILS_WIDTH(401) {
    const int BORDER = ::GetSystemMetrics(SM_CYDLGFRAME);
    const int TITLE_BAR_HEIGHT = ::GetSystemMetrics(SM_CYCAPTION);

    hwnd = h;
    instance = i;
    infoLabelHeight = INFO_LABEL_HEIGHT;
    infoLabelID = IDD_LABEL25;
    infoLabelWidth = DETAILS_WIDTH - 2*BORDER;
    infoLabelX = BORDER;
    infoLabelY = DETAILS_HEIGHT - TITLE_BAR_HEIGHT - INFO_LABEL_HEIGHT - BORDER;
    width = DETAILS_WIDTH;
    height = DETAILS_HEIGHT;
}

VoyaSetting::VoyaSetting(HINSTANCE i, HWND h): INFO_LABEL_HEIGHT(25), SETTING_HEIGHT(499), SETTING_WIDTH(401) {
    const int BORDER = ::GetSystemMetrics(SM_CYDLGFRAME);
    const int TITLE_BAR_HEIGHT = ::GetSystemMetrics(SM_CYCAPTION);

    hwnd = h;
    instance = i;
    infoLabelHeight = INFO_LABEL_HEIGHT;
    infoLabelID = IDI_LABEL7;
    infoLabelWidth = SETTING_WIDTH - 2*BORDER;
    infoLabelX = BORDER;
    infoLabelY = SETTING_HEIGHT - TITLE_BAR_HEIGHT - INFO_LABEL_HEIGHT - BORDER;
    width = SETTING_WIDTH;
    height = SETTING_HEIGHT;
}

void recovery() {
    using namespace std;

    ofstream file(VOYA_CSV);
    file << "Request,ase" << endl << "Log,voya.log" << endl << "Leftmost,0" << endl << "Case-sensitive,0" << endl << "Report,524287" << endl 
        << "Idling,400" << endl << "ASCII @,143" << endl << "Greek ,1" << endl << "Cyrillic A,1" << endl << "f,1" << endl 
        << ",1" << endl << ",1" << endl << ",1" << endl << ",1" << endl << ",1" << endl 
        << ",143" << endl << "~,1" << endl << ",1" << endl << ",1" << endl << ",1" << endl 
        << "A,1" << endl << ",1" << endl << "T,1" << endl << "Greek ,1" << endl << ",1" << endl 
        << ",1" << endl << "Roman Z,1" << endl << "Unit l,1" << endl << "Cyrillic y,1" << endl << ",1" << endl 
        << "salt," << endl << "," << endl << "," << endl << "," << endl;
}

void writeLabelsEditedLines() {
    using namespace _STL;

    char *edited_line;
    const string EMPTY("");
    int line;
    string *p = new string("");
    string cells[CSV_COLUMN_COUNT][ LABEL_COUNT + 1 ];
    cells[0][0] = EMPTY;
    cells[1][0] = EMPTY;

// Copying to produce an array
    for(line = 1; line <= LABEL_COUNT; ++line) {
        edited_line = & vs->edited[line][0];

        delete p;
        p = new string(edited_line);
        cells[1][line] = *p;
        cells[0][line] = vs->label[line];
    }

// Writing
    ofstream file(VOYA_CSV);
    for(line = 1; line <= LABEL_COUNT; ++line) {
        for(int i = 0; i < CSV_COLUMN_COUNT; ++i) {
            file << cells[i][line];

            switch(i) {
            case 0:
                file << ",";
                break;
            default:
                file << endl;
                break;
            }
        }
    }

    delete p;
}

