#include "voyaSjis.h"

extern char byte_table[FULL][FULL];
extern _STL::vector<_STL::string> tori;
extern VoyaSjis *sjis;

namespace {
    const char NEW_LINE[3] = {'\r', '\n', '\0'};
    const int NEW_LINE_SIZE = 2;
}

BOOL CALLBACK sjisDialogFunc(HWND hdwnd, UINT message, WPARAM wParam, LPARAM lParam) {
    return sjis->DialogFunc(hdwnd, message, wParam, lParam);
}

BOOL CALLBACK VoyaSjis::DialogFunc(HWND h, UINT message, WPARAM wParam, LPARAM lParam) {
    BOOL b = 0;
    hdwnd = h;

    switch(message) {
    case WM_COMMAND:
        switch( LOWORD(wParam) ) {
        case IDCANCEL:
            b = TRUE;
            EndDialog(hdwnd, 0);
            break;
        case IDS_CANCEL:
            b = TRUE;
            EndDialog(hdwnd, 0);
            break;
        case IDS_CONVERT:
            clear();
            if( writeSjis() ) {
                showRing();
            } else {
                inform();
            }

            b = TRUE;
            break;
        case IDS_SIZE:
            clear();
            logSize();
            inform();
            b = TRUE;
            break;
        case IDS_HELP:
            ::WinHelp( (HWND)wParam, HELP_FILE_NAME, HELP_CONTEXT, IDH_SJIS);
            b = TRUE;
            break;
        }

        break;
    case WM_INITDIALOG:
        ring_count = produceRingBuffer(SJIS_RING_COUNT);
        set_byte_table();
        setFileName();

        infoLabel = ::CreateWindow(
            "STATIC",
            "",
            WS_CHILD | SS_LEFT,
            infoLabelX,
            infoLabelY,
            infoLabelWidth,
            infoLabelHeight,
            hdwnd,
            (HMENU)infoLabelID,
            instance,
            NULL
        );

        ::ShowWindow(infoLabel, SW_HIDE);
        setWindowText("Sjis");
        centering();
        b = TRUE;
        break;
    }

    return b;
}

bool VoyaSjis::convertRawKeysIntoShiftJIS(const char *voya_log) {
    using namespace _STL;

    bool check[ CHARACTER_COUNT + 1 ];
    char *p;
    char buffer[BUFFER_SIZE];
    char c[BUFFER_SIZE];
    char byte_code;
    const char ABANDON[] = SJIS_ABANDON;
    const string fromTo(FROM_TO);
    int q, x, y;

    ifstream text(voya_log);
    if( text.fail() ) {
        const string FILE_ERROR("Error: ");
        const string MISSING(voya_log);
        info = FILE_ERROR + MISSING;
        return false;
    }

    ofstream binary(BINARY_FILE, ios::binary);

    string *sequence = new string("");
    while( text.getline(c, BUFFER_SIZE) ) {
        delete sequence;
        sequence = new string(c);
        q = sequence->find(fromTo);
        if(q == string::npos) { // FROM_TO is not found.
            continue;
        }

/*
voya.log includes "raw key -> tori" sequences.
D3CDC4DBA9D8A49C -> Amin//dS0M
E4BB98E7905F96F1 -> Ruda/tI7a2
*/
        p = &c[0];
        for(q = 0; q < CHARACTER_COUNT; q++) {
            x = (int)p[0];
            y = (int)p[1];
            byte_code = byte_table[x][y];
            buffer[q] = byte_code;
            check[q] = testSalt(byte_code);

            p += 2;
        }

        buffer[CHARACTER_COUNT] = '\0';
        binary << buffer;

        if( zenkakuFragment(buffer) ) {
            binary << ABANDON;
            continue;
        } else {
            if(*p == ' ') {
/* If raw-key length is sixteen, voya uses ".." as salt; however 
the server may use the second and third bytes (buffer[1] and buffer[2]) as salt. */
                if(check[1] || check[2]) {
                    binary << ABANDON;
                    continue;
                }
            } else {
/*  If raw-key length is eighteen, the second and third bytes (buffer[1] and buffer[2]) are used as salt.
bc4e595b2a76407bNY -> Shizuoka8I 
N (4Eh)
Y (59h)
*/
                p += 2;
            }

        }

        binary << p << NEW_LINE;
    }

    delete sequence;
    return true;
}

bool VoyaSjis::writeSjis() {
    using namespace _STL;

    bool b = true;
    char fileName[SMALL_BUFFER_SIZE];

    ::GetDlgItemText(hdwnd, IDS_EDITBOX, fileName, SMALL_BUFFER_SIZE);
    if( convertRawKeysIntoShiftJIS(fileName) ) {
        seekEndReverse();
    } else {
        b = false;
    }

    return b;
}

int VoyaSjis::modalDialog() {
    int button = ::DialogBox(instance, "voyaSjisDialog", hwnd, (DLGPROC)sjisDialogFunc);
    return button;
}

int VoyaSjis::produceRingBuffer(int size) {
    using namespace _STL;

    sjisRing.clear();
    for(int i = 0; i < size; i += 1) {
        sjisRing.push_back(BLANK);
    }

    return sjisRing.size();
}

long VoyaSjis::logSize() {
    using namespace _STL;

    char fileName[SMALL_BUFFER_SIZE];
    const string GAP(" ");
    ::GetDlgItemText(hdwnd, IDS_EDITBOX, fileName, SMALL_BUFFER_SIZE);
    ifstream in(fileName, ios::binary);
    if( in.fail() ) {
        const string FILE_ERROR("Error: ");
        const string MISSING(fileName);
        info = FILE_ERROR + MISSING;
        return 0L;
    }

    in.seekg(0, ios::end);
    streampos length = in.tellg();
    long size = length;
    string BYTES("bytes");
    if(size >= 100000L) {
        const string KB("KB");
        double d = (double)size;
        d = d/1024.;
        d += 0.5;
        size = (long)floor(d);
        BYTES = KB;
    }

    stringstream ss;
    ss << size << GAP << BYTES;
    info = ss.str();
    return size;
}

void VoyaSjis::seekEndReverse() {
    using namespace _STL;

    bool full;
    char buffer[BUFFER_SIZE];
    ifstream in(BINARY_FILE, ios::binary);
    in.seekg(0, ios::end);
    streampos length = in.tellg();
    int reverse = -1*REVERSE;
    if(reverse < -1*length) {
        full = false;
        in.seekg(0, ios::beg);
        in.read(buffer, length);
        buffer[length] = '\0';
    } else {
        full = true;
        in.seekg(reverse, ios::end);
        in.read(buffer, REVERSE);
        buffer[REVERSE] = '\0';
    }

    const string newLine(NEW_LINE);
    int q;
    string head;
    string tail(&buffer[0]);
    if(full) {
        q = tail.find(newLine);
        if(q != string::npos) {
            head = tail.substr(0, q);
            q += NEW_LINE_SIZE;
            tail = tail.substr(q);
        }
    }

// ring buffer
    for(;;) {
        q = tail.find(newLine);
        if(q != string::npos) {
            head = tail.substr(0, q);
            q += NEW_LINE_SIZE;
            tail = tail.substr(q);

            sjisRing.pop_front();
            sjisRing.push_back(head);
        } else {
            break;
        }
    }
}

void VoyaSjis::setFileName() {
    using namespace _STL;

    bool tori_file_open = false;
    char c[BUFFER_SIZE];
    char toriFileName[SMALL_BUFFER_SIZE];
    const int CURRENT_DIRECTORY_NAME_SIZE = 2; // ".\"
    const string fromTo(FROM_TO);
    ifstream *text = new ifstream(VOYA_LOG);
    int q;
    vector<string>::reverse_iterator i = tori.rbegin();
    string *sequence = new string("");
    string s;
    while( i != tori.rend() ) {
        s = *i;
        s = s.substr(CURRENT_DIRECTORY_NAME_SIZE);
        lstrcpy(toriFileName, s.c_str() );

        delete text;
        text = new ifstream(toriFileName);
        if(! text->fail() ) {
            while( text->getline(c, BUFFER_SIZE) ) {
                delete sequence;
                sequence = new string(c);

                q = sequence->find(fromTo);
                if(q != string::npos) {
                    tori_file_open = true;
                    break;
                }
            }

            text->close();
        }

        if(tori_file_open) {
            break;
        }

        ++i;
    }

    if(tori_file_open) {
        ::SetDlgItemText(hdwnd, IDS_EDITBOX, toriFileName);
    } else {
        ::SetDlgItemText(hdwnd, IDS_EDITBOX, VOYA_LOG);
    }

    delete sequence;
    delete text;
}

void VoyaSjis::showRing() {
    using namespace _STL;

    int m = 0;
    string head;
    for(RING::iterator i = sjisRing.begin(); i != sjisRing.end(); ++i) {
        head = *i;
        if(head != BLANK) {
            textBox[m] = head;
            ++m;
        }
    }

    while(m < SJIS_RING_COUNT) {
        textBox[m] = BLANK;
        ++m;
    }

    ::SetDlgItemText(hdwnd, IDS_LABEL01, textBox[0].c_str() );
    ::SetDlgItemText(hdwnd, IDS_LABEL02, textBox[1].c_str() );
    ::SetDlgItemText(hdwnd, IDS_LABEL03, textBox[2].c_str() );
    ::SetDlgItemText(hdwnd, IDS_LABEL04, textBox[3].c_str() );
    ::SetDlgItemText(hdwnd, IDS_LABEL05, textBox[4].c_str() );
    ::SetDlgItemText(hdwnd, IDS_LABEL06, textBox[5].c_str() );
    ::SetDlgItemText(hdwnd, IDS_LABEL07, textBox[6].c_str() );
    ::SetDlgItemText(hdwnd, IDS_LABEL08, textBox[7].c_str() );
    ::SetDlgItemText(hdwnd, IDS_LABEL09, textBox[8].c_str() );
    ::SetDlgItemText(hdwnd, IDS_LABEL10, textBox[9].c_str() );
    ::SetDlgItemText(hdwnd, IDS_LABEL11, textBox[10].c_str() );
    ::SetDlgItemText(hdwnd, IDS_LABEL12, textBox[11].c_str() );
    ::SetDlgItemText(hdwnd, IDS_LABEL13, textBox[12].c_str() );
    ::SetDlgItemText(hdwnd, IDS_LABEL14, textBox[13].c_str() );
    ::SetDlgItemText(hdwnd, IDS_LABEL15, textBox[14].c_str() );
    ::SetDlgItemText(hdwnd, IDS_LABEL16, textBox[15].c_str() );
    ::SetDlgItemText(hdwnd, IDS_LABEL17, textBox[16].c_str() );
    ::SetDlgItemText(hdwnd, IDS_LABEL18, textBox[17].c_str() );
    ::SetDlgItemText(hdwnd, IDS_LABEL19, textBox[18].c_str() );
    ::SetDlgItemText(hdwnd, IDS_LABEL20, textBox[19].c_str() );
    ::SetDlgItemText(hdwnd, IDS_LABEL21, textBox[20].c_str() );
    ::SetDlgItemText(hdwnd, IDS_LABEL22, textBox[21].c_str() );
}

bool zenkakuFragment(char *buffer) {
// Checking if the line ends with a separated byte of kanji. 
    bool zenkaku = false;
    char line[BUFFER_SIZE];
    int c, k;

    strcpy(line, buffer);
    for(k = 0; line[k] != '\0'; k++) {
        c = charToInteger(line[k]);

        if(zenkaku) {
            zenkaku = false;
            continue;
        }

        if(0x81 <= c && c <= 0xFC) {
            if(KANA_KUTEN <= c && c <= 0xDF) { // 
                zenkaku = false;
            } else {
                zenkaku = TRUE;
            }

        } else {
            zenkaku = false;
        }
    }

    return zenkaku;
}

VoyaSjis::VoyaSjis(HINSTANCE i, HWND h): BLANK(" "), INFO_LABEL_HEIGHT(25), SJIS_HEIGHT(499), SJIS_WIDTH(401) {
    const int BORDER = ::GetSystemMetrics(SM_CYDLGFRAME);
    const int TITLE_BAR_HEIGHT = ::GetSystemMetrics(SM_CYCAPTION);

    hwnd = h;
    instance = i;
    infoLabelHeight = INFO_LABEL_HEIGHT;
    infoLabelID = IDS_LABEL;
    infoLabelWidth = SJIS_WIDTH - 2*BORDER;
    infoLabelX = BORDER;
    infoLabelY = SJIS_HEIGHT - TITLE_BAR_HEIGHT - INFO_LABEL_HEIGHT - BORDER;
    width = SJIS_WIDTH;
    height = SJIS_HEIGHT;
}

