#include "voyaTest.h"

char byte_table[FULL][FULL];
extern VoyaTest *test;

BOOL CALLBACK testDialogFunc(HWND hdwnd, UINT message, WPARAM wParam, LPARAM lParam) {
    return test->DialogFunc(hdwnd, message, wParam, lParam);
}

BOOL CALLBACK VoyaTest::DialogFunc(HWND h, UINT message, WPARAM wParam, LPARAM lParam) {
    BOOL b = 0;
    hdwnd = h;

    switch(message) {
    case WM_COMMAND:
        switch( LOWORD(wParam) ) {
        case IDCANCEL:
            readVoyaTestText();
            b = TRUE;
            ::EndDialog(hdwnd, 0);
            break;
        case IDT_CANCEL:
            readVoyaTestText();
            b = TRUE;
            ::EndDialog(hdwnd, 0);
            break;
        case IDT_HELP:
            ::WinHelp( (HWND)wParam, HELP_FILE_NAME, HELP_CONTEXT, IDH_TEST);
            b = TRUE;
            break;
        case IDT_TEST:
            clear();
            if( writeVoyaTestText() ) {
                readVoyaTestText();
                testResults();
            }

            inform();
            b = TRUE;
            break;
        }

        break;
    case WM_INITDIALOG:
        set_byte_table();
        readVoyaTestText();

        infoLabel = ::CreateWindow(
            "STATIC",
            "",
            WS_CHILD | SS_LEFT,
            infoLabelX,
            infoLabelY,
            infoLabelWidth,
            infoLabelHeight,
            hdwnd,
            (HMENU)infoLabelID,
            instance,
            NULL
        );

        ::ShowWindow(infoLabel, SW_HIDE);
        setWindowText("Test");
        centering();
        b = TRUE;
        break;
    }

    return b;
}

bool VoyaTest::testHexadecimal(_STL::string & s) {
    using namespace _STL;

    bool b = true;
    const int HEXADECIMAL_COUNT = 16;
    const int HEXADECIMAL_COUNT_PLUS = 17;
    const string H("0123456789ABCDEFabcdef");
    int i, q;
    int s_length = s.length();
    string g;
    for(i = 0; i < s_length; ++i) {
        g = s.substr(i, 1);
        q = H.find(g);
        if(q == string::npos) {
            if(i < HEXADECIMAL_COUNT) {
                b = false;
                break;
            } else if(i == HEXADECIMAL_COUNT || i == HEXADECIMAL_COUNT_PLUS) {
                if( testSalt(g) ) {
                    break;
                }

                b = false;
                break;
            } else {
                b = false;
                break;
            }
        }
    }

    return b;
}

bool VoyaTest::testResults() {
    using namespace _STL;

    const int KEY_SIZE = 2 * CHARACTER_COUNT;
    const int KEY_SALT_SIZE = KEY_SIZE + SALT_SIZE;
    const int SEPARATOR_SIZE = SEPARATOR.length();
    char *p;
    char key[ KEY_SALT_SIZE + 1 ];
    char pass[ CHARACTER_COUNT + 1 ];
    char salt[ SALT_SIZE + 1 ];
    char tori[ TEN_FIGURE_TORI + 1 ];
    int higher_nibble, i, keyLength, lower_nibble, q, r, s, u;
    string head, tail;
    stringstream sl;
    for(i = 0; i < TEXT_BOX_ROW_COUNT; ++i) {
        q = info.find(SEPARATOR);
        if(q == string::npos) {
            break;
        } else if(q == 0) {
            break;
        }

        head = info.substr(0, q);
        q += SEPARATOR_SIZE;
        tail = info.substr(q);
        info = tail; // q = info.find(SEPARATOR);

        keyLength = head.length();
        if(keyLength > KEY_SALT_SIZE) {
            const string KEY_SALT_SIZE_ERROR("Error: KEY_SALT_SIZE");
            info = KEY_SALT_SIZE_ERROR;
            return false;
        } else if(keyLength < KEY_SIZE) {
            const string KEY_SIZE_ERROR("Error: KEY_SIZE");
            info = KEY_SIZE_ERROR;
            return false;
        }

        ::ZeroMemory(key, KEY_SALT_SIZE + 1);
        lstrcpy(key, head.c_str() );
        for(u = 0; u < CHARACTER_COUNT; ++u) {
            r = 2*u;
            higher_nibble = key[r];
            lower_nibble = key[ r + 1 ];
            pass[u] = byte_table[higher_nibble][lower_nibble];
        }

        pass[CHARACTER_COUNT] = '\0';
        r = KEY_SIZE;
        if(keyLength == KEY_SIZE) {
            lstrcpy(salt, "..");
        } else if( keyLength == KEY_SIZE + 1 ) {
            salt[0] = key[r];
            salt[1] = '.';
            salt[2] = '\0';
        } else if(keyLength == KEY_SALT_SIZE) {
            salt[0] = key[r];
            salt[1] = key[ r + 1 ];
            salt[2] = '\0';
        }

// Encrypting to produce a tori
        p = crypt3(pass, salt);
        s = lstrlen(p) - TEN_FIGURE_TORI;
        lstrcpy(tori, &p[s]);
        sl << tori << PARTITION;
    }

    string text = sl.str();

    q = text.rfind(PIPE);
    if(q != string::npos) {
// Removing the last pipe
        info = text.substr(0, q);
    } else {
        info = text;
    }

    return true;
}

bool VoyaTest::writeVoyaTestText() {
    using namespace _STL;

    char data1[SMALL_BUFFER_SIZE];
    char data2[SMALL_BUFFER_SIZE];
    char data3[SMALL_BUFFER_SIZE];
    const string HEXADECIMAL_ERROR("Please enter 16 hexadecimal figures. ");

    GetDlgItemText(hdwnd, IDT_EDITBOX1, data1, SMALL_BUFFER_SIZE);
    GetDlgItemText(hdwnd, IDT_EDITBOX2, data2, SMALL_BUFFER_SIZE);
    GetDlgItemText(hdwnd, IDT_EDITBOX3, data3, SMALL_BUFFER_SIZE);
    string d1(data1);
    if(! d1.empty() ) {
        if(! testHexadecimal(d1) ) {
            info = HEXADECIMAL_ERROR + d1;
            return false;
        }
    }

    string d2(data2);
    if(! d2.empty() ) {
        if(! testHexadecimal(d2) ) {
            info = HEXADECIMAL_ERROR + d2;
            return false;
        }
    }

    string d3(data3);
    if(! d3.empty() ) {
        if(! testHexadecimal(d3) ) {
            info = HEXADECIMAL_ERROR + d3;
            return false;
        }
    }

    info = d1 + SEPARATOR + d2 + SEPARATOR + d3 + SEPARATOR;
    ofstream file(VOYA_TEST_TEXT);
    file << d1 << endl << d2 << endl << d3 << endl;
    return true;
}

int VoyaTest::modalDialog() {
    int button = ::DialogBox(instance, "voyaTestDialog", hwnd, (DLGPROC)testDialogFunc);
    return button;
}

void set_byte_table() {
    ::FillMemory(byte_table, FULL*FULL, 0x20);
    byte_table['0']['0'] = 0x00;
    byte_table['0']['1'] = 0x01;
    byte_table['0']['2'] = 0x02;
    byte_table['0']['3'] = 0x03;
    byte_table['0']['4'] = 0x04;
    byte_table['0']['5'] = 0x05;
    byte_table['0']['6'] = 0x06;
    byte_table['0']['7'] = 0x07;
    byte_table['0']['8'] = 0x08;
    byte_table['0']['9'] = 0x09;
    byte_table['0']['A'] = 0x0A;
    byte_table['0']['B'] = 0x0B;
    byte_table['0']['C'] = 0x0C;
    byte_table['0']['D'] = 0x0D;
    byte_table['0']['E'] = 0x0E;
    byte_table['0']['F'] = 0x0F;
    byte_table['1']['0'] = 0x10;
    byte_table['1']['1'] = 0x11;
    byte_table['1']['2'] = 0x12;
    byte_table['1']['3'] = 0x13;
    byte_table['1']['4'] = 0x14;
    byte_table['1']['5'] = 0x15;
    byte_table['1']['6'] = 0x16;
    byte_table['1']['7'] = 0x17;
    byte_table['1']['8'] = 0x18;
    byte_table['1']['9'] = 0x19;
    byte_table['1']['A'] = 0x1A;
    byte_table['1']['B'] = 0x1B;
    byte_table['1']['C'] = 0x1C;
    byte_table['1']['D'] = 0x1D;
    byte_table['1']['E'] = 0x1E;
    byte_table['1']['F'] = 0x1F;
    byte_table['2']['0'] = 0x20;
    byte_table['2']['1'] = 0x21;
    byte_table['2']['2'] = 0x22;
    byte_table['2']['3'] = 0x23;
    byte_table['2']['4'] = 0x24;
    byte_table['2']['5'] = 0x25;
    byte_table['2']['6'] = 0x26;
    byte_table['2']['7'] = 0x27;
    byte_table['2']['8'] = 0x28;
    byte_table['2']['9'] = 0x29;
    byte_table['2']['A'] = 0x2A;
    byte_table['2']['B'] = 0x2B;
    byte_table['2']['C'] = 0x2C;
    byte_table['2']['D'] = 0x2D;
    byte_table['2']['E'] = 0x2E;
    byte_table['2']['F'] = 0x2F;
    byte_table['3']['0'] = 0x30;
    byte_table['3']['1'] = 0x31;
    byte_table['3']['2'] = 0x32;
    byte_table['3']['3'] = 0x33;
    byte_table['3']['4'] = 0x34;
    byte_table['3']['5'] = 0x35;
    byte_table['3']['6'] = 0x36;
    byte_table['3']['7'] = 0x37;
    byte_table['3']['8'] = 0x38;
    byte_table['3']['9'] = 0x39;
    byte_table['3']['A'] = 0x3A;
    byte_table['3']['B'] = 0x3B;
    byte_table['3']['C'] = 0x3C;
    byte_table['3']['D'] = 0x3D;
    byte_table['3']['E'] = 0x3E;
    byte_table['3']['F'] = 0x3F;
    byte_table['4']['0'] = 0x40;
    byte_table['4']['1'] = 0x41;
    byte_table['4']['2'] = 0x42;
    byte_table['4']['3'] = 0x43;
    byte_table['4']['4'] = 0x44;
    byte_table['4']['5'] = 0x45;
    byte_table['4']['6'] = 0x46;
    byte_table['4']['7'] = 0x47;
    byte_table['4']['8'] = 0x48;
    byte_table['4']['9'] = 0x49;
    byte_table['4']['A'] = 0x4A;
    byte_table['4']['B'] = 0x4B;
    byte_table['4']['C'] = 0x4C;
    byte_table['4']['D'] = 0x4D;
    byte_table['4']['E'] = 0x4E;
    byte_table['4']['F'] = 0x4F;
    byte_table['5']['0'] = 0x50;
    byte_table['5']['1'] = 0x51;
    byte_table['5']['2'] = 0x52;
    byte_table['5']['3'] = 0x53;
    byte_table['5']['4'] = 0x54;
    byte_table['5']['5'] = 0x55;
    byte_table['5']['6'] = 0x56;
    byte_table['5']['7'] = 0x57;
    byte_table['5']['8'] = 0x58;
    byte_table['5']['9'] = 0x59;
    byte_table['5']['A'] = 0x5A;
    byte_table['5']['B'] = 0x5B;
    byte_table['5']['C'] = 0x5C;
    byte_table['5']['D'] = 0x5D;
    byte_table['5']['E'] = 0x5E;
    byte_table['5']['F'] = 0x5F;
    byte_table['6']['0'] = 0x60;
    byte_table['6']['1'] = 0x61;
    byte_table['6']['2'] = 0x62;
    byte_table['6']['3'] = 0x63;
    byte_table['6']['4'] = 0x64;
    byte_table['6']['5'] = 0x65;
    byte_table['6']['6'] = 0x66;
    byte_table['6']['7'] = 0x67;
    byte_table['6']['8'] = 0x68;
    byte_table['6']['9'] = 0x69;
    byte_table['6']['A'] = 0x6A;
    byte_table['6']['B'] = 0x6B;
    byte_table['6']['C'] = 0x6C;
    byte_table['6']['D'] = 0x6D;
    byte_table['6']['E'] = 0x6E;
    byte_table['6']['F'] = 0x6F;
    byte_table['7']['0'] = 0x70;
    byte_table['7']['1'] = 0x71;
    byte_table['7']['2'] = 0x72;
    byte_table['7']['3'] = 0x73;
    byte_table['7']['4'] = 0x74;
    byte_table['7']['5'] = 0x75;
    byte_table['7']['6'] = 0x76;
    byte_table['7']['7'] = 0x77;
    byte_table['7']['8'] = 0x78;
    byte_table['7']['9'] = 0x79;
    byte_table['7']['A'] = 0x7A;
    byte_table['7']['B'] = 0x7B;
    byte_table['7']['C'] = 0x7C;
    byte_table['7']['D'] = 0x7D;
    byte_table['7']['E'] = 0x7E;
    byte_table['7']['F'] = 0x7F;
    byte_table['8']['0'] = 0x0080;
    byte_table['8']['1'] = 0x0081;
    byte_table['8']['2'] = 0x0082;
    byte_table['8']['3'] = 0x0083;
    byte_table['8']['4'] = 0x0084;
    byte_table['8']['5'] = 0x0085;
    byte_table['8']['6'] = 0x0086;
    byte_table['8']['7'] = 0x0087;
    byte_table['8']['8'] = 0x0088;
    byte_table['8']['9'] = 0x0089;
    byte_table['8']['A'] = 0x008A;
    byte_table['8']['B'] = 0x008B;
    byte_table['8']['C'] = 0x008C;
    byte_table['8']['D'] = 0x008D;
    byte_table['8']['E'] = 0x008E;
    byte_table['8']['F'] = 0x008F;
    byte_table['9']['0'] = 0x0090;
    byte_table['9']['1'] = 0x0091;
    byte_table['9']['2'] = 0x0092;
    byte_table['9']['3'] = 0x0093;
    byte_table['9']['4'] = 0x0094;
    byte_table['9']['5'] = 0x0095;
    byte_table['9']['6'] = 0x0096;
    byte_table['9']['7'] = 0x0097;
    byte_table['9']['8'] = 0x0098;
    byte_table['9']['9'] = 0x0099;
    byte_table['9']['A'] = 0x009A;
    byte_table['9']['B'] = 0x009B;
    byte_table['9']['C'] = 0x009C;
    byte_table['9']['D'] = 0x009D;
    byte_table['9']['E'] = 0x009E;
    byte_table['9']['F'] = 0x009F;
    byte_table['A']['0'] = 0x00A0;
    byte_table['A']['1'] = 0x00A1;
    byte_table['A']['2'] = 0x00A2;
    byte_table['A']['3'] = 0x00A3;
    byte_table['A']['4'] = 0x00A4;
    byte_table['A']['5'] = 0x00A5;
    byte_table['A']['6'] = 0x00A6;
    byte_table['A']['7'] = 0x00A7;
    byte_table['A']['8'] = 0x00A8;
    byte_table['A']['9'] = 0x00A9;
    byte_table['A']['A'] = 0x00AA;
    byte_table['A']['B'] = 0x00AB;
    byte_table['A']['C'] = 0x00AC;
    byte_table['A']['D'] = 0x00AD;
    byte_table['A']['E'] = 0x00AE;
    byte_table['A']['F'] = 0x00AF;
    byte_table['B']['0'] = 0x00B0;
    byte_table['B']['1'] = 0x00B1;
    byte_table['B']['2'] = 0x00B2;
    byte_table['B']['3'] = 0x00B3;
    byte_table['B']['4'] = 0x00B4;
    byte_table['B']['5'] = 0x00B5;
    byte_table['B']['6'] = 0x00B6;
    byte_table['B']['7'] = 0x00B7;
    byte_table['B']['8'] = 0x00B8;
    byte_table['B']['9'] = 0x00B9;
    byte_table['B']['A'] = 0x00BA;
    byte_table['B']['B'] = 0x00BB;
    byte_table['B']['C'] = 0x00BC;
    byte_table['B']['D'] = 0x00BD;
    byte_table['B']['E'] = 0x00BE;
    byte_table['B']['F'] = 0x00BF;
    byte_table['C']['0'] = 0x00C0;
    byte_table['C']['1'] = 0x00C1;
    byte_table['C']['2'] = 0x00C2;
    byte_table['C']['3'] = 0x00C3;
    byte_table['C']['4'] = 0x00C4;
    byte_table['C']['5'] = 0x00C5;
    byte_table['C']['6'] = 0x00C6;
    byte_table['C']['7'] = 0x00C7;
    byte_table['C']['8'] = 0x00C8;
    byte_table['C']['9'] = 0x00C9;
    byte_table['C']['A'] = 0x00CA;
    byte_table['C']['B'] = 0x00CB;
    byte_table['C']['C'] = 0x00CC;
    byte_table['C']['D'] = 0x00CD;
    byte_table['C']['E'] = 0x00CE;
    byte_table['C']['F'] = 0x00CF;
    byte_table['D']['0'] = 0x00D0;
    byte_table['D']['1'] = 0x00D1;
    byte_table['D']['2'] = 0x00D2;
    byte_table['D']['3'] = 0x00D3;
    byte_table['D']['4'] = 0x00D4;
    byte_table['D']['5'] = 0x00D5;
    byte_table['D']['6'] = 0x00D6;
    byte_table['D']['7'] = 0x00D7;
    byte_table['D']['8'] = 0x00D8;
    byte_table['D']['9'] = 0x00D9;
    byte_table['D']['A'] = 0x00DA;
    byte_table['D']['B'] = 0x00DB;
    byte_table['D']['C'] = 0x00DC;
    byte_table['D']['D'] = 0x00DD;
    byte_table['D']['E'] = 0x00DE;
    byte_table['D']['F'] = 0x00DF;
    byte_table['E']['0'] = 0x00E0;
    byte_table['E']['1'] = 0x00E1;
    byte_table['E']['2'] = 0x00E2;
    byte_table['E']['3'] = 0x00E3;
    byte_table['E']['4'] = 0x00E4;
    byte_table['E']['5'] = 0x00E5;
    byte_table['E']['6'] = 0x00E6;
    byte_table['E']['7'] = 0x00E7;
    byte_table['E']['8'] = 0x00E8;
    byte_table['E']['9'] = 0x00E9;
    byte_table['E']['A'] = 0x00EA;
    byte_table['E']['B'] = 0x00EB;
    byte_table['E']['C'] = 0x00EC;
    byte_table['E']['D'] = 0x00ED;
    byte_table['E']['E'] = 0x00EE;
    byte_table['E']['F'] = 0x00EF;
    byte_table['F']['0'] = 0x00F0;
    byte_table['F']['1'] = 0x00F1;
    byte_table['F']['2'] = 0x00F2;
    byte_table['F']['3'] = 0x00F3;
    byte_table['F']['4'] = 0x00F4;
    byte_table['F']['5'] = 0x00F5;
    byte_table['F']['6'] = 0x00F6;
    byte_table['F']['7'] = 0x00F7;
    byte_table['F']['8'] = 0x00F8;
    byte_table['F']['9'] = 0x00F9;
    byte_table['F']['A'] = 0x00FA;
    byte_table['F']['B'] = 0x00FB;
    byte_table['F']['C'] = 0x00FC;
    byte_table['F']['D'] = 0x00FD;
    byte_table['F']['E'] = 0x00FE;
    byte_table['F']['F'] = 0x00FF;
    byte_table['0']['a'] = 0x0a;
    byte_table['0']['b'] = 0x0b;
    byte_table['0']['c'] = 0x0c;
    byte_table['0']['d'] = 0x0d;
    byte_table['0']['e'] = 0x0e;
    byte_table['0']['f'] = 0x0f;
    byte_table['1']['a'] = 0x1a;
    byte_table['1']['b'] = 0x1b;
    byte_table['1']['c'] = 0x1c;
    byte_table['1']['d'] = 0x1d;
    byte_table['1']['e'] = 0x1e;
    byte_table['1']['f'] = 0x1f;
    byte_table['2']['a'] = 0x2a;
    byte_table['2']['b'] = 0x2b;
    byte_table['2']['c'] = 0x2c;
    byte_table['2']['d'] = 0x2d;
    byte_table['2']['e'] = 0x2e;
    byte_table['2']['f'] = 0x2f;
    byte_table['3']['a'] = 0x3a;
    byte_table['3']['b'] = 0x3b;
    byte_table['3']['c'] = 0x3c;
    byte_table['3']['d'] = 0x3d;
    byte_table['3']['e'] = 0x3e;
    byte_table['3']['f'] = 0x3f;
    byte_table['4']['a'] = 0x4a;
    byte_table['4']['b'] = 0x4b;
    byte_table['4']['c'] = 0x4c;
    byte_table['4']['d'] = 0x4d;
    byte_table['4']['e'] = 0x4e;
    byte_table['4']['f'] = 0x4f;
    byte_table['5']['a'] = 0x5a;
    byte_table['5']['b'] = 0x5b;
    byte_table['5']['c'] = 0x5c;
    byte_table['5']['d'] = 0x5d;
    byte_table['5']['e'] = 0x5e;
    byte_table['5']['f'] = 0x5f;
    byte_table['6']['a'] = 0x6a;
    byte_table['6']['b'] = 0x6b;
    byte_table['6']['c'] = 0x6c;
    byte_table['6']['d'] = 0x6d;
    byte_table['6']['e'] = 0x6e;
    byte_table['6']['f'] = 0x6f;
    byte_table['7']['a'] = 0x7a;
    byte_table['7']['b'] = 0x7b;
    byte_table['7']['c'] = 0x7c;
    byte_table['7']['d'] = 0x7d;
    byte_table['7']['e'] = 0x7e;
    byte_table['7']['f'] = 0x7f;
    byte_table['8']['a'] = 0x008a;
    byte_table['8']['b'] = 0x008b;
    byte_table['8']['c'] = 0x008c;
    byte_table['8']['d'] = 0x008d;
    byte_table['8']['e'] = 0x008e;
    byte_table['8']['f'] = 0x008f;
    byte_table['9']['a'] = 0x009a;
    byte_table['9']['b'] = 0x009b;
    byte_table['9']['c'] = 0x009c;
    byte_table['9']['d'] = 0x009d;
    byte_table['9']['e'] = 0x009e;
    byte_table['9']['f'] = 0x009f;
    byte_table['a']['0'] = 0x00a0;
    byte_table['a']['1'] = 0x00a1;
    byte_table['a']['2'] = 0x00a2;
    byte_table['a']['3'] = 0x00a3;
    byte_table['a']['4'] = 0x00a4;
    byte_table['a']['5'] = 0x00a5;
    byte_table['a']['6'] = 0x00a6;
    byte_table['a']['7'] = 0x00a7;
    byte_table['a']['8'] = 0x00a8;
    byte_table['a']['9'] = 0x00a9;
    byte_table['a']['a'] = 0x00aa;
    byte_table['a']['b'] = 0x00ab;
    byte_table['a']['c'] = 0x00ac;
    byte_table['a']['d'] = 0x00ad;
    byte_table['a']['e'] = 0x00ae;
    byte_table['a']['f'] = 0x00af;
    byte_table['b']['0'] = 0x00b0;
    byte_table['b']['1'] = 0x00b1;
    byte_table['b']['2'] = 0x00b2;
    byte_table['b']['3'] = 0x00b3;
    byte_table['b']['4'] = 0x00b4;
    byte_table['b']['5'] = 0x00b5;
    byte_table['b']['6'] = 0x00b6;
    byte_table['b']['7'] = 0x00b7;
    byte_table['b']['8'] = 0x00b8;
    byte_table['b']['9'] = 0x00b9;
    byte_table['b']['a'] = 0x00ba;
    byte_table['b']['b'] = 0x00bb;
    byte_table['b']['c'] = 0x00bc;
    byte_table['b']['d'] = 0x00bd;
    byte_table['b']['e'] = 0x00be;
    byte_table['b']['f'] = 0x00bf;
    byte_table['c']['0'] = 0x00c0;
    byte_table['c']['1'] = 0x00c1;
    byte_table['c']['2'] = 0x00c2;
    byte_table['c']['3'] = 0x00c3;
    byte_table['c']['4'] = 0x00c4;
    byte_table['c']['5'] = 0x00c5;
    byte_table['c']['6'] = 0x00c6;
    byte_table['c']['7'] = 0x00c7;
    byte_table['c']['8'] = 0x00c8;
    byte_table['c']['9'] = 0x00c9;
    byte_table['c']['a'] = 0x00ca;
    byte_table['c']['b'] = 0x00cb;
    byte_table['c']['c'] = 0x00cc;
    byte_table['c']['d'] = 0x00cd;
    byte_table['c']['e'] = 0x00ce;
    byte_table['c']['f'] = 0x00cf;
    byte_table['d']['0'] = 0x00d0;
    byte_table['d']['1'] = 0x00d1;
    byte_table['d']['2'] = 0x00d2;
    byte_table['d']['3'] = 0x00d3;
    byte_table['d']['4'] = 0x00d4;
    byte_table['d']['5'] = 0x00d5;
    byte_table['d']['6'] = 0x00d6;
    byte_table['d']['7'] = 0x00d7;
    byte_table['d']['8'] = 0x00d8;
    byte_table['d']['9'] = 0x00d9;
    byte_table['d']['a'] = 0x00da;
    byte_table['d']['b'] = 0x00db;
    byte_table['d']['c'] = 0x00dc;
    byte_table['d']['d'] = 0x00dd;
    byte_table['d']['e'] = 0x00de;
    byte_table['d']['f'] = 0x00df;
    byte_table['e']['0'] = 0x00e0;
    byte_table['e']['1'] = 0x00e1;
    byte_table['e']['2'] = 0x00e2;
    byte_table['e']['3'] = 0x00e3;
    byte_table['e']['4'] = 0x00e4;
    byte_table['e']['5'] = 0x00e5;
    byte_table['e']['6'] = 0x00e6;
    byte_table['e']['7'] = 0x00e7;
    byte_table['e']['8'] = 0x00e8;
    byte_table['e']['9'] = 0x00e9;
    byte_table['e']['a'] = 0x00ea;
    byte_table['e']['b'] = 0x00eb;
    byte_table['e']['c'] = 0x00ec;
    byte_table['e']['d'] = 0x00ed;
    byte_table['e']['e'] = 0x00ee;
    byte_table['e']['f'] = 0x00ef;
    byte_table['f']['0'] = 0x00f0;
    byte_table['f']['1'] = 0x00f1;
    byte_table['f']['2'] = 0x00f2;
    byte_table['f']['3'] = 0x00f3;
    byte_table['f']['4'] = 0x00f4;
    byte_table['f']['5'] = 0x00f5;
    byte_table['f']['6'] = 0x00f6;
    byte_table['f']['7'] = 0x00f7;
    byte_table['f']['8'] = 0x00f8;
    byte_table['f']['9'] = 0x00f9;
    byte_table['f']['a'] = 0x00fa;
    byte_table['f']['b'] = 0x00fb;
    byte_table['f']['c'] = 0x00fc;
    byte_table['f']['d'] = 0x00fd;
    byte_table['f']['e'] = 0x00fe;
    byte_table['f']['f'] = 0x00ff;
}

void VoyaTest::readVoyaTestText() {
    using namespace _STL;

    for(int i = 0; i < TEXT_BOX_ROW_COUNT; ++i) {
        h[i] = EMPTY;
    }

    char c[SMALL_BUFFER_SIZE];
    ifstream in(VOYA_TEST_TEXT);
    int line = 1;
    string *p = new string("");
    int m = 0;
    while( in.getline(c, SMALL_BUFFER_SIZE) ) {
        delete p;
        p = new string(c);
        h[m++] = *p;

        if(++line > TEXT_BOX_ROW_COUNT) {
            break;
        }
    }

    in.close();
    delete p;
    SetDlgItemText(hdwnd, IDT_EDITBOX1, h[0].c_str() );
    SetDlgItemText(hdwnd, IDT_EDITBOX2, h[1].c_str() );
    SetDlgItemText(hdwnd, IDT_EDITBOX3, h[2].c_str() );
}

VoyaTest::VoyaTest(HINSTANCE i, HWND h): EMPTY(""), INFO_LABEL_HEIGHT(25), PARTITION("  |  "), PARTITION_SIZE(5), PIPE("|"), SEPARATOR("/"), TEST_HEIGHT(309), TEST_WIDTH(335) {
    const int BORDER = ::GetSystemMetrics(SM_CYDLGFRAME);
    const int TITLE_BAR_HEIGHT = ::GetSystemMetrics(SM_CYCAPTION);

    hwnd = h;
    instance = i;
    infoLabelHeight = INFO_LABEL_HEIGHT;
    infoLabelID = IDT_LABEL;
    infoLabelWidth = TEST_WIDTH - 2*BORDER;
    infoLabelX = BORDER;
    infoLabelY = TEST_HEIGHT - TITLE_BAR_HEIGHT - INFO_LABEL_HEIGHT - BORDER;
    width = TEST_WIDTH;
    height = TEST_HEIGHT;
}

