#include "ModalDialog.h"

extern VybmManage *manage;
extern VybmVersion *version;

BOOL CALLBACK manageDialogFunc(HWND hdwnd, UINT message, WPARAM wParam, LPARAM lParam) {
    return manage->DialogFunc(hdwnd, message, wParam, lParam);
}

BOOL CALLBACK VybmManage::DialogFunc(HWND h, UINT message, WPARAM wParam, LPARAM lParam) {
    BOOL b = 0;
    LONG checkBox;
    hdwnd = h;

    switch(message) {
    case WM_COMMAND:
        switch( LOWORD(wParam) ) {
        case IDCANCEL:
            b = TRUE;
            ::EndDialog(hdwnd, 0);
            break;
        case IDT_SAVE:
            checkBox = SendDlgItemMessage(hdwnd, IDT_BACKUP, BM_GETCHECK, 0, 0);
            if(checkBox == BST_CHECKED) {
                backUp = true;
            } else {
                backUp = false;
            }

            checkBox = SendDlgItemMessage(hdwnd, IDT_NARROW, BM_GETCHECK, 0, 0);
            if(checkBox == BST_CHECKED) {
                narrowPane = true;
            } else {
                narrowPane = false;
            }

            checkBox = SendDlgItemMessage(hdwnd, IDT_REMOVE, BM_GETCHECK, 0, 0);
            if(checkBox == BST_CHECKED) {
                removeFirst = true;
            } else {
                removeFirst = false;
            }

            checkBox = SendDlgItemMessage(hdwnd, IDT_SERIAL, BM_GETCHECK, 0, 0);
            if(checkBox == BST_CHECKED) {
                serialNumber = true;
            } else {
                serialNumber = false;
            }

            checkBox = SendDlgItemMessage(hdwnd, IDT_SHORT, BM_GETCHECK, 0, 0);
            if(checkBox == BST_CHECKED) {
                shortPane = true;
            } else {
                shortPane = false;
            }

            if( writeManageText() ) {
                b = TRUE;
                ::EndDialog(hdwnd, IDT_SAVE);
            } else {
                inform();
            }

            break;
        case IDT_HELP:
            ::WinHelp( (HWND)wParam, HELP_FILE_NAME, HELP_CONTEXT, IDH_OVERVIEW);
            b = TRUE;
            break;
        }

        break;
    case WM_INITDIALOG:
        infoLabel = ::CreateWindow(
            "STATIC",
            "",
            WS_CHILD | SS_LEFT,
            infoLabelX,
            infoLabelY,
            infoLabelWidth,
            infoLabelHeight,
            hdwnd,
            (HMENU)infoLabelID,
            instance,
            NULL
        );

        inform();
        setWindowText("Manage");
        centering();

        if(backUp) {
            SendDlgItemMessage(hdwnd, IDT_BACKUP, BM_SETCHECK, BST_CHECKED, 0);
        } else {
            SendDlgItemMessage(hdwnd, IDT_BACKUP, BM_SETCHECK, BST_UNCHECKED, 0);
        }

        if(narrowPane) {
            SendDlgItemMessage(hdwnd, IDT_NARROW, BM_SETCHECK, BST_CHECKED, 0);
        } else {
            SendDlgItemMessage(hdwnd, IDT_NARROW, BM_SETCHECK, BST_UNCHECKED, 0);
        }

        if(removeFirst) {
            SendDlgItemMessage(hdwnd, IDT_REMOVE, BM_SETCHECK, BST_CHECKED, 0);
        } else {
            SendDlgItemMessage(hdwnd, IDT_REMOVE, BM_SETCHECK, BST_UNCHECKED, 0);
        }

        if(serialNumber) {
            SendDlgItemMessage(hdwnd, IDT_SERIAL, BM_SETCHECK, BST_CHECKED, 0);
        } else {
            SendDlgItemMessage(hdwnd, IDT_SERIAL, BM_SETCHECK, BST_UNCHECKED, 0);
        }

        if(shortPane) {
            SendDlgItemMessage(hdwnd, IDT_SHORT, BM_SETCHECK, BST_CHECKED, 0);
        } else {
            SendDlgItemMessage(hdwnd, IDT_SHORT, BM_SETCHECK, BST_UNCHECKED, 0);
        }

        b = TRUE;
        break;
    }

    return b;
}

BOOL CALLBACK versionDialogFunc(HWND hdwnd, UINT message, WPARAM wParam, LPARAM lParam) {
    return version->DialogFunc(hdwnd, message, wParam, lParam);
}

BOOL CALLBACK VybmVersion::DialogFunc(HWND h, UINT message, WPARAM wParam, LPARAM lParam) {
    BOOL b = 0;
    hdwnd = h;

    switch(message) {
    case WM_COMMAND:
        switch( LOWORD(wParam) ) {
        case IDCANCEL:
            b = TRUE;
            ::EndDialog(hdwnd, 0);
            break;
        case IDO_HELP:
            ::WinHelp( (HWND)wParam, HELP_FILE_NAME, HELP_CONTEXT, IDH_OVERVIEW);
            b = TRUE;
            break;
        }

        break;
    case WM_INITDIALOG:
        infoLabel = ::CreateWindow(
            "STATIC",
            "",
            WS_CHILD | SS_LEFT,
            infoLabelX,
            infoLabelY,
            infoLabelWidth,
            infoLabelHeight,
            hdwnd,
            (HMENU)infoLabelID,
            instance,
            NULL
        );

        inform();
        setWindowText("Version");
        centering();
        b = TRUE;
        break;
    }

    return b;
}

bool VybmManage::writeManageText() {
    using namespace std;

    ifstream in(MANAGE_TEXT);
    if( in.fail() ) {
        stringstream os;
        os << MANAGE_TEXT << " is not found.";
        info = os.str();
        return false;
    }

// Reading and copying to produce an array
    char *stop;
    char c[TINY_BUFFER_SIZE];
    int line = 1;
    int setting[ END_START_LIST_SIZE + 1 ];
    setting[0] = 0;
    while( in.getline(c, TINY_BUFFER_SIZE) ) {
        setting[line] = (int)strtol(c, &stop, 10);

        if(++line > END_START_LIST_SIZE) {
            break;
        }
    }

    setting[16] = (int)backUp;
    setting[20] = (int)narrowPane;
    setting[21] = (int)removeFirst;
    setting[22] = (int)serialNumber;
    setting[23] = (int)shortPane;

// Writing
    ofstream file(MANAGE_TEXT);
    for(int i = 1; i <= END_START_LIST_SIZE; ++i) {
        file << setting[i] << endl;
    }

    return true;
}

int VybmManage::modalDialog() {
    int button = ::DialogBox(instance, "vybmCheckBoxDialog", hwnd, (DLGPROC)manageDialogFunc);
    return button;
}

int VybmVersion::modalDialog() {
    int button = ::DialogBox(instance, "vybmIconDialog", hwnd, (DLGPROC)versionDialogFunc);
    return button;
}

void createVersionText(bool comments) {
    using namespace std;

    ofstream file("version.txt");
    file << "vybm Version " << VERSION << "\n" << endl;

    if(comments) {
        file << "vybm -a # backup " << endl 
            << "vybm -b # batch " << endl 
            << "vybm -c3 # corner-3 " << endl 
            << "vybm -l # serial number " << endl 
            << "vybm -n # narrow header " << endl 
            << "vybm -q # quit " << endl 
            << "vybm -r # Remove temporary files first. " << endl 
            << "vybm -s # short window " << endl 
            << "vybm -t # Show where is the cursor. " << endl 
            << "vybm -V # version " << endl 
            << "vybm -VC # version with comments " << endl 
            << "vybm -w # wide window " << endl 
            << "vybm -y " << endl;
    } else {
        file << "vybm -a " << endl 
            << "vybm -b " << endl 
            << "vybm -c3 " << endl 
            << "vybm -l " << endl 
            << "vybm -n " << endl 
            << "vybm -q " << endl 
            << "vybm -r " << endl 
            << "vybm -s " << endl 
            << "vybm -t " << endl 
            << "vybm -V " << endl 
            << "vybm -VC " << endl 
            << "vybm -w " << endl 
            << "vybm -y " << endl;
    }

}

void VybmManage::setting(bool b, bool n, bool r, bool s, bool h) {
    backUp = b;
    narrowPane = n;
    removeFirst = r;
    serialNumber = s;
    shortPane = h;
}

void ModalDialog::centering() {
    const int HALF_WIDTH = width/2;
    const int HALF_HEIGHT = height/2;

    const BOOL REPAINT = TRUE;
    const int MENU_HEIGHT = ::GetSystemMetrics(SM_CYMENU);
    const int SCREEN_WIDTH = ::GetSystemMetrics(SM_CXFULLSCREEN);
    int screen_height = ::GetSystemMetrics(SM_CYFULLSCREEN);
    screen_height += MENU_HEIGHT;
    const int CENTER_X = SCREEN_WIDTH/2 - HALF_WIDTH;
    const int CENTER_Y = screen_height/2 - HALF_HEIGHT;

    ::MoveWindow(hdwnd, CENTER_X, CENTER_Y, width, height, REPAINT);
}

void ModalDialog::inform() {
    ::SetWindowText(infoLabel, info.c_str() );
    ::ShowWindow(infoLabel, SW_SHOW);
}

void ModalDialog::clear() {
    using namespace std;

    const string EMPTY("");
    info = EMPTY;
    ::SetWindowText(infoLabel, info.c_str() );
    ::ShowWindow(infoLabel, SW_HIDE);
}

void ModalDialog::postMessage(int UM) {
    ::PostMessage(hwnd, UM, (WPARAM)0, (LPARAM)0);
}

void ModalDialog::setWindowText(char * s) {
    ::SetWindowText(hdwnd, s);
}

VybmManage::VybmManage(HINSTANCE i, HWND h): INFO_LABEL_HEIGHT(25), MANAGE_HEIGHT(283), MANAGE_WIDTH(273) {
    using namespace std;

    const int BORDER = ::GetSystemMetrics(SM_CYDLGFRAME);
    const int TITLE_BAR_HEIGHT = ::GetSystemMetrics(SM_CYCAPTION);

    hwnd = h;
    instance = i;
    infoLabelHeight = INFO_LABEL_HEIGHT;
    infoLabelID = IDT_LABEL;
    infoLabelWidth = MANAGE_WIDTH - 2*BORDER;
    infoLabelX = BORDER;
    infoLabelY = MANAGE_HEIGHT - TITLE_BAR_HEIGHT - INFO_LABEL_HEIGHT - BORDER;
    width = MANAGE_WIDTH;
    height = MANAGE_HEIGHT;

    stringstream os;
    os << "File: " << MANAGE_TEXT;
    info = os.str();
}

VybmVersion::VybmVersion(HINSTANCE i, HWND h): INFO_LABEL_HEIGHT(25), VERSION_HEIGHT(283), VERSION_WIDTH(273) {
    using namespace std;

    const int BORDER = ::GetSystemMetrics(SM_CYDLGFRAME);
    const int TITLE_BAR_HEIGHT = ::GetSystemMetrics(SM_CYCAPTION);

    hwnd = h;
    instance = i;
    infoLabelHeight = INFO_LABEL_HEIGHT;
    infoLabelID = IDO_LABEL;
    infoLabelWidth = VERSION_WIDTH - 2*BORDER;
    infoLabelX = BORDER;
    infoLabelY = VERSION_HEIGHT - TITLE_BAR_HEIGHT - INFO_LABEL_HEIGHT - BORDER;
    width = VERSION_WIDTH;
    height = VERSION_HEIGHT;

    stringstream os;
    os << "Version " << VERSION;
    info = os.str();
}

