#ifndef TIME_NAME_H
#define TIME_NAME_H

#pragma warn -8004
#pragma warn -8012
#pragma warn -8026
#pragma warn -8057
#pragma warn -8071

#define ABBREVIATION_SIZE 3 // strlen("Fri")
#define AFTER_NAME "F"
#define AFTER_NAME_TEXT "afterName.txt"
#define AT_MARK "@"
#define BEFORE_AFTER_OPTIONAL "-n"
#define BEFORE_NAME "F"
#define BUFFER_SIZE 4224
#define CSV_COLUMN_COUNT 2
#define CSV_FILE ".csv"
#define DIAGONAL_CORNER 3
#define DRAFO "drf"
#define DRAFO_CSV "drafo.csv"
#define DRAFO_ERROR_TEXT "error.txt"
#define FILE_NAME_SIZE 40
#define FIRST_BYTE 1
#define GROUP_FIELD_WIDTH 224
#define KANA_KUTEN 0xA1
#define KANJI_SIZE_PLUS_ONE 3
#define LINE_COUNT_WIDTH 60
#define MAIN_HEIGHT 680
#define MAIN_WIDTH 473
#define MAXIMUM_RING_COUNT 34
#define MESSAGE_WIDTH 295
#define NARROW_GROUP_FIELD_WIDTH 94
#define NARROW_PANE 207
#define NARROW_VALUE_FIELD_WIDTH 94
#define NON_SAMPLING_PERIOD 3000
#define PANEL_COUNT 6
#define REMOVE_ERROR_TEXT "remove_e.txt"
#define REPEATEDLY_NORMALIZING 30
#define ROW_COUNT 22
#define SALT_LINE 31
#define SECOND_BYTE 2
#define SERIAL_TEXT "serial.txt"
#define SHIFT_JIS_WHOTXT_NAME "who60-61.txt"
#define SHORT_PANE 232
#define STATUS_BAR_HEIGHT 20
#define SUM_EQUALS_ZERO "The sum of weight equals zero."
#define TABLE_SIZE 7552 // > 7338 + 63 + 95
#define TAB_NAME_SIZE 13
#define TEXT_FILE ".txt"
#define TIME_TEXT_NAME_WIDTH 100
#define TITLE_BAR_USED_AS_LABEL 0
#define TOO_LARGE_SUM "Large sum of weight. "
#define URL_FREE_TEXT "who60-62.txt"
#define VALUE_FIELD_WIDTH 224
#define VOYA "voy"
#define VOYA_CSV "voya.csv"
#define VOYA_CTRL_TEXT "voyaCtrl.txt"
#define VOYA_LOG "voya.log"
#define VOYA_LOG_BATCH "voyaLog.bat"
#define VYBM_CSV "vybm.csv"
#define VYBM_ERROR_TEXT "error.txt"
#define WEIGHT_ARRAY_LENGTH 24 // SALT_LINE - IDLING_LINE - 1 = 24 
#define WHO "who"
#define WHO_TEXT "who.txt"
#define WIDE_PANE 712
#include <fstream>
#include <iostream>
#include <sstream>
#include <string>
#include <sys/stat.h>
#include <time.h>

class AfterBefore {
    bool acquired;
    const int BEFORE_AFTER_OPTIONAL_SIZE, LINE_END;
    const std::string afterName, atMark, beforeName, EMPTY, optional;
    std::string after, afterNameText, before, sequence;

public:
    AfterBefore(const char *p): afterName(AFTER_NAME), atMark(AT_MARK), beforeName(BEFORE_NAME), EMPTY(""), optional(BEFORE_AFTER_OPTIONAL), BEFORE_AFTER_OPTIONAL_SIZE(2), LINE_END(BUFFER_SIZE) {
        std::string f(p);
        afterNameText = f;
        acquired = false;
        after = EMPTY;
        before = EMPTY;
        sequence = EMPTY;
    }

    bool match_include(std::string & a) {
        using namespace std;

        bool matching;
        bool including = false;
        int d, v;

        if(a == optional) {
            sequence = EMPTY;
            matching = true;
        } else {
            matching = false;
            d = a.find(optional);
            if(d != string::npos) { // "-n" can be found.
                sequence = a.substr(d, LINE_END);
                v = sequence.length();
                if(v > BEFORE_AFTER_OPTIONAL_SIZE) {
                    including = true;
                } else {
                    including = false;
                }
            } else {
                including = false;
            }
        }

        if(matching || including) {
            return true;
        }

        return false;
    }

    bool match_include(const char *p) {
        std::string a(p);
        return match_include(a);
    }

    bool beforeAfterName() {
        using namespace std;

        const int AT_MARK_SIZE = 1;
        int a, d, u, v, b;

        d = sequence.find(optional);
        if(d != string::npos) { // -n
            v = d + BEFORE_AFTER_OPTIONAL_SIZE;
            a = sequence.find(atMark);
            if(a != string::npos) { // @
                u = a + AT_MARK_SIZE;
// -nbefore_name@after_name
// d v          au
                b = a - v;
                before = sequence.substr(v, b);
                after = sequence.substr(u, LINE_END);
                if( before.length() ) {
                    if( after.length() == 0 ) {
// -nbefore_name@
// d v          au
                        after = afterName;
                    }
                } else {
// -n@after_name
// d au
                    before = beforeName;
                    if( after.length() == 0 ) {
                        after = afterName;
                    }
                }

                acquired = true;
            } else { // @ is not found.
                acquired = false;
            }
        }

        return acquired;
    }

    std::string getAfter() const {
        return after;
    }

    std::string getBefore() const {
        return before;
    }

    std::string getSequence() const {
        return sequence;
    }

    bool beforeAfterAcquired() const {
        return acquired;
    }

    bool readAfterNameText();
};

class Numbered {
    char *p;
    int length, span, non_number_length;
    std::string *f;
    std::string *s;

public:
    ~Numbered() {
        delete f;
        delete p;
        delete s;
    }

    Numbered(char *a) {
        using namespace std;

        length = strlen(a);
        p = new char[ length + 1 ];

// Copying all
        strcpy(p, a);
        span = strspn(strrev(p), "0123456789");
        non_number_length = length - span;

// Copying the second part of a
        memcpy(p, a + non_number_length, span);
        p[span] = '\0';
        s = new string(p);

// Copying the first part of a
        memcpy(p, a, non_number_length);
        p[non_number_length] = '\0';
        f = new string(p);
    }

    Numbered(std::string & a) {
        using namespace std;

        length = a.length();
        p = new char[ length + 1 ];

// Copying all
        strcpy(p, a.c_str() );
        span = strspn(strrev(p), "0123456789");
        non_number_length = length - span;
        string head = a.substr(0, non_number_length);
        string tail = a.substr(non_number_length);

// Copying the second part of a
        s = new string(tail);

// Copying the first part of a
        f = new string(head);
    }

    int number() const {
        char *stop;
        int n = (int)strtol(s->c_str(), &stop, 10);
        return n;
    }

    std::string numberString() const {
        return *s;
    }

    std::string parameter() const {
        return *f;
    }

};

class TimeName {
    const std::string HYPHEN;
    std::string base, name, extension, timeName;

public:
    TimeName(std::string & b, std::string & e): HYPHEN("-") {
        base = b;
        extension = e;
        name = base;
        readSerialNumber();
    }

    TimeName(std::string & b, const char *e): HYPHEN("-") {
        base = b;
        const std::string ex(e);
        extension = ex;
        name = base;
        readSerialNumber();
    }

    TimeName(const char *b, const char *e): HYPHEN("-") {
        const std::string ba(b);
        base = ba;
        const std::string ex(e);
        extension = ex;
        name = base;
        readSerialNumber();
    }

    ~TimeName(){}

    std::string getTimeName() const {
        return timeName;
    }

    std::string temporaryName(bool);
    void readSerialNumber();
    void setSerialNumber();
    void zeroSerialNumber();
};

bool removeTemporaryFiles();
int zenkakuFragment(std::string & s, int mark_index, int mark);

#endif // TIME_NAME_H
