#include "vybm.h"

/*
vybm Version 1.03
Editing voya.csv
Copyright 2015 Sogaya

Borland C++ 5.5.1 for Win32

*/

VybmManage *manage;
VybmVersion *version;

namespace {
    bool backUp, batch, kanjiDetails, reserve, removeFirst, serialNumber, shortPane, narrowPane, showWhere, widePane;
    char c[TABLE_SIZE][KANJI_SIZE_PLUS_ONE]; // list of characters
    char group_field_name[] = "Word";
    char tabName[7][TAB_NAME_SIZE] = {"Setting", "ASCII", "Cyrillic", "Greek", "Kanji", "Mathematical", "Salt"};
    char value_field_name[] = "Data";
    const BOOL ERACE_BACKGROUND = TRUE;
    const BOOL NO_PAINTING = 0;
    const bool NO_STATUS_BAR = false;
    const BOOL REPAINT = TRUE;
    const bool STATUS_BAR = true;
    const char ELLIPSIS[] = "...";
    const double AIM = 30000.;
    const int COLUMN_SPACING = 8;
    const int COMMA_SIZE = 1;
    const int ELLIPSIS_SIZE = 3;
    const int ELLIPSIS_SIZE_1 = 2; // ELLIPSIS_SIZE - 1
    const int GROUP_FIELD_NAME_SIZE = 41;
    const int INCLUDE_UNDERSCORE = 4;
    const int LOWER_LIMIT = 20;
    const int PROMPTLY = 60;
    const int RING_COUNT = 21;
    const int TAB_CONTROL_LABEL_HEIGHT = 35;
    const int VALUE_FIELD_NAME_SIZE = 11;
    const RECT *WHOLE_WINDOW = NULL;
    const std::string COMMA(",");
    const std::string COMMA_CODE("COMMA");
    const std::string COMMENT("'");
    const std::string DATA("Data");
    const std::string EMPTY("");
    const std::string GAP(" ");
    const std::string GAP_CODE("GAP");
    const std::string HEADER_DATA("Data");
    const std::string HEADER_WORD("Word");
    const std::string SPACE("@");
    const std::string SPACE_CODE("SPACE");
    const std::string WIDE_2("  ");
    const std::string WIDE_2_CODE("WIDE_2");
    const std::string WIDE_3("   ");
    const std::string WIDE_3_CODE("WIDE_3");
    const std::string WIDE_4("    ");
    const std::string WIDE_4_CODE("WIDE_4");
    const std::string WIDE_5("     ");
    const std::string WIDE_5_CODE("WIDE_5");
    const std::string WIDE_6("      ");
    const std::string WIDE_6_CODE("WIDE_6");
    const std::string WIDE_7("       ");
    const std::string WIDE_7_CODE("WIDE_7");
    double EPSILON = 0.5;
    GROUP_VALUE  groupValue;
    GROUP_VALUE *gv = &groupValue;
    GROUP_VALUE group[ MAXIMUM_RING_COUNT + 1 ];
    GROUP_VALUE value[ MAXIMUM_RING_COUNT + 1 ];
    HWND editBox, header, statusBar, tabControl;
    int ASCIIend, ASCIIstart, bound, corner, cyrillicEnd, cyrillicStart, details, greekEnd, greekStart, groupLine, headerHeight, kanjiEnd, kanjiStart, main_height, main_width, main_x, main_y, mathematicalEnd, mathematicalStart, previousGroupLine, previousValueLine, ring_count, saltEnd, saltStart, settingEnd, settingStart, standard_main_x, standard_main_y, tabSelected, upperLimit, valueLine;
    int fieldWidth[2];
    int row[ LABEL_COUNT + 1 ];
    int tabPage[ LABEL_COUNT + 1 ];
    int w[TABLE_SIZE]; // weight list
    std::string info, timeNameText;
    std::string cells[CSV_COLUMN_COUNT][ LABEL_COUNT + 1 ];
    VYBM_CONTROL  vybmControl;
    VYBM_CONTROL *vc = &vybmControl;
    VYBM_SETTING  vybmSetting;
    VYBM_SETTING *vs = &vybmSetting;
}

bool copyEdited(int n, char *buffer) {
    bool b = false;
    char *edited_line;
    int m = n - 1;
    for(int line = 1; line <= LABEL_COUNT; ++line) {
        if(tabPage[line] == tabSelected) {
            if(row[line] == m) {
                edited_line = & vs->edited[line][0];
                strcpy(edited_line, buffer);
                b = true;
            }
        }
    }

    return b;
}

bool copyLabel(int n, std::string & label) {
    bool b = false;
    int m = n - 1;
    for(int line = 1; line <= LABEL_COUNT; ++line) {
        if(tabPage[line] == tabSelected) {
            if(row[line] == m) {
                vs->label[line] = label;
                b = true;
            }
        }
    }

    return b;
}

bool createRemoveErrorText() { // Clean
    using namespace std;

    bool found;
    const int FILE_TYPE_COUNT = 8;
    const string e0("voy*.csv");
    const string e1("");
    const string e2("");
    const string e3("");
    const string e4("");
    const string e5("");
    const string e6("");
    const string e7("");
    string file_type[FILE_TYPE_COUNT];
    HANDLE find_first_next_handle;
    ofstream ret(REMOVE_ERROR_TEXT);
    WIN32_FIND_DATA find_data;

    file_type[0] = e0;
    file_type[1] = e1;
    file_type[2] = e2;
    file_type[3] = e3;
    file_type[4] = e4;
    file_type[5] = e5;
    file_type[6] = e6;
    file_type[7] = e7;
    for(int i = 0; i < FILE_TYPE_COUNT; ++i) {
        if( file_type[i].empty() ) {
            break;
        }

        find_first_next_handle = ::FindFirstFile( file_type[i].c_str(), &find_data);
        if(find_first_next_handle == INVALID_HANDLE_VALUE) {
            continue;
        }

        found = true;
        while(found) {
            ret << find_data.cFileName << endl;
            found = ::FindNextFile(find_first_next_handle, &find_data);
        }

        ::FindClose(find_first_next_handle);
    }

    return true;
}

bool movingInside(HWND hwnd, bool showWhere) {
    const int BORDER = ::GetSystemMetrics(SM_CYSIZEFRAME);
    const int TITLE_BAR_HEIGHT = ::GetSystemMetrics(SM_CYCAPTION);
    const int MENU_HEIGHT = ::GetSystemMetrics(SM_CYMENU);
    const int HEAD = BORDER + TITLE_BAR_HEIGHT + MENU_HEIGHT + TAB_CONTROL_LABEL_HEIGHT;
    POINT pos;
    RECT windowRect;

    ::GetWindowRect(hwnd, &windowRect);
    ::GetCursorPos(&pos);

    bool through = false;
    char buffer[BUFFER_SIZE];
    int upper_left_x = windowRect.left;
    int upper_left_y = windowRect.top + HEAD;
    int upper_right_x = windowRect.right;
    int lower_left_y = windowRect.bottom - STATUS_BAR_HEIGHT - BORDER;
    if(upper_left_x < pos.x && pos.x < upper_right_x) {
        if(upper_left_y < pos.y && pos.y < lower_left_y) {
            through = true;
        }
    }

    if(showWhere) {
        if(through) {
            wsprintf(buffer, "%d : %d ( inside ) ", pos.x, pos.y);
        } else {
            wsprintf(buffer, "%d : %d ( outside ) ", pos.x, pos.y);
        }

        ::SendMessage(statusBar, SB_SETTEXT, (WPARAM)1, (LPARAM)buffer);
    } else {
        ::KillTimer(hwnd, IDM_TIMER);
    }

    return through;
}

bool readVoyaCSV() {
    using namespace std;

    if( voya_vybm_rearrange() ) {
        ;
    } else {
        return false;
    }

    ifstream in(VYBM_CSV);

// Reading and copying to produce an array
    char *edited_line;
    char c[SMALL_BUFFER_SIZE];
    const string WIDE_7_COMMA_GAP_COMMA = WIDE_7 + COMMA + GAP + COMMA;
    int line = 1;
    int q;
    string *p = new string("");
    DividingCSV *t = new DividingCSV(*p);
    string b, s, tail;

    cells[0][0] = HEADER_WORD;
    cells[1][0] = HEADER_DATA;

    while( in.getline(c, SMALL_BUFFER_SIZE) ) {
        delete p;
        p = new string(c);
        q = p->find(COMMENT);
        if(q != string::npos) {
            s = p->substr(0, q);
            if( s.length() == 0 ) {
                s = WIDE_7_COMMA_GAP_COMMA;
            }

            q = s.find(COMMA); // COMMA-1
            if(q != string::npos) {
                q += COMMA_SIZE;
                tail = s.substr(q);
                q = tail.find(COMMA); // COMMA-2
                if(q != string::npos) {
                    ;
                } else {
                    s = WIDE_7_COMMA_GAP_COMMA;
                }

            } else {
                s = WIDE_7_COMMA_GAP_COMMA;
            }
        } else {
            s = *p;
        }

        delete t;
        t = new DividingCSV(s);
        for(int i = 0; i < CSV_COLUMN_COUNT; ++i) {
            cells[i][line] = t->a(i);
        }

        if(++line > LABEL_COUNT) {
            break;
        }
    }

// Decoding and storing
    for(line = 0; line <= LABEL_COUNT; ++line) {
        if(cells[0][line] == COMMA_CODE) {
            vs->label[line] = COMMA;
        } else if(cells[0][line] == GAP_CODE) {
            vs->label[line] = GAP;
        } else if(cells[0][line] == SPACE_CODE) {
            vs->label[line] = SPACE;
        } else if(cells[0][line] == WIDE_2_CODE) {
            vs->label[line] = WIDE_2;
        } else if(cells[0][line] == WIDE_3_CODE) {
            vs->label[line] = WIDE_3;
        } else if(cells[0][line] == WIDE_4_CODE) {
            vs->label[line] = WIDE_4;
        } else if(cells[0][line] == WIDE_5_CODE) {
            vs->label[line] = WIDE_5;
        } else if(cells[0][line] == WIDE_6_CODE) {
            vs->label[line] = WIDE_6;
        } else if(cells[0][line] == WIDE_7_CODE) {
            vs->label[line] = WIDE_7;
            cells[1][line] = GAP_CODE;
        } else {
            vs->label[line] = cells[0][line];
        }

        if(cells[1][line] == COMMA_CODE) {
            b = COMMA;
        } else if(cells[1][line] == GAP_CODE) {
            b = GAP;
        } else if(cells[1][line] == SPACE_CODE) {
            b = SPACE;
        } else if(cells[1][line] == WIDE_2_CODE) {
            b = WIDE_2;
        } else if(cells[1][line] == WIDE_3_CODE) {
            b = WIDE_3;
        } else if(cells[1][line] == WIDE_4_CODE) {
            b = WIDE_4;
        } else if(cells[1][line] == WIDE_5_CODE) {
            b = WIDE_5;
        } else if(cells[1][line] == WIDE_6_CODE) {
            b = WIDE_6;
        } else if(cells[1][line] == WIDE_7_CODE) {
            b = WIDE_7;
        } else {
            b = cells[1][line];
        }

        edited_line = & vs->edited[line][0];
        strcpy(edited_line, b.c_str() );
    }

    delete p;
    delete t;
    return true;
}

bool readManageText() {
    using namespace std;

    ifstream in(MANAGE_TEXT);
    if( in.fail() ) {
        stringstream os;
        os << MANAGE_TEXT << " is not found.";
        info = os.str();
        return false;
    }

// Reading and copying to produce an array
    char *stop;
    char c[SMALL_BUFFER_SIZE];
    int line = 1;
    int setting[ END_START_LIST_SIZE + 1 ];
    setting[0] = 0;
    while( in.getline(c, SMALL_BUFFER_SIZE) ) {
        setting[line] = (int)strtol(c, &stop, 10);

        if( ++line > END_START_LIST_SIZE ) {
            break;
        }
    }

    ASCIIend = setting[1];
    ASCIIstart = setting[2];
    cyrillicEnd = setting[3];
    cyrillicStart = setting[4];
    greekEnd = setting[5];
    greekStart = setting[6];
    kanjiEnd = setting[7];
    kanjiStart = setting[8];
    mathematicalEnd = setting[9];
    mathematicalStart = setting[10];
    saltEnd = setting[11];
    saltStart = setting[12];
    settingEnd = setting[13];
    settingStart = setting[14];
    upperLimit = setting[15];

// backUp, corner, reserve, kanjiDetails, narrowPane, removeFirst, serialNumber, shortPane, showWhere, widePane;
// 16      17      18       19            20          21           22            23         24         25       
    switch(setting[16]) {
    case 0:
        vc->backUp = false;
        break;
    default:
        vc->backUp = true;
        break;
    }

    vc->corner = setting[17];

    switch(setting[18]) {
    case 0:
        vc->reserve = false;
        break;
    default:
        vc->reserve = true;
        break;
    }

    switch(setting[19]) {
    case 0:
        vc->kanjiDetails = false;
        break;
    default:
        vc->kanjiDetails = true;
        break;
    }

    switch(setting[20]) {
    case 0:
        vc->narrowPane = false;
        break;
    default:
        vc->narrowPane = true;
        break;
    }

    switch(setting[21]) {
    case 0:
        vc->removeFirst = false;
        break;
    default:
        vc->removeFirst = true;
        break;
    }

    switch(setting[22]) {
    case 0:
        vc->serialNumber = false;
        break;
    default:
        vc->serialNumber = true;
        break;
    }

    switch(setting[23]) {
    case 0:
        vc->shortPane = false;
        break;
    default:
        vc->shortPane = true;
        break;
    }

    switch(setting[24]) {
    case 0:
        vc->showWhere = false;
        break;
    default:
        vc->showWhere = true;
        break;
    }

    switch(setting[25]) {
    case 0:
        vc->widePane = false;
        break;
    default:
        vc->widePane = true;
        break;
    }

    return true;
}

bool removeTemporaryFiles() {
    using namespace std;

    char c[FILE_NAME_SIZE];
    ifstream in(REMOVE_ERROR_TEXT);
    if( in.fail() ) {
        return false;
    }

    while( in.getline(c, FILE_NAME_SIZE) ) {
        remove(c);
    }

    return true;
}

bool voya_vybm_rearrange() {
    using namespace std;

    ifstream in(VOYA_CSV);
    if( in.fail() ) {
        stringstream os;
        os << VOYA_CSV << " is not found.";
        info = os.str();
        return false;
    }

    char *stop;
    char c[SMALL_BUFFER_SIZE];
    int a[WEIGHT_ARRAY_LENGTH]; // SALT_LINE - IDLING_LINE - 1 = 24
    int i = 0;
    int line = 1;
    string *p = new string("");
    DividingCSV *t = new DividingCSV(*p);
    string s;
    string setting[ LABEL_COUNT + 1 ];
    stringstream os;
    setting[0] = EMPTY;
    while( in.getline(c, SMALL_BUFFER_SIZE) ) {
        os << c << endl;

        delete p;
        p = new string(c);
        if(IDLING_LINE < line && line < SALT_LINE) {
            delete t;
            t = new DividingCSV(*p);
            s = t->a(1);
            a[i] = (int)strtol( s.c_str(), &stop, 10);
            ++i;
        }

        setting[line] = *p + COMMA;

        if(++line > LABEL_COUNT) {
            break;
        }
    }

    if(backUp) {
        TimeName csv(VOYA, CSV_FILE);
        timeNameText = csv.temporaryName(serialNumber);
        ofstream file( timeNameText.c_str() );
        string whole = os.str();
        file << whole;
    }

// weight
    vs->ascii_weight = a[0];
    vs->capital_omega_weight = a[1];
    vs->capital_ya_weight = a[2];
    vs->capital_z_weight = a[3];
    vs->cap_weight = a[4];
    vs->circle_weight = a[5];
    vs->cup_weight = a[6];
    vs->double_integral_weight = a[7];
    vs->exists_weight = a[8];
    vs->hankaku_katakana_weight = a[9];
    vs->heisei_weight = a[10];
    vs->hiragana_weight = a[11];
    vs->jis_level_one_kanji_weight = a[12];
    vs->jis_level_two_kanji_weight = a[13];
    vs->katakana_weight = a[14];
    vs->keisen_weight = a[15];
    vs->nine_weight = a[16];
    vs->omega_weight = a[17];
    vs->paragraph_weight = a[18];
    vs->parallel_weight = a[19];
    vs->roman_weight = a[20];
    vs->square_meter_weight = a[21];
    vs->ya_weight = a[22];
    vs->z_weight = a[23];

// Rearranging
    ofstream file(VYBM_CSV);
    file << setting[1] << endl << setting[2] << endl << setting[3] << endl << setting[4] << endl << setting[5] << endl 
        << setting[6] << endl << setting[7] << endl << setting[10] << endl << setting[30] << endl << setting[9] << endl 
        << setting[29] << endl << setting[8] << endl << setting[24] << endl << setting[13] << endl << setting[16] << endl 
        << setting[17] << endl << setting[18] << endl << setting[19] << endl << setting[20] << endl << setting[21] << endl 
        << setting[11] << endl << setting[12] << endl << setting[14] << endl << setting[15] << endl << setting[22] << endl 
        << setting[23] << endl << setting[25] << endl << setting[26] << endl << setting[27] << endl << setting[28] << endl 
        << setting[31] << endl << setting[32] << endl << setting[33] << endl << setting[34] << endl;

    delete p;
    delete t;
    return true;
}

bool vybm_voya_rearrange() {
    using namespace std;

    ifstream in(VYBM_CSV);
    if( in.fail() ) {
        stringstream os;
        os << VYBM_CSV << " is not found.";
        info = os.str();
        return false;
    }

    char c[SMALL_BUFFER_SIZE];
    int q;
    int line = 1;
    string *p = new string("");
    string s;
    string setting[ LABEL_COUNT + 1 ];
    setting[0] = EMPTY;
    while( in.getline(c, SMALL_BUFFER_SIZE) ) {
        delete p;
        p = new string(c);
        q = p->rfind(COMMA);
        s = p->substr(0, q);
        setting[line] = s;

        if(++line > LABEL_COUNT) {
            break;
        }
    }

    ofstream file(VOYA_CSV);
    file << setting[1] << endl << setting[2] << endl << setting[3] << endl << setting[4] << endl << setting[5] << endl 
        << setting[6] << endl << setting[7] << endl << setting[12] << endl << setting[10] << endl << setting[8] << endl 
        << setting[21] << endl << setting[22] << endl << setting[14] << endl << setting[23] << endl << setting[24] << endl 
        << setting[15] << endl << setting[16] << endl << setting[17] << endl << setting[18] << endl << setting[19] << endl 
        << setting[20] << endl << setting[25] << endl << setting[26] << endl << setting[13] << endl << setting[27] << endl 
        << setting[28] << endl << setting[29] << endl << setting[30] << endl << setting[11] << endl << setting[9] << endl 
        << setting[31] << endl << setting[32] << endl << setting[33] << endl << setting[34] << endl;

    delete p;
    return true;
}

bool voyaLogBatchProcessing() {
    using namespace std;

    ifstream voya_log_batch(VOYA_LOG_BATCH);
    if( voya_log_batch.fail() ) {
        stringstream os;
        os << VOYA_LOG_BATCH << " is not found.";
        info = os.str();
        return false;
    }

    vector<string> sublog, voya_error;
    char c[BUFFER_SIZE];
    char voyaErrorName[ MAX_PATH + 1 ];
    const int DEL_COMMAND_SIZE = 4;
    const int REN_COMMAND_SIZE = 4;
    const int PLUS_SIZE = 3; // plus, dot and yen
    const int TEMPORARY_SIZE = 4;
    const string DEL_COMMAND("DEL ");
    const string GAP(" ");
    const string PLUS("+.\\");
    const string REN_COMMAND("REN ");
    const string TEMPORARY(".tmp");
    ifstream voyaError;
    int q;
    string *b = new string("");
    string head, s, tail;

/*
COPY /B voya.log+.\voyCA.tmp+.\voyCB.tmp voyCC.tmp
DEL voya.log
REN voyCC.tmp voya.log
DEL .\voyCA.tmp
DEL .\voyCB.tmp
*/
    if( voya_log_batch.getline(c, BUFFER_SIZE) ) {
// COPY line
        delete b;
        b = new string(c);
        q = b->find(PLUS);
        if(q != string::npos) {
            q += PLUS_SIZE;
            tail = b->substr(q);
            for(;;) {
                q = tail.find(PLUS);
                if(q != string::npos) {
                    head = tail.substr(0, q);
                    q += PLUS_SIZE;
                    tail = tail.substr(q);
                    sublog.push_back(head);
                } else {
                    q = tail.find(GAP);
                    if(q != string::npos) {
                        head = tail.substr(0, q);
                        sublog.push_back(head);
                    }

                    break;
                }

            } // for(;;)

        } else {
            const string PLUS_ERROR("q = b->find(PLUS)");
            info = PLUS_ERROR;
            return false;
        }

// DEL lines
        while( voya_log_batch.getline(c, BUFFER_SIZE) ) {
            delete b;
            b = new string(c);

            q = b->find(DEL_COMMAND);
            if(q != string::npos) {
                q += DEL_COMMAND_SIZE;
                tail = b->substr(q);
                q = tail.find(TEMPORARY);
                if(q != string::npos) {
                    q += TEMPORARY_SIZE;
                    head = tail.substr(0, q);
                    voya_error.push_back(head);
                } else {
                    continue; // voya.log line
                }
            } else {
// REN line
                q = b->find(REN_COMMAND);
                if(q != string::npos) {
                    q += REN_COMMAND_SIZE;
                    tail = b->substr(q);
                    q = tail.find(TEMPORARY);
                    if(q != string::npos) {
                        q += TEMPORARY_SIZE;
                        head = tail.substr(0, q);
                        voya_error.push_back(head);
                    } else {
                        const string TEMPORARY_ERROR("q = tail.find(TEMPORARY)");
                        info = TEMPORARY_ERROR;
                        return false;
                    }

                } else {
                    const string REN_ERROR("q = b->find(REN_COMMAND)");
                    info = REN_ERROR;
                    return false;
                }

                continue;
            }
        }

    } else {
        cerr << "if( voya_log_batch.getline(c, BUFFER_SIZE) )" << endl;
        return false;
    }

// log file
    ofstream voyaLog(vs->log_file_name, ios::app);
    if( voyaLog.fail() ) {
        stringstream os;
        os << vs->log_file_name;
        info = os.str();
        return false;
    }

    for(vector<string>::iterator i = sublog.begin(); i != sublog.end(); ++i) {
        s = *i;
        strcpy(voyaErrorName, s.c_str() );

// open a sublog file
        voyaError.open(voyaErrorName);
        if( voyaError.fail() ) {
            stringstream os;
            os << voyaErrorName << ": voyaError.fail()";
            info = os.str();
            return false;
        }

        while( voyaError.getline(c, BUFFER_SIZE) ) {
// Adding sublog to the log
            voyaLog << c << endl;
        }

        voyaError.close();
    }

    voyaLog.close();

// Deleting voya error files
    for(vector<string>::iterator i = voya_error.begin(); i != voya_error.end(); ++i) {
        s = *i;
        strcpy(voyaErrorName, s.c_str() );
        remove(voyaErrorName);
    }

    delete b;
    return true;
}

bool withinRange(int n) {
    bool b = false;
    int m = n - 1;
    for(int line = 1; line <= LABEL_COUNT; ++line) {
        if(tabPage[line] == tabSelected) {
            if(row[line] == m) {
                b = true;
            }
        }
    }

    return b;
}

double shiftJisKanaAscii() {
    const int LOWER_BYTE = 0x00FF;
    int higher, lower;
    int d = 0;

    double sum = 0.0;
    for(int i = 0x8140; i <= 0x81AC; ++i, ++d) { // 
        lower = i & LOWER_BYTE;
        higher = i >> 8;
        c[d][0] = higher;
        c[d][1] = lower;
        c[d][2] = '\0';
        w[d] = vs->parallel_weight;
        sum += (double)w[d];
    }

    for(int i = 0x81B8; i <= 0x81BF; ++i, ++d) { // 
        lower = i & LOWER_BYTE;
        higher = i >> 8;
        c[d][0] = higher;
        c[d][1] = lower;
        c[d][2] = '\0';
        w[d] = vs->cap_weight;
        sum += (double)w[d];
    }

    for(int i = 0x81C8; i <= 0x81CE; ++i, ++d) { // 
        lower = i & LOWER_BYTE;
        higher = i >> 8;
        c[d][0] = higher;
        c[d][1] = lower;
        c[d][2] = '\0';
        w[d] = vs->exists_weight;
        sum += (double)w[d];
    }

    for(int i = 0x81DA; i <= 0x81E8; ++i, ++d) { // 
        lower = i & LOWER_BYTE;
        higher = i >> 8;
        c[d][0] = higher;
        c[d][1] = lower;
        c[d][2] = '\0';
        w[d] = vs->double_integral_weight;
        sum += (double)w[d];
    }

    for(int i = 0x81F0; i <= 0x81F7; ++i, ++d) { // 
        lower = i & LOWER_BYTE;
        higher = i >> 8;
        c[d][0] = higher;
        c[d][1] = lower;
        c[d][2] = '\0';
        w[d] = vs->paragraph_weight;
        sum += (double)w[d];
    }

    int m = 0x81FC; // 
    lower = m & LOWER_BYTE;
    higher = m >> 8;
    c[d][0] = higher;
    c[d][1] = lower;
    c[d][2] = '\0';
    w[d] = vs->circle_weight;
    sum += (double)w[d];
    d++;

    for(int i = 0x824F; i <= 0x8258; ++i, ++d) { // X
        lower = i & LOWER_BYTE;
        higher = i >> 8;
        c[d][0] = higher;
        c[d][1] = lower;
        c[d][2] = '\0';
        w[d] = vs->nine_weight;
        sum += (double)w[d];
    }

    for(int i = 0x8260; i <= 0x8279; ++i, ++d) { // y
        lower = i & LOWER_BYTE;
        higher = i >> 8;
        c[d][0] = higher;
        c[d][1] = lower;
        c[d][2] = '\0';
        w[d] = vs->capital_z_weight;
        sum += (double)w[d];
    }

    for(int i = 0x8281; i <= 0x829A; ++i, ++d) { // 
        lower = i & LOWER_BYTE;
        higher = i >> 8;
        c[d][0] = higher;
        c[d][1] = lower;
        c[d][2] = '\0';
        w[d] = vs->z_weight;
        sum += (double)w[d];
    }

    for(int i = 0x829F; i <= 0x82F1; ++i, ++d) { // 
        lower = i & LOWER_BYTE;
        higher = i >> 8;
        c[d][0] = higher;
        c[d][1] = lower;
        c[d][2] = '\0';
        w[d] = vs->hiragana_weight;
        sum += (double)w[d];
    }

    for(int i = 0x8340; i <= 0x8396; ++i, ++d) { // 
        if(i == 0x837F) {
            continue;
        }

        lower = i & LOWER_BYTE;
        higher = i >> 8;
        c[d][0] = higher;
        c[d][1] = lower;
        c[d][2] = '\0';
        w[d] = vs->katakana_weight;
        sum += (double)w[d];
    }

    for(int i = 0x839F; i <= 0x83B6; ++i, ++d) { // 
        lower = i & LOWER_BYTE;
        higher = i >> 8;
        c[d][0] = higher;
        c[d][1] = lower;
        c[d][2] = '\0';
        w[d] = vs->capital_omega_weight;
        sum += (double)w[d];
    }

    for(int i = 0x83BF; i <= 0x83D6; ++i, ++d) { // 
        lower = i & LOWER_BYTE;
        higher = i >> 8;
        c[d][0] = higher;
        c[d][1] = lower;
        c[d][2] = '\0';
        w[d] = vs->omega_weight;
        sum += (double)w[d];
    }

    for(int i = 0x8440; i <= 0x8460; ++i, ++d) { // `
        lower = i & LOWER_BYTE;
        higher = i >> 8;
        c[d][0] = higher;
        c[d][1] = lower;
        c[d][2] = '\0';
        w[d] = vs->capital_ya_weight;
        sum += (double)w[d];
    }

    for(int i = 0x8470; i <= 0x8491; ++i) { // 
        lower = i & LOWER_BYTE;
        if(lower == 0x7F) {
            continue;
        }

        higher = i >> 8;
        c[d][0] = higher;
        c[d][1] = lower;
        c[d][2] = '\0';
        w[d] = vs->ya_weight;
        sum += (double)w[d];
        d++;
    }

    for(int i = 0x849F; i <= 0x84BE; ++i, ++d) { // 
        lower = i & LOWER_BYTE;
        higher = i >> 8;
        c[d][0] = higher;
        c[d][1] = lower;
        c[d][2] = '\0';
        w[d] = vs->keisen_weight;
        sum += (double)w[d];
    }

    for(int i = 0x8740; i <= 0x875D; ++i, ++d) { // ]
        lower = i & LOWER_BYTE;
        higher = i >> 8;
        c[d][0] = higher;
        c[d][1] = lower;
        c[d][2] = '\0';
        w[d] = vs->roman_weight;
        sum += (double)w[d];
    }

    for(int i = 0x875F; i <= 0x8775; ++i, ++d) { // u
        lower = i & LOWER_BYTE;
        higher = i >> 8;
        c[d][0] = higher;
        c[d][1] = lower;
        c[d][2] = '\0';
        w[d] = vs->square_meter_weight;
        sum += (double)w[d];
    }

    int n = 0x877E; // ~
    lower = n & LOWER_BYTE;
    higher = n >> 8;
    c[d][0] = higher;
    c[d][1] = lower;
    c[d][2] = '\0';
    w[d] = vs->heisei_weight;
    sum += (double)w[d];
    d++;

    for(int i = 0x8780; i <= 0x879C; ++i, ++d) { // 
        lower = i & LOWER_BYTE;
        higher = i >> 8;
        c[d][0] = higher;
        c[d][1] = lower;
        c[d][2] = '\0';
        w[d] = vs->cup_weight;
        sum += (double)w[d];
    }

    for(int i = 0x889F; i <= 0x88FC; ++i, ++d) { // JIS level-1 kanji set starts idling 0x889F (  )
        lower = i & LOWER_BYTE;
        higher = i >> 8;
        c[d][0] = higher;
        c[d][1] = lower;
        c[d][2] = '\0';
        w[d] = vs->jis_level_one_kanji_weight;
        sum += (double)w[d];
    }

    for(int i = 0x8940; i <= 0x9FFC; ++i) {
        if(0x9873 <=i && i <= 0x989E) { // JIS level-2 kanji set starts idling 0x989F (  )
            continue;
        }

        lower = i & LOWER_BYTE;
        if(lower < 0x40) {
            continue;
        } else if(lower == 0x7F) {
            continue;
        } else if(0xFD <= lower) {
            continue;
        }

        higher = i >> 8;
        c[d][0] = higher;
        c[d][1] = lower;
        c[d][2] = '\0';
        w[d] = vs->jis_level_two_kanji_weight;
        sum += (double)w[d];
        d++;
    }

    for(int i = 0xE040; i <= 0xE9FC; ++i) {
        lower = i & LOWER_BYTE;
        if(lower < 0x40) {
            continue;
        } else if(lower == 0x7F) {
            continue;
        } else if(0xFD <= lower) {
            continue;
        }

        higher = i >> 8;
        c[d][0] = higher;
        c[d][1] = lower;
        c[d][2] = '\0';
        w[d] = vs->jis_level_two_kanji_weight;
        sum += (double)w[d];
        d++;
    }

    for(int i = 0xEA40; i <= 0xEAA4; ++i) {
        lower = i & LOWER_BYTE;
        if(lower == 0x7F) {
            continue;
        }

        higher = i >> 8;
        c[d][0] = higher;
        c[d][1] = lower;
        c[d][2] = '\0';
        w[d] = vs->jis_level_two_kanji_weight;
        sum += (double)w[d];
        d++;
    }

    for(int i = 0xED40; i <= 0xEDFC; ++i) {
        lower = i & LOWER_BYTE;
        if(lower == 0x7F) {
            continue;
        }

        higher = i >> 8;
        c[d][0] = higher;
        c[d][1] = lower;
        c[d][2] = '\0';
        w[d] = vs->jis_level_two_kanji_weight;
        sum += (double)w[d];
        d++;
    }

    for(int i = 0xEE40; i <= 0xEEEC; ++i) {
        lower = i & LOWER_BYTE;
        if(lower == 0x7F) {
            continue;
        }

        higher = i >> 8;
        c[d][0] = higher;
        c[d][1] = lower;
        c[d][2] = '\0';
        w[d] = vs->jis_level_two_kanji_weight;
        sum += (double)w[d];
        d++;
    }

    for(int i = 0xEEEF; i <= 0xEEFC; ++i, ++d) {
        lower = i & LOWER_BYTE;
        higher = i >> 8;
        c[d][0] = higher;
        c[d][1] = lower;
        c[d][2] = '\0';
        w[d] = vs->jis_level_two_kanji_weight;
        sum += (double)w[d];
    }

// hankaku katakana
    for(int i = 0xA1; i <= 0xDF; ++i, ++d) { // 
        c[d][0] = i;
        c[d][1] = '\0';
        c[d][2] = '\0';
        w[d] = vs->hankaku_katakana_weight;
        sum += (double)w[d];
    }

// ascii
    for(int i = 0x20; i <= 0x7E; ++i, ++d) { //  !"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_`abcdefghijklmnopqrstuvwxyz{|}~
        c[d][0] = i;
        c[d][1] = '\0';
        c[d][2] = '\0';
        w[d] = vs->ascii_weight;
        sum += (double)w[d];
    }

    bound = d;
    return sum;
}

double summing(bool status_bar) {
    using namespace std;

    double sum = shiftJisKanaAscii();

    switch( sumOfWeight(sum) ) {
    case -1:
        if(status_bar) {
            ::SendMessage(statusBar, SB_SETTEXT, (WPARAM)1, (LPARAM)TOO_LARGE_SUM);
        }

        break;
    case 0:
        if(status_bar) {
            ::SendMessage(statusBar, SB_SETTEXT, (WPARAM)1, (LPARAM)SUM_EQUALS_ZERO);
        }

        break;
    default:
        if(status_bar) {
            ::SendMessage(statusBar, SB_SETTEXT, (WPARAM)1, (LPARAM)VOYA_CSV);
        }

        break;
    }

    stringstream os;
    os << sum;
    string s = os.str();
    if(status_bar) {
        ::SendMessage(statusBar, SB_SETTEXT, (WPARAM)0, (LPARAM)s.c_str() );
    }

    return sum;
}

int checkKanji(char *c, int length) {
    int q = length - 1;
    unsigned char byte = (unsigned char)c[q];
    int mark = (int)byte;
    std::string s(c);

    switch( zenkakuFragment(s, q, mark) ) {
    case FIRST_BYTE:
        length -= 1;
        break;
    case SECOND_BYTE:
        break;
    default:
        break;
    }

    return length;
}

int groupColumn(HWND hwnd) {
    const int BORDER = ::GetSystemMetrics(SM_CYSIZEFRAME);
    const int TITLE_BAR_HEIGHT = ::GetSystemMetrics(SM_CYCAPTION);
    const int MENU_HEIGHT = ::GetSystemMetrics(SM_CYMENU);
    const int HEAD = BORDER + TITLE_BAR_HEIGHT + MENU_HEIGHT + TAB_CONTROL_LABEL_HEIGHT + headerHeight;
    POINT pos;
    RECT windowRect;

    ::GetWindowRect(hwnd, &windowRect);
    ::GetCursorPos(&pos);

    bool through = false;
    const int UPPER_LEFT_X = windowRect.left + BORDER;
    const int UPPER_RIGHT_X = UPPER_LEFT_X + fieldWidth[0];
    int upper_left_y = windowRect.top + HEAD;
    int lower_left_y = windowRect.bottom - STATUS_BAR_HEIGHT - BORDER;
    if(UPPER_LEFT_X < pos.x && pos.x < UPPER_RIGHT_X) {
        if(upper_left_y < pos.y && pos.y < lower_left_y) {
            through = true;
        }
    }

    if(through) {
        int n;
        for(int line = 1; line <= LABEL_COUNT; ++line) {
            if(tabPage[line] == tabSelected) {
                n = row[line] + 1;
                upper_left_y = windowRect.top + BORDER + TITLE_BAR_HEIGHT + MENU_HEIGHT + group[n].y;
                lower_left_y = upper_left_y + group[n].height;
                if(upper_left_y < pos.y && pos.y < lower_left_y) {
                    break;
                }

            }

        }

        return n;
    } else {
        return 0;
    }
}

int sumOfWeight(double sum) {
    const double LIMIT = 65536.0; // unsigned 16-bit integers
    const int TOO_LARGE = -1;
    int s;
    if(sum > LIMIT) {
        s = TOO_LARGE;
    } else {
        s = (int)sum;
    }

    return s;
}

int valueColumn(HWND hwnd) {
    const int BORDER = ::GetSystemMetrics(SM_CYSIZEFRAME);
    const int TITLE_BAR_HEIGHT = ::GetSystemMetrics(SM_CYCAPTION);
    const int MENU_HEIGHT = ::GetSystemMetrics(SM_CYMENU);
    const int HEAD = BORDER + TITLE_BAR_HEIGHT + MENU_HEIGHT + TAB_CONTROL_LABEL_HEIGHT + headerHeight;
    POINT pos;
    RECT windowRect;

    ::GetWindowRect(hwnd, &windowRect);
    ::GetCursorPos(&pos);

    bool through = false;
    const int UPPER_LEFT_X = windowRect.left + BORDER + fieldWidth[0];
    const int UPPER_RIGHT_X = UPPER_LEFT_X + fieldWidth[1];
    int upper_left_y = windowRect.top + HEAD;
    int lower_left_y = windowRect.bottom - STATUS_BAR_HEIGHT - BORDER;
    if(UPPER_LEFT_X < pos.x && pos.x < UPPER_RIGHT_X) {
        if(upper_left_y < pos.y && pos.y < lower_left_y) {
            through = true;
        }
    }

    if(through) {
        int n;
        for(int line = 1; line <= LABEL_COUNT; ++line) {
            if(tabPage[line] == tabSelected) {
                n = row[line] + 1;
                upper_left_y = windowRect.top + BORDER + TITLE_BAR_HEIGHT + MENU_HEIGHT + value[n].y;
                lower_left_y = upper_left_y + value[n].height;
                if(upper_left_y < pos.y && pos.y < lower_left_y) {
                    break;
                }

            }

        }

        return n;
    } else {
        return 0;
    }
}

int WINAPI WinMain(HINSTANCE instance, HINSTANCE hPrevInst, LPSTR lpszArgs, int nWinMode) {
    using namespace std;

    ifstream test(VOYA_CSV);
    if( test.fail() ) {
        ofstream file(VYBM_ERROR_TEXT);
        file << VOYA_CSV << " is not found." << endl;
        return 0;
    }

    ifstream in(MANAGE_TEXT);
    if( in.fail() ) {
        createManageText();
    } else {
        in.close();
    }

    readManageText();
    backUp = vc->backUp;
    corner = vc->corner;
    kanjiDetails = vc->kanjiDetails;
    reserve = vc->reserve;
    narrowPane = vc->narrowPane;
    removeFirst = vc->removeFirst;
    serialNumber = vc->serialNumber;
    shortPane = vc->shortPane;
    showWhere = vc->showWhere;
    widePane = vc->widePane;

// command line
    bool quit = false;
    const bool WITHOUT_COMMENTS = false;
    const bool WITH_COMMENTS = true;
    const bool NO_READING = false;
    const int ARG_COUNT = 12;
    const int PARAMETER_SIZE = 2;
    const string BACKUP("-a");
    const string BATCH("-b");
    const string CORNER("-c");
    const string INCLUDE("-i");
    const string RESERVE("-e");
    const string KANJI("-k");
    const string NARROW("-n");
    const string QUIT("-q");
    const string RECOVERY("-y");
    const string REMOVE("-r");
    const string SERIAL("-l");
    const string SHORT("-s");
    const string THROUGH("-t");
    const string version("-V");
    const string version_comment("-VC");
    const string WIDE("-w");
    Numbered *n = new Numbered("-c3");
    string non_number;
    string argv[ARG_COUNT];
    string command( ::GetCommandLine() );
    Dividing t(command);
    int argc = t.itemCount();
    argv[0] = t.a(0);
    argv[1] = t.a(1);
    argv[2] = t.a(2);
    argv[3] = t.a(3);
    argv[4] = t.a(4);
    argv[5] = t.a(5);
    argv[6] = t.a(6);
    argv[7] = t.a(7);
    argv[8] = t.a(8);
    argv[9] = t.a(9);
    argv[10] = t.a(10);
    argv[11] = t.a(11);

    for(int i = 0; i < argc; ++i) {
        if(BACKUP == argv[i]) {
            backUp = true;
        } else if(BATCH == argv[i]) {
            batch = true;
        } else if(KANJI == argv[i]) {
            kanjiDetails = true;
        } else if(INCLUDE == argv[i]) {
            reserve = true;
        } else if(NARROW == argv[i]) {
            narrowPane = true;
        } else if(QUIT == argv[i]) {
            quit = true;
        } else if(RECOVERY == argv[i]) {
            recovery(NO_READING);
        } else if(REMOVE == argv[i]) {
            removeFirst = true;
        } else if(SERIAL == argv[i]) {
            serialNumber = true;
        } else if(SHORT == argv[i]) {
            shortPane = true;
        } else if(THROUGH == argv[i]) {
            showWhere = true;
        } else if(version == argv[i]) {
            createVersionText(WITHOUT_COMMENTS);
            return 0;

        } else if(version_comment == argv[i]) {
            createVersionText(WITH_COMMENTS);
            return 0;

        } else if(WIDE == argv[i]) {
            widePane = true;
        } else {
            if( argv[i].length() > PARAMETER_SIZE ) {
                delete n;
                n = new Numbered(argv[i]);
                non_number = n->parameter();

                if(CORNER == non_number) {
                    corner = n->number();
                }

            }

        }

    }

    delete n;

    if(narrowPane) {
        widePane = false;
    }

    if(removeFirst) {
        removeTemporaryFiles();
        remove(REMOVE_ERROR_TEXT);
    }

    if(batch) {
        corner = DIAGONAL_CORNER;
        narrowPane = true;
        shortPane = true;
        showWhere = false;
        widePane = false;
    } else {
        batch = false;
    }

    if(quit) {
        return 0;
    }

    ring_count = RING_COUNT;

    tabPage[0] = page1;
    int m = 0;
    row[0] = 0;

    for(int line = 1; line <= LABEL_COUNT; ++line, ++m) {
        if(settingStart <= line && line <= settingEnd) {
            if(settingStart == line) {
                m = 0;
            }

            row[line] = m;
            tabPage[line] = page7;
        } else if(ASCIIstart <= line && line <= ASCIIend) {
            if(ASCIIstart == line) {
                m = 0;
            }

            row[line] = m;
            tabPage[line] = page1;
        } else if(cyrillicStart <= line && line <= cyrillicEnd) {
            if(cyrillicStart == line) {
                m = 0;
            }

            row[line] = m;
            tabPage[line] = page2;
        } else if(greekStart <= line && line <= greekEnd) {
            if(greekStart == line) {
                m = 0;
            }

            row[line] = m;
            tabPage[line] = page3;
        } else if(kanjiStart <= line && line <= kanjiEnd) {
            if(kanjiStart == line) {
                m = 0;
            }

            row[line] = m;
            tabPage[line] = page4;
        } else if(mathematicalStart <= line && line <= mathematicalEnd) {
            if(mathematicalStart == line) {
                m = 0;
            }

            row[line] = m;
            tabPage[line] = page5;
        } else if(saltStart <= line && line <= saltEnd) {
            if(saltStart == line) {
                m = 0;
            }

            row[line] = m;
            tabPage[line] = page6;
        } else {
            row[line] = 0;
            tabPage[line] = page7;
        }

    }

// Log
   strcpy(vs->log_file_name, VOYA_LOG);

// Creating a window
    char windowName[] = "Menu Window";
    WNDCLASSEX wcl;

    wcl.cbSize = sizeof(WNDCLASSEX); 
    wcl.hInstance = instance;
    wcl.lpszClassName = windowName;
    wcl.lpfnWndProc = WindowFunc;
    wcl.style = 0;
    wcl.hIcon = ::LoadIcon(instance, "vybm_icon");
    wcl.hIconSm = NULL;
    wcl.hCursor = ::LoadCursor(NULL, IDC_ARROW);
    wcl.cbClsExtra = 0;
    wcl.cbWndExtra = 0;
    wcl.hbrBackground = (HBRUSH) ::GetStockObject(WHITE_BRUSH); 
    HWND hwnd;

    wcl.lpszMenuName = "vybmMenu";
    if(! ::RegisterClassEx(&wcl) ) {
        return 0;
    }

// width, height
    if(narrowPane) {
        main_width = NARROW_PANE;
    } else {
        if(widePane) {
            main_width = WIDE_PANE;
        } else {
            main_width = MAIN_WIDTH;
        }
    }

    if(shortPane) {
        main_height = SHORT_PANE;
    } else {
        main_height = MAIN_HEIGHT;
    }

    if(batch) {
        standard_main_x = 817;
        standard_main_y = 508;
    } else {
        standard_main_x = 276;
        standard_main_y = 30;
    }

    const int MENU_HEIGHT = ::GetSystemMetrics(SM_CYMENU);
    const int SCREEN_WIDTH = ::GetSystemMetrics(SM_CXFULLSCREEN);
    int screen_height = ::GetSystemMetrics(SM_CYFULLSCREEN);
    screen_height += MENU_HEIGHT;

    switch(corner) {
    case 1:
        main_x = 0;
        main_y = 0;
        break;
    case 2:
        main_x = SCREEN_WIDTH - main_width;
        main_y = 0;
        break;
    case 3:
        main_x = SCREEN_WIDTH - main_width;
        main_y = screen_height - main_height;
        break;
    case 4:
        main_x = 0;
        main_y = screen_height - main_height;
        break;
    default:
        main_x = SCREEN_WIDTH/2 - main_width/2;
        main_y = screen_height/2 - main_height/2;
        break;
    }

    hwnd = ::CreateWindow(
        windowName,
        "vybm", 
        WS_OVERLAPPEDWINDOW, 
        main_x, 
        main_y, 
        main_width, 
        main_height, 
        NULL, 
        NULL, 
        instance, 
        NULL
    );

    ::MoveWindow(hwnd, main_x, main_y, main_width, main_height, REPAINT);
    ::ShowWindow(hwnd, nWinMode);
    if(batch) {
        ::ShowWindow(hwnd, SW_HIDE);
    }

    RECT windowRect;
    ::GetClientRect(hwnd, &windowRect);

// Common controls: status bar and header
    INITCOMMONCONTROLSEX cc;
    cc.dwSize = sizeof(INITCOMMONCONTROLSEX);
    cc.dwICC = ICC_LISTVIEW_CLASSES;
    ::InitCommonControlsEx(&cc); 

// Tab control
    const int TAB_CONTROL_WIDTH = windowRect.right;
    TCITEM tabControlItem;
    tabControl = ::CreateWindow(
        WC_TABCONTROL,
        "",
        WS_VISIBLE | WS_TABSTOP | WS_CHILD,
        0,
        0,
        TAB_CONTROL_WIDTH,
        TAB_CONTROL_LABEL_HEIGHT,
        hwnd,
        (HMENU)IDM_TAB_CONTROL,
        instance,
        NULL
    );

    tabControlItem.mask = TCIF_TEXT;
    tabControlItem.iImage = -1;
    tabControlItem.pszText = tabName[0];
    TabCtrl_InsertItem(tabControl, 0, &tabControlItem);
    tabControlItem.pszText = tabName[1];
    TabCtrl_InsertItem(tabControl, 1, &tabControlItem);
    tabControlItem.pszText = tabName[2];
    TabCtrl_InsertItem(tabControl, 2, &tabControlItem);
    tabControlItem.pszText = tabName[3];
    TabCtrl_InsertItem(tabControl, 3, &tabControlItem);
    tabControlItem.pszText = tabName[4];
    TabCtrl_InsertItem(tabControl, 4, &tabControlItem);
    tabControlItem.pszText = tabName[5];
    TabCtrl_InsertItem(tabControl, 5, &tabControlItem);
    tabControlItem.pszText = tabName[6];
    TabCtrl_InsertItem(tabControl, 6, &tabControlItem);

// Selecting the Setting tab
    readVoyaCSV();
    TabCtrl_SetCurSel(tabControl, SETTING_TAB);
    tabSelected = SETTING_TAB; // page7
    if(kanjiDetails) {
        details = KANJI_TAB;
    } else {
        details = ASCII_TAB;
    }

    groupLine = 0;
    valueLine = 0;

// status bar
    statusBar = ::CreateStatusWindowA(
        WS_CHILD | WS_VISIBLE, 
        "",
        hwnd,
        IDM_STATUS_BAR
    );

    int panel[PANEL_COUNT];
    panel[0] = TIME_TEXT_NAME_WIDTH;
    panel[1] = panel[0] + MESSAGE_WIDTH;
    panel[2] = panel[1] + LINE_COUNT_WIDTH;
    ::SendMessage(statusBar, SB_SETPARTS, (WPARAM)PANEL_COUNT, (LPARAM)panel);
    summing(STATUS_BAR);

// header, which will be destroyed by narrowHeader() or standardHeader()
    header = ::CreateWindow(
        WC_HEADER,
        NULL,
        WS_CHILD | WS_BORDER,
        0,
        TAB_CONTROL_LABEL_HEIGHT,
        0,
        0,
        hwnd,
        (HMENU)IDM_HEADER,
        instance,
        NULL
    );

// Producing header labels
    const bool HEADER_ONLY = false;
    strcpy(group_field_name, "Word");
    strcpy(value_field_name, "Data");
    if(narrowPane) {
        narrowHeader(hwnd);
    } else {
        standardHeader(hwnd, HEADER_ONLY);
    }

// edit box, which wil be moved by buttonUpHandler()
    editBox = CreateWindowEx(
        WS_EX_CLIENTEDGE,
        "EDIT", 
        value[1].buffer,
        WS_CHILD | ES_LEFT,
        fieldWidth[0],
        TAB_CONTROL_LABEL_HEIGHT + headerHeight,
        value[1].width,
        value[1].height + INCLUDE_UNDERSCORE,
        hwnd,
        (HMENU)IDM_EDIT_BOX, 
        instance, 
        NULL
    );

    ::ShowWindow(editBox, SW_HIDE);

    info = EMPTY;
    HACCEL haccelerator = ::LoadAccelerators(instance, "vybmMenu");
    MSG msg;
    vs->instance = instance;
    while( ::GetMessage(&msg, NULL, 0, 0) ) {
        if( ::TranslateAccelerator(hwnd, haccelerator, &msg) ) {
            ;
        } else {
            ::TranslateMessage(&msg);
            ::DispatchMessage(&msg);
        }
    }

    return msg.wParam;
}

LRESULT CALLBACK WindowFunc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam) {
    using namespace std;

    char *p;
    char *edited_line;
    char *stop;
    const bool PERFECT = true;
    const bool READ_VOYA_CSV = true;
    double sum;
    HDC hdc;
    HDITEM *hdiptr;
    int button, line_length, line_spacing, upper_limit;
    NMHEADER *hdnptr;
    NMHDR *d;
    PAINTSTRUCT ps;
    RECT windowRect;
    SIZE size;
    TEXTMETRIC tm;

    switch(message) {
    case WM_COMMAND:
        switch( LOWORD(wParam) ) {
        case IDM_ASCII:
            selectTab(hwnd, page1);
            break;
        case IDM_BATCH:
            if( voyaLogBatchProcessing() ) {
                ;
            } else {
                ::SendMessage(statusBar, SB_SETTEXT, (WPARAM)1, (LPARAM)info.c_str() );
            }

            break;
        case IDM_CENTER:
            centering(hwnd);
            break;
        case IDM_CLEAN:
            createRemoveErrorText();
            break;
        case IDM_CORNER:
            cornerHandler(hwnd);
            break;
        case IDM_CYRILLIC:
            selectTab(hwnd, page2);
            break;
        case IDM_DETAILS:
            selectTab(hwnd, details);
            break;
        case IDM_DIAGONAL:
            diagonalHandler(hwnd);
            break;
        case IDM_EXIT:
            closing();
            break;
        case IDM_GREEK:
            selectTab(hwnd, page3);
            break;
        case IDM_HEIGHT:
            heightHandler(hwnd);
            break;
        case IDM_HELP:
            ::WinHelp( (HWND)wParam, HELP_FILE_NAME, HELP_CONTEXT, IDH_OVERVIEW);
            break;
        case IDM_INQUIRY:
            selectTab(hwnd, page7);
            shortPane = true;
            main_height = SHORT_PANE;
            narrowHeader(hwnd);
            break;
        case IDM_INSERT:
            valueLine = valueColumn(hwnd);
            if(valueLine) {
                if(valueLine == previousValueLine) {
                    ::GetWindowText(editBox, value[valueLine].buffer, SMALL_BUFFER_SIZE);
                    copyEdited(valueLine, value[valueLine].buffer);
                    writeLabelsEditedLines();
                }
            } else {
                groupLine = groupColumn(hwnd);
                if(groupLine) {
                    if(groupLine == previousGroupLine) {
                        ::GetWindowText(editBox, group[groupLine].buffer, SMALL_BUFFER_SIZE);
                        string LABEL(group[groupLine].buffer);
                        copyLabel(groupLine, LABEL);
                        writeLabelsEditedLines();
                    }
                }
            }

            ::ShowWindow(editBox, SW_HIDE);
            groupLine = 0;
            valueLine = 0;
            readVoyaCSV();
            ::InvalidateRect(hwnd, WHOLE_WINDOW, ERACE_BACKGROUND);
            break;
        case IDM_KANJI:
            selectTab(hwnd, page4);
            break;
        case IDM_MANAGE:
            manage = new VybmManage(vs->instance, hwnd);
            manage->setting(backUp, narrowPane, removeFirst, serialNumber, shortPane);
            button = manage->modalDialog();
            if(button == IDT_SAVE) {
                readManageText();
                backUp = vc->backUp;
                corner = vc->corner;
                kanjiDetails = vc->kanjiDetails;
                reserve = vc->reserve;
                narrowPane = vc->narrowPane;
                removeFirst = vc->removeFirst;
                serialNumber = vc->serialNumber;
                shortPane = vc->shortPane;
                showWhere = vc->showWhere;
                widePane = vc->widePane;
                if(vc->narrowPane) {
                    narrowHeader(hwnd);
                }

                if(vc->shortPane) {
                    main_height = SHORT_PANE;
                    ::MoveWindow(hwnd, main_x, main_y, main_width, main_height, REPAINT);
                }

            }

            delete manage;
            break;
        case IDM_MATHEMATICAL:
            selectTab(hwnd, page5);
            break;
        case IDM_MINIMIZE:
            minimizeHandler(hwnd);
            break;
        case IDM_NARROW:
            narrowHeader(hwnd);
            break;
        case IDM_NORMALIZE:
            writeLabelsEditedLines();
            if( readVoyaCSV() ){
                sum = shiftJisKanaAscii();
                normalizing(sum, AIM);
                writeLabelsEditedLines();
                summing(STATUS_BAR);
                ::InvalidateRect(hwnd, WHOLE_WINDOW, ERACE_BACKGROUND);
            } else {
                closing();
            }

            break;
        case IDM_OMIT:
            omit(hwnd);
            break;
        case IDM_RECOVERY:
            recovery(READ_VOYA_CSV);
            ::ShowWindow(editBox, SW_HIDE);
            groupLine = 0;
            valueLine = 0;
            ::InvalidateRect(hwnd, WHOLE_WINDOW, ERACE_BACKGROUND);
            break;
        case IDM_RIGHT:
            rightHandler(hwnd);
            break;
        case IDM_SALT:
            selectTab(hwnd, page6);
            break;
        case IDM_SETTING:
            selectTab(hwnd, page7);
            break;
        case IDM_STANDARD:
            standardHeader(hwnd, PERFECT);
            break;
        case IDM_TOUCH:
            touchVoyaCtrlText();
            break;
        case IDM_UNDER:
            underHandler(hwnd);
            break;
        case IDM_VERSION:
            version = new VybmVersion(vs->instance, hwnd);
            version->modalDialog();
            delete version;
            break;
        case IDM_WIDTH:
            widthHandler(hwnd);
            break;
        }

        break;
    case WM_CREATE: 
        if(batch) {
            ::SetTimer(hwnd, IDM_TIMER, PROMPTLY, (TIMERPROC)batchProcessing);
        } else {
            vs->non_sampling = NON_SAMPLING_PERIOD;
            ::SetTimer(hwnd, IDM_TIMER, vs->non_sampling, (TIMERPROC)TimerProc);
        }

        break;
    case WM_DESTROY:
        closing();
        break;
    case WM_LBUTTONDOWN:
        valueLine = valueColumn(hwnd);
        if(valueLine) {
            previousValueLine = valueLine;
            previousGroupLine = 0;
        } else {
            previousValueLine = 0;
            groupLine = groupColumn(hwnd);
            previousGroupLine = groupLine;
        }

        break;
    case WM_LBUTTONUP:
        valueLine = valueColumn(hwnd);
        if(valueLine) {
            if(valueLine == previousValueLine) {
                ::MoveWindow(editBox, value[valueLine].x - COLUMN_SPACING, value[valueLine].y, value[valueLine].width, value[valueLine].height + INCLUDE_UNDERSCORE, REPAINT);
                ::SetWindowText(editBox, value[valueLine].buffer);
                ::ShowWindow(editBox, SW_SHOW);
            }
        } else {
            groupLine = groupColumn(hwnd);
            if(groupLine) {
                if(groupLine == previousGroupLine) {
                    ::MoveWindow(editBox, group[groupLine].x - COLUMN_SPACING, group[groupLine].y, group[groupLine].width, group[groupLine].height + INCLUDE_UNDERSCORE, REPAINT);
                    ::SetWindowText(editBox, group[groupLine].buffer);
                    ::ShowWindow(editBox, SW_SHOW);
                }
            }
        }

        break;
    case WM_NOTIFY:
        if( LOWORD(wParam) == IDM_HEADER ) {
            hdnptr = (NMHEADER *)lParam;
            hdiptr = (HDITEM *)hdnptr->pitem;
            switch(hdnptr->hdr.code) {
            case HDN_ENDTRACK:
// item.mask = HDI_FORMAT | HDI_WIDTH | HDI_TEXT;
                if(hdiptr->cxy < LOWER_LIMIT) {
                    hdiptr->cxy = LOWER_LIMIT;              
                    fieldWidth[hdnptr->iItem] = LOWER_LIMIT;
                } else {
                    fieldWidth[hdnptr->iItem] = hdiptr->cxy;
                    ::InvalidateRect(hwnd, WHOLE_WINDOW, ERACE_BACKGROUND);
                }

                break;
            case HDN_TRACK:
                ::GetClientRect(hwnd, &windowRect);
                if(hdiptr->cxy < LOWER_LIMIT) {
                    hdiptr->cxy = LOWER_LIMIT;              
                    fieldWidth[hdnptr->iItem] = LOWER_LIMIT;
                } else {
                    fieldWidth[hdnptr->iItem] = hdiptr->cxy;
                    windowRect.top = headerHeight;
                    ::InvalidateRect(hwnd, WHOLE_WINDOW, ERACE_BACKGROUND);
                }

                break;
            }  // switch(hdnptr->hdr.code) 
        } // if( LOWORD(wParam) == IDM_HEADER ) 

        d = (NMHDR *)lParam;
        if(d->code == TCN_SELCHANGE) {
            tabSelected = TabCtrl_GetCurSel( (HWND)d->hwndFrom );
            if(tabSelected) {
                details = tabSelected;
            }

            groupLine = 0;
            valueLine = 0;
            ::ShowWindow(editBox, SW_HIDE);
            ::InvalidateRect(hwnd, WHOLE_WINDOW, ERACE_BACKGROUND);
        }

        switch(d->idFrom) {
        case IDM_STATUS_BAR:
            if(d->code == NM_CLICK) {
                omit(hwnd);
            }

            break;
        }

        break;
    case WM_PAINT:
        hdc = ::BeginPaint(hwnd, &ps);

        ::GetTextMetrics(hdc, &tm);
        line_spacing = tm.tmHeight + tm.tmInternalLeading;

        for(int line = 1; line <= LABEL_COUNT; ++line) {
            if(tabPage[line] == tabSelected) {
// Word
                p = & gv->buffer[0];
                strcpy(p, vs->label[line].c_str() );
                gv->length = lstrlen(p);

                ::GetTextExtentPoint32(hdc, gv->buffer, gv->length, &size);
                upper_limit = fieldWidth[0] - COLUMN_SPACING;
                line_length = fieldWidth[0]/tm.tmAveCharWidth - ELLIPSIS_SIZE_1;
                while(upper_limit < size.cx) {
                    line_length = checkKanji(gv->buffer, line_length);
                    p = & gv->buffer[line_length];
                    strcpy(p, ELLIPSIS);
                    p = & gv->buffer[0];
                    gv->length = lstrlen(p);
                    line_length -= 1;

                    ::GetTextExtentPoint32(hdc, gv->buffer, gv->length, &size);
                }

                gv->height = line_spacing;
                gv->width = fieldWidth[0];
                gv->x = COLUMN_SPACING;
                gv->y = TAB_CONTROL_LABEL_HEIGHT + headerHeight + row[line] * line_spacing;
                group[ row[line] + 1 ] = *gv;
                ::TextOut(hdc, gv->x, gv->y, gv->buffer, gv->length);

// Data
                edited_line = & vs->edited[line][0];
                p = & gv->buffer[0];
                strcpy(p, edited_line);
                gv->length = lstrlen(p);
                gv->w = strtod(p, &stop);

                ::GetTextExtentPoint32(hdc, gv->buffer, gv->length, &size);
                upper_limit = fieldWidth[1] - COLUMN_SPACING;
                line_length = fieldWidth[1]/tm.tmAveCharWidth - ELLIPSIS_SIZE_1;
                while(upper_limit < size.cx) {
                    line_length = checkKanji(gv->buffer, line_length);
                    p = & gv->buffer[line_length];
                    strcpy(p, ELLIPSIS);
                    p = & gv->buffer[0];
                    gv->length = lstrlen(p);
                    line_length -= 1;

                    ::GetTextExtentPoint32(hdc, gv->buffer, gv->length, &size);
                }

                gv->height = line_spacing;
                gv->width = fieldWidth[1];
                gv->x = fieldWidth[0] + COLUMN_SPACING;
                gv->y = TAB_CONTROL_LABEL_HEIGHT + headerHeight + row[line] * line_spacing;
                value[ row[line] + 1 ] = *gv;
                ::TextOut(hdc, gv->x, gv->y, gv->buffer, gv->length);
            }

        }

        ::EndPaint(hwnd, &ps);
        break;
    case WM_KEYDOWN:
        switch( (char)wParam ) {
        case VK_DOWN:
            if(valueLine) {
                valueLine += 1;
                if(valueLine < 1) {
                    valueLine = 1;
                } else if(valueLine > ROW_COUNT) {
                    valueLine = ROW_COUNT;
                }

                if( withinRange(valueLine) ) {
                    ::MoveWindow(editBox, value[valueLine].x - COLUMN_SPACING, value[valueLine].y, value[valueLine].width, value[valueLine].height + INCLUDE_UNDERSCORE, REPAINT);
                    ::SetWindowText(editBox, value[valueLine].buffer);
                    ::ShowWindow(editBox, SW_SHOW);
                } else {
                    valueLine = 1;
                }
            } else {
                if(groupLine) {
                    groupLine += 1;
                    if(groupLine < 1) {
                        groupLine = 1;
                    } else if(groupLine > ROW_COUNT) {
                        groupLine = ROW_COUNT;
                    }

                    if( withinRange(groupLine) ) {
                        ::MoveWindow(editBox, group[groupLine].x - COLUMN_SPACING, group[groupLine].y, group[groupLine].width, group[groupLine].height + INCLUDE_UNDERSCORE, REPAINT);
                        ::SetWindowText(editBox, group[groupLine].buffer);
                        ::ShowWindow(editBox, SW_SHOW);
                    } else {
                        groupLine = 1;
                    }
                }
            }

            break;
        case VK_HOME:
            groupLine = 1;
            valueLine = 0;
            ::MoveWindow(editBox, group[groupLine].x - COLUMN_SPACING, group[groupLine].y, group[groupLine].width, group[groupLine].height + INCLUDE_UNDERSCORE, REPAINT);
            ::SetWindowText(editBox, group[groupLine].buffer);
            ::ShowWindow(editBox, SW_SHOW);
            break;
        case VK_LEFT:
            if(valueLine) {
                if(valueLine < 1) {
                    valueLine = 1;
                } else if(valueLine > ROW_COUNT) {
                    valueLine = ROW_COUNT;
                }

                if( withinRange(valueLine) ) {
                    groupLine = valueLine;
                    valueLine = 0;
                    ::MoveWindow(editBox, group[groupLine].x - COLUMN_SPACING, group[groupLine].y, group[groupLine].width, group[groupLine].height + INCLUDE_UNDERSCORE, REPAINT);
                    ::SetWindowText(editBox, group[groupLine].buffer);
                    ::ShowWindow(editBox, SW_SHOW);
                } else {
                    valueLine = 1;
                }
            } else {
                if(groupLine) {
                    if(groupLine < 1) {
                        groupLine = 1;
                    } else if(groupLine > ROW_COUNT) {
                        groupLine = ROW_COUNT;
                    }

                }

            }

            break;
        case VK_RIGHT:
            if(valueLine) {
                if(valueLine < 1) {
                    valueLine = 1;
                } else if(valueLine > ROW_COUNT) {
                    valueLine = ROW_COUNT;
                }

            } else {
                if(groupLine) {
                    if(groupLine < 1) {
                        groupLine = 1;
                    } else if(groupLine > ROW_COUNT) {
                        groupLine = ROW_COUNT;
                    }

                    if( withinRange(groupLine) ) {
                        valueLine = groupLine;
                        groupLine = 0;
                        ::MoveWindow(editBox, value[valueLine].x - COLUMN_SPACING, value[valueLine].y, value[valueLine].width, value[valueLine].height + INCLUDE_UNDERSCORE, REPAINT);
                        ::SetWindowText(editBox, value[valueLine].buffer);
                        ::ShowWindow(editBox, SW_SHOW);
                    } else {
                        groupLine = 1;
                    }
                }
            }

            break;
        case VK_UP:
            if(valueLine) {
                valueLine -= 1;
                if(valueLine < 1) {
                    valueLine = 1;
                } else if(valueLine > ROW_COUNT) {
                    valueLine = ROW_COUNT;
                }

                if( withinRange(valueLine) ) {
                    ::MoveWindow(editBox, value[valueLine].x - COLUMN_SPACING, value[valueLine].y, value[valueLine].width, value[valueLine].height + INCLUDE_UNDERSCORE, REPAINT);
                    ::SetWindowText(editBox, value[valueLine].buffer);
                    ::ShowWindow(editBox, SW_SHOW);
                }
            } else {
                if(groupLine) {
                    groupLine -= 1;
                    if(groupLine < 1) {
                        groupLine = 1;
                    } else if(groupLine > ROW_COUNT) {
                        groupLine = ROW_COUNT;
                    }

                    if( withinRange(groupLine) ) {
                        ::MoveWindow(editBox, group[groupLine].x - COLUMN_SPACING, group[groupLine].y, group[groupLine].width, group[groupLine].height + INCLUDE_UNDERSCORE, REPAINT);
                        ::SetWindowText(editBox, group[groupLine].buffer);
                        ::ShowWindow(editBox, SW_SHOW);
                    }
                }
            }

            break;
        }

        break;
    case WM_SIZE:
        ::SendMessage(statusBar, WM_SIZE, wParam, lParam);
        layout(hwnd);
        break;
    default: 
        return ::DefWindowProc(hwnd, message, wParam, lParam);
    }

    return 0;
}

VOID CALLBACK batchProcessing(HWND hwnd, UINT uMsg, UINT idEvent, DWORD dwTime) {
    ::KillTimer(hwnd, IDM_TIMER);
    if( voyaLogBatchProcessing() ) {
        closing();
    } else {
        ::SendMessage(statusBar, SB_SETTEXT, (WPARAM)1, (LPARAM)info.c_str() );
        ::ShowWindow(hwnd, SW_SHOW);
    }

}

VOID CALLBACK TimerProc(HWND hwnd, UINT uMsg, UINT idEvent, DWORD dwTime) {
    movingInside(hwnd, showWhere);
}

void centering(HWND hwnd) {
    corner = 0;

    const int MENU_HEIGHT = ::GetSystemMetrics(SM_CYMENU);
    const int SCREEN_WIDTH = ::GetSystemMetrics(SM_CXFULLSCREEN);
    int screen_height = ::GetSystemMetrics(SM_CYFULLSCREEN);
    screen_height += MENU_HEIGHT;

    ::ShowWindow(hwnd, SW_RESTORE);
    main_x = SCREEN_WIDTH/2 - main_width/2;
    main_y = screen_height/2 - main_height/2;
    ::MoveWindow(hwnd, main_x, main_y, main_width, main_height, REPAINT);

    standard_main_x = main_x;
    standard_main_y = main_y;
}

void closing() {
    using namespace std;

    if( info.length() ) {
        ofstream file(VYBM_ERROR_TEXT);
        file << info << endl;
    }

    ::PostQuitMessage(0);
}

void cornerHandler(HWND hwnd) {
    corner = 1;

    ::ShowWindow(hwnd, SW_RESTORE);
    main_x = 0;
    main_y = 0;
    ::MoveWindow(hwnd, main_x, main_y, main_width, main_height, REPAINT);
}

void createManageText() {
    using namespace std;

    ofstream file(MANAGE_TEXT);
    file << "9" << endl << "7" << endl << "11" << endl << "10" << endl << "13" << endl << "12" << endl 
        << "20" << endl << "14" << endl << "30" << endl << "21" << endl << "34" << endl << "31" << endl 
        << "6" << endl << "1" << endl << "100" << endl << "0" << endl << "0" << endl << "0" << endl 
        << "1" << endl << "0" << endl << "0" << endl << "0" << endl << "0" << endl << "0" << endl 
        << "0" << endl;
}

void diagonalHandler(HWND hwnd) {
    corner = 3;

    const int MENU_HEIGHT = ::GetSystemMetrics(SM_CYMENU);
    const int SCREEN_WIDTH = ::GetSystemMetrics(SM_CXFULLSCREEN);
    int screen_height = ::GetSystemMetrics(SM_CYFULLSCREEN);
    screen_height += MENU_HEIGHT;

    ::ShowWindow(hwnd, SW_RESTORE);
    main_x = SCREEN_WIDTH - main_width;
    main_y = screen_height - main_height;
    ::MoveWindow(hwnd, main_x, main_y, main_width, main_height, REPAINT);
}

void editedLinePrint(int line, int a) {
    using namespace std;

    stringstream os;
    os << a;
    string s = os.str();
    char *edited_line = & vs->edited[line][0];
    strcpy(edited_line, s.c_str() );
}

void heightHandler(HWND hwnd) {
    const int MENU_HEIGHT = ::GetSystemMetrics(SM_CYMENU);
    const int SCREEN_WIDTH = ::GetSystemMetrics(SM_CXFULLSCREEN);
    int screen_height = ::GetSystemMetrics(SM_CYFULLSCREEN);
    RECT windowRect;

    screen_height += MENU_HEIGHT;
    ::ShowWindow(hwnd, SW_RESTORE);
    ::GetWindowRect(hwnd, &windowRect);
    main_x = windowRect.left;
    main_y = windowRect.top;

    if(shortPane) {
        shortPane = false;
        main_height = MAIN_HEIGHT;
        int y = screen_height - MAIN_HEIGHT;
        if(y < main_y) {
             main_y = y;
        }

    } else {
        shortPane = true;
        main_height = SHORT_PANE;
    }

    ::MoveWindow(hwnd, main_x, main_y, main_width, main_height, REPAINT);
}

void layout(HWND hwnd) {
    using namespace std;

    int height;
    RECT windowRect;
    ::GetClientRect(hwnd, &windowRect);

// tab control
    const int TAB_CONTROL_WIDTH = windowRect.right;
    ::MoveWindow(tabControl, 0, 0, TAB_CONTROL_WIDTH, TAB_CONTROL_LABEL_HEIGHT, REPAINT);

// line count ( threadNew() will produce a new ring buffer.)
    height = windowRect.bottom - STATUS_BAR_HEIGHT - TAB_CONTROL_LABEL_HEIGHT;
    ring_count = RING_COUNT * height/398;
    if(ring_count > MAXIMUM_RING_COUNT) {
        ring_count = MAXIMUM_RING_COUNT;
    }

    stringstream os;
    os << ring_count << " lines";
    const string s = os.str();
    ::SendMessage(statusBar, SB_SETTEXT, (WPARAM)2, (LPARAM)s.c_str() );

// header
    HDLAYOUT layout;
    WINDOWPOS windowPos;
    layout.prc = &windowRect;
    layout.pwpos = &windowPos;
    Header_Layout(header, &layout);
    ::MoveWindow(header, windowPos.x, windowPos.y + TAB_CONTROL_LABEL_HEIGHT, windowPos.cx, windowPos.cy, REPAINT);
}

void minimizeHandler(HWND hwnd) {
    WINDOWPLACEMENT wp;

    if( ::GetWindowPlacement(hwnd, &wp) ) {
        wp.showCmd = SW_MINIMIZE;
        ::SetWindowPlacement(hwnd, &wp);
    }
}

void narrowHeader(HWND hwnd) {
    narrowPane = true;

    widePane = false;
    main_width = NARROW_PANE;
    if(shortPane) {
        main_height = SHORT_PANE;
    } else {
        main_height = MAIN_HEIGHT;
    }

    ::ShowWindow(hwnd, SW_RESTORE);
    const int MENU_HEIGHT = ::GetSystemMetrics(SM_CYMENU);
    const int SCREEN_WIDTH = ::GetSystemMetrics(SM_CXFULLSCREEN);
    int screen_height = ::GetSystemMetrics(SM_CYFULLSCREEN);
    screen_height += MENU_HEIGHT;

    switch(corner) {
    case 1:
        main_x = 0;
        main_y = 0;
        break;
    case 2:
        main_x = SCREEN_WIDTH - main_width;
        main_y = 0;
        break;
    case 3:
        main_x = SCREEN_WIDTH - main_width;
        main_y = screen_height - main_height;
        break;
    case 4:
        main_x = 0;
        main_y = screen_height - main_height;
        break;
    default:
        main_x = SCREEN_WIDTH/2 - main_width/2;
        main_y = screen_height/2 - main_height/2;
        break;
    }

    ::MoveWindow(hwnd, main_x, main_y, main_width, main_height, REPAINT);
    RECT windowRect;
    ::GetClientRect(hwnd, &windowRect);

// header
    ::DestroyWindow(header);
    header = ::CreateWindow(
        WC_HEADER,
        NULL,
        WS_CHILD | WS_BORDER,
        0,
        TAB_CONTROL_LABEL_HEIGHT,
        0,
        0,
        hwnd,
        (HMENU)IDM_HEADER,
        vs->instance,
        NULL
    );

    HDLAYOUT layout;
    WINDOWPOS windowPos;
    layout.pwpos = &windowPos;
    layout.prc = &windowRect;
    Header_Layout(header, &layout);
    ::MoveWindow(header, windowPos.x, windowPos.y + TAB_CONTROL_LABEL_HEIGHT, windowPos.cx, windowPos.cy, NO_PAINTING);

    headerHeight = windowPos.cy;

    HDITEM item;
    item.mask = HDI_FORMAT | HDI_WIDTH | HDI_TEXT;

    item.pszText = &group_field_name[0];
    item.cchTextMax = GROUP_FIELD_NAME_SIZE;
    item.cxy = NARROW_GROUP_FIELD_WIDTH;
    fieldWidth[0] = NARROW_GROUP_FIELD_WIDTH;
    item.fmt = HDF_STRING | HDF_CENTER;
    Header_InsertItem(header, 0, &item);

    item.pszText = &value_field_name[0];
    item.cchTextMax = VALUE_FIELD_NAME_SIZE;
    item.cxy = NARROW_VALUE_FIELD_WIDTH;
    fieldWidth[1] = NARROW_VALUE_FIELD_WIDTH;
    item.fmt = HDF_STRING | HDF_CENTER;
    Header_InsertItem(header, 1, &item);

    ::ShowWindow(header, SW_SHOW);
    ::InvalidateRect(hwnd, WHOLE_WINDOW, ERACE_BACKGROUND);
    ::UpdateWindow(hwnd);
}

void normalizing(double sum, double aim) {
    double d, rate;
    double weight[WEIGHT_ARRAY_LENGTH];
    int a[WEIGHT_ARRAY_LENGTH];

    a[0] = vs->ascii_weight;
    a[1] = vs->capital_omega_weight;
    a[2] = vs->capital_ya_weight;
    a[3] = vs->capital_z_weight;
    a[4] = vs->cap_weight;
    a[5] = vs->circle_weight;
    a[6] = vs->cup_weight;
    a[7] = vs->double_integral_weight;
    a[8] = vs->exists_weight;
    a[9] = vs->hankaku_katakana_weight;
    a[10] = vs->heisei_weight;
    a[11] = vs->hiragana_weight;
    a[12] = vs->jis_level_one_kanji_weight;
    a[13] = vs->jis_level_two_kanji_weight;
    a[14] = vs->katakana_weight;
    a[15] = vs->keisen_weight;
    a[16] = vs->nine_weight;
    a[17] = vs->omega_weight;
    a[18] = vs->paragraph_weight;
    a[19] = vs->parallel_weight;
    a[20] = vs->roman_weight;
    a[21] = vs->square_meter_weight;
    a[22] = vs->ya_weight;
    a[23] = vs->z_weight;

    for(int n = 0; n < REPEATEDLY_NORMALIZING; ++n) {
        for(int i = 0; i < WEIGHT_ARRAY_LENGTH; ++i) {
            weight[i] = (double) a[i];
        }

        rate = aim/sum;
        for(int i = 0; i < WEIGHT_ARRAY_LENGTH; ++i) {
            d = rate * weight[i];
            if(1 < d) {
                a[i] = (int)d;
            } else if(EPSILON < d) {
                a[i] = 1;
            } else {
                a[i] = 0;
            }

        }

        vs->ascii_weight = a[0];
        vs->capital_omega_weight = a[1];
        vs->capital_ya_weight = a[2];
        vs->capital_z_weight = a[3];
        vs->cap_weight = a[4];
        vs->circle_weight = a[5];
        vs->cup_weight = a[6];
        vs->double_integral_weight = a[7];
        vs->exists_weight = a[8];
        vs->hankaku_katakana_weight = a[9];
        vs->heisei_weight = a[10];
        vs->hiragana_weight = a[11];
        vs->jis_level_one_kanji_weight = a[12];
        vs->jis_level_two_kanji_weight = a[13];
        vs->katakana_weight = a[14];
        vs->keisen_weight = a[15];
        vs->nine_weight = a[16];
        vs->omega_weight = a[17];
        vs->paragraph_weight = a[18];
        vs->parallel_weight = a[19];
        vs->roman_weight = a[20];
        vs->square_meter_weight = a[21];
        vs->ya_weight = a[22];
        vs->z_weight = a[23];
        sum = shiftJisKanaAscii();
    }

    editedLinePrint(7, a[0]);
    editedLinePrint(8, a[3]);
    editedLinePrint(9, a[23]);
    editedLinePrint(10, a[2]);
    editedLinePrint(11, a[22]);
    editedLinePrint(12, a[1]);
    editedLinePrint(13, a[17]);
    editedLinePrint(14, a[6]);
    editedLinePrint(15, a[9]);
    editedLinePrint(16, a[10]);
    editedLinePrint(17, a[11]);
    editedLinePrint(18, a[12]);
    editedLinePrint(19, a[13]);
    editedLinePrint(20, a[14]);
    editedLinePrint(21, a[4]);
    editedLinePrint(22, a[5]);
    editedLinePrint(23, a[7]);
    editedLinePrint(24, a[8]);
    editedLinePrint(25, a[15]);
    editedLinePrint(26, a[16]);
    editedLinePrint(27, a[18]);
    editedLinePrint(28, a[19]);
    editedLinePrint(29, a[20]);
    editedLinePrint(30, a[21]);
}

void omit(HWND hwnd) {
    int m;

    if(valueLine) {
        m = valueLine - 1;
    } else {
        if(groupLine) {
            m = groupLine - 1;
        } else {
            return;
        }
    }

    std::string CLEAR = WIDE_7;
    for(int line = 1; line <= LABEL_COUNT; ++line) {
        if(row[line] == m && tabPage[line] == tabSelected) {
            if(valueLine) {
                copyEdited(valueLine, " ");
                copyLabel(valueLine, CLEAR);
                writeLabelsEditedLines();
            } else {
                if(groupLine) {
                    copyLabel(groupLine, CLEAR);
                    copyEdited(groupLine, " ");
                    writeLabelsEditedLines();
                }
            }

        }

    }

    ::ShowWindow(editBox, SW_HIDE);
    groupLine = 0;
    valueLine = 0;
    readVoyaCSV();
    ::InvalidateRect(hwnd, WHOLE_WINDOW, ERACE_BACKGROUND);
}

void recovery(bool r) {
    using namespace std;

    ofstream file(VOYA_CSV);
    file << "Request,ase" << endl << "Log,voya.log" << endl << "Leftmost,0" << endl << "Case-sensitive,0" << endl << "Report,524287" << endl 
        << "Idling,400" << endl << "ASCII @,143" << endl << "Greek ,1" << endl << "Cyrillic A,1" << endl << "f,1" << endl 
        << ",1" << endl << ",1" << endl << ",1" << endl << ",1" << endl << ",1" << endl 
        << ",143" << endl << "~,1" << endl << ",1" << endl << ",1" << endl << ",1" << endl 
        << "A,1" << endl << ",1" << endl << "T,1" << endl << "Greek ,1" << endl << ",1" << endl 
        << ",1" << endl << "Roman Z,1" << endl << "Unit l,1" << endl << "Cyrillic y,1" << endl << ",1" << endl 
        << "salt," << endl << "," << endl << "," << endl << "," << endl;

    if(r) {
        readVoyaCSV();
    }

}

void rightHandler(HWND hwnd) {
    corner = 2;

    const int SCREEN_WIDTH = ::GetSystemMetrics(SM_CXFULLSCREEN);
    ::ShowWindow(hwnd, SW_RESTORE);
    main_x = SCREEN_WIDTH - main_width;
    main_y = 0;
    ::MoveWindow(hwnd, main_x, main_y, main_width, main_height, REPAINT);
}

void selectTab(HWND hwnd, int page) {
    TabCtrl_SetCurSel(tabControl, page);
    tabSelected = page;
    if(tabSelected) {
        details = tabSelected;
    }

    groupLine = 0;
    valueLine = 0;
    ::ShowWindow(editBox, SW_HIDE);
    ::InvalidateRect(hwnd, WHOLE_WINDOW, ERACE_BACKGROUND);
}

void standardHeader(HWND hwnd, bool perfect) {
    if(perfect) {
        corner = 0;
        main_x = standard_main_x;
        main_y = standard_main_y;
        narrowPane = false;
        shortPane = false;
        widePane = false;
    }

    if(widePane) {
        main_width = WIDE_PANE;
    } else {
        main_width = MAIN_WIDTH;
    }

    if(shortPane) {
        main_height = SHORT_PANE;
    } else {
        main_height = MAIN_HEIGHT;
    }

    ::ShowWindow(hwnd, SW_RESTORE);
    const int MENU_HEIGHT = ::GetSystemMetrics(SM_CYMENU);
    const int SCREEN_WIDTH = ::GetSystemMetrics(SM_CXFULLSCREEN);
    int screen_height = ::GetSystemMetrics(SM_CYFULLSCREEN);
    screen_height += MENU_HEIGHT;

    switch(corner) {
    case 1:
        main_x = 0;
        main_y = 0;
        break;
    case 2:
        main_x = SCREEN_WIDTH - main_width;
        main_y = 0;
        break;
    case 3:
        main_x = SCREEN_WIDTH - main_width;
        main_y = screen_height - main_height;
        break;
    case 4:
        main_x = 0;
        main_y = screen_height - main_height;
        break;
    default:
        main_x = SCREEN_WIDTH/2 - main_width/2;
        main_y = screen_height/2 - main_height/2;
        break;
    }

    ::MoveWindow(hwnd, main_x, main_y, main_width, main_height, REPAINT);
    RECT windowRect;
    ::GetClientRect(hwnd, &windowRect);

// header
    ::DestroyWindow(header);
    header = ::CreateWindow(
        WC_HEADER,
        NULL,
        WS_CHILD | WS_BORDER,
        0,
        TAB_CONTROL_LABEL_HEIGHT,
        0,
        0,
        hwnd,
        (HMENU)IDM_HEADER,
        vs->instance,
        NULL
    );

    HDLAYOUT layout;
    WINDOWPOS windowPos;
    layout.pwpos = &windowPos;
    layout.prc = &windowRect;
    Header_Layout(header, &layout);
    ::MoveWindow(header, windowPos.x, windowPos.y + TAB_CONTROL_LABEL_HEIGHT, windowPos.cx, windowPos.cy, NO_PAINTING);

    headerHeight = windowPos.cy;

    HDITEM item;
    item.mask = HDI_FORMAT | HDI_WIDTH | HDI_TEXT;

    item.pszText = &group_field_name[0];
    item.cchTextMax = GROUP_FIELD_NAME_SIZE;
    item.cxy = GROUP_FIELD_WIDTH;
    fieldWidth[0] = GROUP_FIELD_WIDTH;
    item.fmt = HDF_STRING | HDF_CENTER;
    Header_InsertItem(header, 0, &item);

    item.pszText = &value_field_name[0];
    item.cchTextMax = VALUE_FIELD_NAME_SIZE;
    item.cxy = VALUE_FIELD_WIDTH;
    fieldWidth[1] = VALUE_FIELD_WIDTH;
    item.fmt = HDF_STRING | HDF_CENTER;
    Header_InsertItem(header, 1, &item);

    ::ShowWindow(header, SW_SHOW);
    ::InvalidateRect(hwnd, WHOLE_WINDOW, ERACE_BACKGROUND);
    ::UpdateWindow(hwnd);

    if(perfect) {
        standard_main_x = main_x;
        standard_main_y = main_y;
    }

}

bool touchVoyaCtrlText() {
    using namespace std;

    ofstream text(VOYA_CTRL_TEXT, ios::app);
    if( text.fail() ) {
        ofstream file(VYBM_ERROR_TEXT);
        file << VOYA_CTRL_TEXT << endl;
        return false;
    }

    text << GAP;
    return true;
}

void underHandler(HWND hwnd) {
    corner = 4;

    const int MENU_HEIGHT = ::GetSystemMetrics(SM_CYMENU);
    int screen_height = ::GetSystemMetrics(SM_CYFULLSCREEN);
    screen_height += MENU_HEIGHT;

    ::ShowWindow(hwnd, SW_RESTORE);
    main_x = 0;
    main_y = screen_height - main_height;
    ::MoveWindow(hwnd, main_x, main_y, main_width, main_height, REPAINT);
}

void widthHandler(HWND hwnd) {
    const int SCREEN_WIDTH = ::GetSystemMetrics(SM_CXFULLSCREEN);
    RECT windowRect;

    ::ShowWindow(hwnd, SW_RESTORE);
    ::GetWindowRect(hwnd, &windowRect);
    main_x = windowRect.left;
    main_y = windowRect.top;

    if(widePane) {
        widePane = false;
        if(narrowPane) {
            main_width = NARROW_PANE;
        } else {
            main_width = MAIN_WIDTH;
        }

    } else {
        widePane = true;
        main_width = WIDE_PANE;
        int x = SCREEN_WIDTH - WIDE_PANE;
        if(x < main_x) {
             main_x = x;
        }

    }

    ::MoveWindow(hwnd, main_x, main_y, main_width, main_height, REPAINT);
}

void writeLabelsEditedLines() {
    using namespace std;

    char *edited_line;
    string *p = new string("");
    cells[0][0] = HEADER_WORD;
    cells[1][0] = HEADER_DATA;

// Copying to produce an array
    for(int line = 1; line <= LABEL_COUNT; ++line) {
        edited_line = & vs->edited[line][0];

        delete p;
        p = new string(edited_line);
        cells[1][line] = *p;
        cells[0][line] = vs->label[line];
    }

// Encoding and writing
    ofstream file(VYBM_CSV);
    for(int line = 1; line <= LABEL_COUNT; ++line) {
        for(int i = 0; i < CSV_COLUMN_COUNT; ++i) {
            if(cells[i][line] == COMMA) {
                file << COMMA_CODE;
            } else if(cells[i][line] == GAP) {
                file << GAP_CODE;
            } else if(cells[i][line] == SPACE) {
                file << SPACE_CODE;
            } else if(cells[i][line] == WIDE_2) {
                file << WIDE_2_CODE;
            } else if(cells[i][line] == WIDE_3) {
                file << WIDE_3_CODE;
            } else if(cells[i][line] == WIDE_4) {
                file << WIDE_4_CODE;
            } else if(cells[i][line] == WIDE_5) {
                file << WIDE_5_CODE;
            } else if(cells[i][line] == WIDE_6) {
                file << WIDE_6_CODE;
            } else if(cells[i][line] == WIDE_7) {
                file << WIDE_7_CODE;
                if(i == 0) {
                    cells[1][line] = GAP_CODE;
                }
            } else {
                file << cells[i][line];
            }

            switch(i) {
            case 0:
                file << COMMA;
                break;
            default:
                file << COMMA << endl;
                break;
            }
        }
    }

    vybm_voya_rearrange();
    delete p;
}

